// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "AnimGraphNode_NewPoseCalc.h"
#include "AnimationGraph.h"
#include "DetailCategoryBuilder.h"
#include "DetailLayoutBuilder.h"
#include "IDetailPropertyRow.h"
#include "PropertyHandle.h"


/////////////////////////////////////////////////////
// UAnimGraphNode_NewPoseCalc

#define LOCTEXT_NAMESPACE "AnimGraphNode_NewPoseCalc"

UAnimGraphNode_NewPoseCalc::UAnimGraphNode_NewPoseCalc(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
	
}

FLinearColor UAnimGraphNode_NewPoseCalc::GetNodeTitleColor() const
{
	return FLinearColor(0.41f, 0.25f, 0.24f);
}

FText UAnimGraphNode_NewPoseCalc::GetTooltipText() const
{
	return LOCTEXT("NewPoseCalcTooltip", "Calculate New Pose");
}

FText UAnimGraphNode_NewPoseCalc::GetNodeTitle(ENodeTitleType::Type TitleType) const
{
	return LOCTEXT("NewPoseCalc", "NewPoseCalc");
}

FString UAnimGraphNode_NewPoseCalc::GetNodeCategory() const
{
	return TEXT("Motion Capture");
}

void UAnimGraphNode_NewPoseCalc::PreloadRequiredAssets()
{
	UAnimBlueprint* AnimBP = GetAnimBlueprint();
	if (nullptr != AnimBP)
	{
		if (nullptr != AnimBP->TargetSkeleton)
		{
			Node.SetTargetSkeleton(AnimBP->TargetSkeleton);
			if (!bHasInit)
			{
				if (Node.InitBoneMap())
				{
					bHasInit = true;
				}
			}
		}
	}
}

void UAnimGraphNode_NewPoseCalc::CustomizeDetails(IDetailLayoutBuilder& DetailBuilder)
{
	const FString BoneMapPropName = FString::Printf(TEXT("Node.%s"), GET_MEMBER_NAME_STRING_CHECKED(FAnimationNode_NewPoseCalc, BoneMap));	

	IDetailCategoryBuilder& SkeletonsRetargetingCategory = DetailBuilder.EditCategory("SkeletonsRetargeting");
	TSharedPtr<IPropertyHandle> PropertyHandle = DetailBuilder.GetProperty(*BoneMapPropName, GetClass());
	SkeletonsRetargetingCategory.AddProperty(PropertyHandle);

	DetailBuilder.HideCategory("Settings");
}

void UAnimGraphNode_NewPoseCalc::PostEditChangeChainProperty(struct FPropertyChangedChainEvent& PropertyChangedEvent)
{
	const FName PropertyName = (PropertyChangedEvent.Property != nullptr) ? PropertyChangedEvent.Property->GetFName() : NAME_None;

	if (EPropertyChangeType::ValueSet == PropertyChangedEvent.ChangeType)
	{
		if (PropertyName == GET_MEMBER_NAME_CHECKED (FAnimationNode_NewPoseCalc, SkeletonNamePrefix))
		{
			Node.InitBoneMap ();
			goto SuperChange;
		}

		if (PropertyName == GET_MEMBER_NAME_CHECKED(FAnimationNode_NewPoseCalc, BoneMap))
		{
			Node.Revert_BoneMap();
			goto SuperChange;
		}

		int32 ChangeAtIndex = PropertyChangedEvent.GetArrayIndex("BoneMap");
		check(INDEX_NONE != ChangeAtIndex);

		if (PropertyName == GET_MEMBER_NAME_CHECKED(FBoneMappingInfo, CTName))
		{
			Node.ValueSet_Name(ChangeAtIndex);
		}
		else if (PropertyName == GET_MEMBER_NAME_CHECKED(FBoneMappingInfo, PNID))
		{
			Node.ValueSet_ID(ChangeAtIndex);
		}
	}
	else if (EPropertyChangeType::ArrayAdd == PropertyChangedEvent.ChangeType)
	{
		if (PropertyName == GET_MEMBER_NAME_CHECKED(FAnimationNode_NewPoseCalc, BoneMap))
		{
			int32 AddAtIndex = PropertyChangedEvent.GetArrayIndex("BoneMap");
			Node.Add_BoneMap(AddAtIndex);
		}
	}
	else if (EPropertyChangeType::Unspecified == PropertyChangedEvent.ChangeType)
	{
		if (PropertyName == GET_MEMBER_NAME_CHECKED(FAnimationNode_NewPoseCalc, BoneMap))
		{
			Node.Del_BoneMap();
		}
	}

SuperChange:
	Super::PostEditChangeChainProperty(PropertyChangedEvent);
}

#undef LOCTEXT_NAMESPACE