// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "PropertyEditing.h"



/**
 * Implements a details view customization for the BVH asset.
 */
class FBVHAssetDetailsCustomization
	: public IDetailCustomization
{
public:

	// IDetailCustomization interface
	virtual void CustomizeDetails(IDetailLayoutBuilder& DetailBuilder) override;

public:

	/**
	 * Creates an instance of this class.
	 *
	 * @return The new instance.
	 */
	static TSharedRef<IDetailCustomization> MakeInstance()
	{
		return MakeShareable(new FBVHAssetDetailsCustomization());
	}

private:
	/** Callback for getting the selected path in the BVH file picker widget. */
	FString HandleBVHFilePickerFilePath() const;
 
 	/** Callback for getting the file type filter for the BVH file picker. */
 	FString HandleBVHFilePickerFileTypeFilter() const;
 
 	/** Callback for picking a path in the BVH file picker. */
 	void HandleBVHFilePickerPathPicked( const FString& PickedPath );
 
	/** Callback for getting the visibility of warning icon for invalid file path. */
	EVisibility HandleFilePathWarningIconVisibility() const;

private:

	/** The collection of UMediaPlayer assets being customized. */
	TArray<TWeakObjectPtr<UObject>> CustomizedBVHAssets;

	/** Pointer to the FilePath property handle. */
	TSharedPtr<IPropertyHandle> FilePathProperty;
};
