// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "BVHAssetFactory.h"
#include "BVHAsset.h"
#include "BVHAssetStyle.h"
#include "DesktopPlatformModule.h"
#include "Runtime/Slate/Public/Framework/Application/SlateApplication.h"


UBVHAssetFactory::UBVHAssetFactory(const FObjectInitializer& ObjectInitializer)
: Super(ObjectInitializer)
{
	bCreateNew = true;
	bEditAfterNew = true;
	bEditorImport = true;
	SupportedClass = UBVHAsset::StaticClass();

	Formats.Add(TEXT("bvh;BVH file"));

	BrowseTitle = FText::FromString(TEXT("Import a BVH file"));
	CurrentFilename = TEXT("");
#if (ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION < 18)
	BrowseDirectory = FPaths::GameContentDir() / TEXT("BVH");
#else
	BrowseDirectory = FPaths::ProjectContentDir() / TEXT("BVH");
#endif
}

bool UBVHAssetFactory::ConfigureProperties()
{
	return true;
}

FName UBVHAssetFactory::GetNewAssetThumbnailOverride() const
{
	return TEXT("ClassThumbnail.BVHAsset");;
}

UObject* UBVHAssetFactory::FactoryCreateNew(UClass* Class, UObject* InParent, FName Name, EObjectFlags Flags, UObject* Context, FFeedbackContext* Warn)
{
	check(Class->IsChildOf(UBVHAsset::StaticClass()));

	UBVHAsset* NewBVHAsset = static_cast<UBVHAsset*>(NewObject<UBVHAsset>(InParent, Class, Name, Flags));
	if (NewBVHAsset)
	{
		CurrentFilename = GetCurrentFilename();

		if (FString(TEXT("")) == CurrentFilename)
		{
			IDesktopPlatform* DesktopPlatform = FDesktopPlatformModule::Get();

			if (DesktopPlatform == nullptr)
			{
				return nullptr;
			}

			const FString DefaultPath = BrowseDirectory;


			// show the BVH file browse dialog
			TSharedPtr<SWindow> ParentWindow = FSlateApplication::Get().GetActiveTopLevelWindow();
			void* ParentWindowHandle = (ParentWindow.IsValid() && ParentWindow->GetNativeWindow().IsValid()) ? ParentWindow->GetNativeWindow()->GetOSWindowHandle() : nullptr;

			TArray<FString> OutFiles;

			FString FileTypeFilter = TEXT("All files (*.*)|*.*");
			FileTypeFilter = FileTypeFilter + TEXT("|BVH File (*.bvh)|*.bvh");

			if (DesktopPlatform->OpenFileDialog(ParentWindowHandle, BrowseTitle.ToString(),
				DefaultPath, TEXT(""), FileTypeFilter, EFileDialogFlags::None, OutFiles))
			{
				CurrentFilename = OutFiles[0];
			}
		}

		if (FString(TEXT("")) != CurrentFilename)
		{
			FPaths::NormalizeFilename(CurrentFilename);
			FPaths::MakePathRelativeTo(CurrentFilename, *BrowseDirectory);
			NewBVHAsset->BVHFilePath = CurrentFilename;
			NewBVHAsset->Load();
		}
	}

	return NewBVHAsset;
}

UObject* UBVHAssetFactory::FactoryCreateBinary
(
UClass*				Class,
UObject*			InParent,
FName				Name,
EObjectFlags		Flags,
UObject*			Context,
const TCHAR*		Type,
const uint8*&		Buffer,
const uint8*		BufferEnd,
FFeedbackContext*	Warn,
bool&				bOutOperationCanceled
)
{
	UObject* NewObject = NULL;

	return NewObject;
}

bool UBVHAssetFactory::FactoryCanImport(const FString& Filename)
{
	const FString Extension = FPaths::GetExtension(Filename);

	if (Extension == TEXT("bvh"))
	{
		CurrentFilename = Filename;
		return true;
	}
	return false;
}
