// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "BVHAssetStyle.h"
#include "Styling/SlateStyle.h"
#include "Styling/SlateStyleRegistry.h"
#include "Interfaces/IPluginManager.h"
#include "ClassIconFinder.h"


#define IMAGE_PLUGIN_BRUSH( RelativePath, ... ) FSlateImageBrush( FBVHAssetStyle::InContent( RelativePath, ".png" ), __VA_ARGS__ )


FString FBVHAssetStyle::InContent(const FString& RelativePath, const ANSICHAR* Extension)
{
	static FString ContentDir = IPluginManager::Get().FindPlugin(TEXT("MotionCapture"))->GetContentDir();
	return (ContentDir / RelativePath) + Extension;
}

TSharedPtr< FSlateStyleSet > FBVHAssetStyle::StyleSet = nullptr;
TSharedPtr< class ISlateStyle > FBVHAssetStyle::Get() { return StyleSet; }

FName FBVHAssetStyle::GetStyleSetName()
{
	static FName BVHAssetStyleName(TEXT("BVHAssetStyle"));
	return BVHAssetStyleName;
}

void FBVHAssetStyle::Initialize()
{
	// Const icon sizes
	const FVector2D Icon16x16(16.0f, 16.0f);
	const FVector2D Icon64x64(64.0f, 64.0f);

	// Only register once
	if (StyleSet.IsValid())
	{
		return;
	}

	StyleSet = MakeShareable(new FSlateStyleSet(GetStyleSetName()));

	StyleSet->Set("ClassIcon.BVHAsset", new IMAGE_PLUGIN_BRUSH(TEXT("Icons/BVHAsset_16x"), Icon16x16));
	StyleSet->Set("ClassThumbnail.BVHAsset", new IMAGE_PLUGIN_BRUSH(TEXT("Icons/BVHAsset_64x"), Icon64x64));

	FSlateStyleRegistry::RegisterSlateStyle(*StyleSet.Get());
};


#undef IMAGE_PLUGIN_BRUSH


void FBVHAssetStyle::Shutdown()
{
	if (StyleSet.IsValid())
	{
		FSlateStyleRegistry::UnRegisterSlateStyle(*StyleSet.Get());
		ensure(StyleSet.IsUnique());
		StyleSet.Reset();
	}
}
