// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "BoneMappingInfoCustomization.h"
#include "PropertyHandle.h"
#include "DetailWidgetRow.h"
#include "BoneMappingInfo.h"
#include "CTBoneSelectionWidget.h"
#include "AnimGraphNode_NewPoseCalc.h"
#include "Animation/AnimBlueprint.h"
#include "SlateOptMacros.h"

#define LOCTEXT_NAMESPACE "BoneMapingInfoCustomization"


TSharedRef<IPropertyTypeCustomization> FBoneMapingInfoCustomization::MakeInstance()
{
	return MakeShareable(new FBoneMapingInfoCustomization());
}

BEGIN_SLATE_FUNCTION_BUILD_OPTIMIZATION
void FBoneMapingInfoCustomization::CustomizeHeader(TSharedRef<IPropertyHandle> StructPropertyHandle, class FDetailWidgetRow& HeaderRow, IPropertyTypeCustomizationUtils& StructCustomizationUtils)
{
	BoneIDProperty = StructPropertyHandle->GetChildHandle(GET_MEMBER_NAME_CHECKED(FBoneMappingInfo, PNID));
	BoneNameProperty = StructPropertyHandle->GetChildHandle(GET_MEMBER_NAME_CHECKED(FBoneMappingInfo, CTName));

	check(BoneIDProperty->IsValidHandle());
	check(BoneNameProperty->IsValidHandle());

	TArray<UObject*> Objects;
	StructPropertyHandle->GetOuterObjects(Objects);
	
	USkeleton* TargetSkeleton = NULL;
	for (auto OuterIter = Objects.CreateIterator(); OuterIter; ++OuterIter)
	{
		UAnimBlueprint* AnimBP = nullptr;

		UAnimGraphNode_NewPoseCalc* AnimGraphNode = Cast<UAnimGraphNode_NewPoseCalc>(*OuterIter);
		if (nullptr == AnimGraphNode)
		{
			continue;
		}
		else
		{
			AnimBP = AnimGraphNode->GetAnimBlueprint ();
		}

		if (nullptr == AnimBP)
		{
			continue;
		}

		TargetSkeleton = AnimBP->TargetSkeleton;
		break;
	}

	if (TargetSkeleton)
	{
		FText FinalTooltip = FText(LOCTEXT("BonePDIDToolTip", "Click to choose a different PN bone"));
		BoneIDProperty->SetToolTipText(FinalTooltip);

		HeaderRow.NameContent()
		[
			StructPropertyHandle->CreatePropertyNameWidget()
		];

		HeaderRow.ValueContent()
		.MinDesiredWidth(300)
		.MaxDesiredWidth(300)
		[
			SNew(SHorizontalBox)
			.IsEnabled(true)
			+SHorizontalBox::Slot()
			.HAlign(HAlign_Fill)
			[
				BoneIDProperty->CreatePropertyValueWidget()
			]
			+SHorizontalBox::Slot()
			.HAlign(HAlign_Fill)
			[
				SNew(SCTBoneSelectionWidget)
				.Skeleton(TargetSkeleton)
				.Tooltip(StructPropertyHandle->GetToolTipText())
				.OnBoneSelectionChanged(this, &FBoneMapingInfoCustomization::OnBoneSelectionChanged)
				.OnGetSelectedBone(this, &FBoneMapingInfoCustomization::GetSelectedBone)
			]
		];
	}
}
END_SLATE_FUNCTION_BUILD_OPTIMIZATION

void FBoneMapingInfoCustomization::CustomizeChildren(TSharedRef<IPropertyHandle> StructPropertyHandle, class IDetailChildrenBuilder& ChildBuilder, IPropertyTypeCustomizationUtils& StructCustomizationUtils)
{
	// No child customizations as the properties are shown in the header
}

void FBoneMapingInfoCustomization::OnBoneSelectionChanged(FName Name)
{
	BoneNameProperty->SetValue(Name);
}

FName FBoneMapingInfoCustomization::GetSelectedBone() const
{
	FString OutText;

	BoneNameProperty->GetValueAsFormattedString(OutText);

	return FName(*OutText);
}

#undef LOCTEXT_NAMESPACE