// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "UnrealEd.h"
#include "AssetTypeActions_BVH.h"
#include "BVHAsset.h"
#include "BVHAssetStyle.h"
#include "BVHAssetDetailsCustomization.h"
#include "BoneMappingInfoCustomization.h"


class FMotionCaptureEditor : public IModuleInterface
{
	/** IModuleInterface implementation */
	virtual void StartupModule() override
	{
   		// This code will execute after your module is loaded into memory (but after global variables are initialized, of course.)
   
		// Register slate style overrides
		FBVHAssetStyle::Initialize();

   		// Register asset types
   		IAssetTools& AssetTools = FModuleManager::LoadModuleChecked<FAssetToolsModule>("AssetTools").Get();
   
		RegisterAssetTypeAction(AssetTools, MakeShareable(new FAssetTypeActions_BVH(EAssetTypeCategories::Misc)));

		// Register the details customizations
		{
			FPropertyEditorModule& PropertyModule = FModuleManager::LoadModuleChecked<FPropertyEditorModule>("PropertyEditor");
			PropertyModule.RegisterCustomClassLayout(UBVHAsset::StaticClass()->GetFName(), FOnGetDetailCustomizationInstance::CreateStatic(&FBVHAssetDetailsCustomization::MakeInstance));

			PropertyModule.RegisterCustomPropertyTypeLayout("BoneMappingInfo", FOnGetPropertyTypeCustomizationInstance::CreateStatic(&FBoneMapingInfoCustomization::MakeInstance));

			PropertyModule.NotifyCustomizationModuleChanged();
		}
	}
  
	virtual void ShutdownModule() override
	{
  		// This function may be called during shutdown to clean up your module. For modules that support dynamic reloading,
  		// we call this function before unloading the module.
  
  		// Unregister all the asset types that we registered
  		if (FModuleManager::Get().IsModuleLoaded("AssetTools"))
  		{
  			IAssetTools& AssetTools = FModuleManager::GetModuleChecked<FAssetToolsModule>("AssetTools").Get();
  
  			for (int32 Index = 0; Index < CreatedAssetTypeActions.Num(); ++Index)
  			{
  				AssetTools.UnregisterAssetTypeActions(CreatedAssetTypeActions[Index].ToSharedRef());
  			}
  		}

		CreatedAssetTypeActions.Empty();

		// Unregister slate style overrides
		FBVHAssetStyle::Shutdown();

		// unregsiter when shut down
		if (FModuleManager::Get().IsModuleLoaded("PropertyEditor"))
		{
			FPropertyEditorModule& PropertyModule = FModuleManager::LoadModuleChecked<FPropertyEditorModule>("PropertyEditor");
			PropertyModule.UnregisterCustomClassLayout(UBVHAsset::StaticClass()->GetFName());

			PropertyModule.UnregisterCustomPropertyTypeLayout("BoneMappingInfo");

			PropertyModule.NotifyCustomizationModuleChanged();
		}
	}
  
	/** All created asset type actions.  Cached here so that we can unregister them during shutdown. */
	TArray< TSharedPtr<IAssetTypeActions> > CreatedAssetTypeActions;
  
	void RegisterAssetTypeAction(IAssetTools& AssetTools, TSharedRef<IAssetTypeActions> Action)
	{
   		AssetTools.RegisterAssetTypeActions(Action);
   		CreatedAssetTypeActions.Add(Action);
	}
};
  
IMPLEMENT_MODULE(FMotionCaptureEditor, MotionCaptureEditor)
