// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "AnimNode_NewPoseCalc.h"
#include "EventManager.h"
#include "AnimationDataManager.h"
#include "MotionCaptureAvatarComponent.h"
#include "PoseContainer.h"
#include "BVHAsset.h"
#include "BoneMappingInfo.h"
#include "MotionCaptureHelper.h"
#include "HI5Glove.h"
#include "GlobalDefine.h"
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 11)
#include "Runtime/Engine/Public/Animation/AnimInstanceProxy.h"
#endif

#if USE_ALICE
#include "IHybridData.h"
#endif

using namespace Hi5;

/////////////////////////////////////////////////////
// FAnimationNode_NewPoseCalc

FAnimationNode_NewPoseCalc::FAnimationNode_NewPoseCalc()
{
}

#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION < 17)
void FAnimationNode_NewPoseCalc::Initialize(const FAnimationInitializeContext& Context)
#else
void FAnimationNode_NewPoseCalc::Initialize_AnyThread (const FAnimationInitializeContext& Context)
#endif
{

}

#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION < 17)
void FAnimationNode_NewPoseCalc::CacheBones(const FAnimationCacheBonesContext& Context) 
#else
void FAnimationNode_NewPoseCalc::CacheBones_AnyThread (const FAnimationCacheBonesContext& Context)
#endif
{
	
}

#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION < 17)
void FAnimationNode_NewPoseCalc::Update(const FAnimationUpdateContext& Context)
#else
void FAnimationNode_NewPoseCalc::Update_AnyThread (const FAnimationUpdateContext& Context)
#endif
{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION < 22)
	EvaluateGraphExposedInputs.Execute( Context );
#else
	GetEvaluateGraphExposedInputs( ).Execute( Context );
#endif

	BVHFramePointer += Context.GetDeltaTime();
}

#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION < 17)
void FAnimationNode_NewPoseCalc::Evaluate(FPoseContext& Output)
#else
void FAnimationNode_NewPoseCalc::Evaluate_AnyThread (FPoseContext& Output)
#endif
{
	USkeletalMeshComponent* SkeleMeshComp = nullptr;
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 11)
	SkeleMeshComp =	Output.AnimInstanceProxy->GetSkelMeshComponent();
#else
	SkeleMeshComp = Output.AnimInstance->GetSkelMeshComponent();
#endif
	if (nullptr == SkeleMeshComp)
	{
		return;
	}
	
	{
		Output.ResetToRefPose();
	}

	AActor* OwnerActor = SkeleMeshComp->GetOwner();
	if (nullptr == OwnerActor)
	{
		return;
	}

	UMotionCaptureAvatarComponent* PNComp = Cast<UMotionCaptureAvatarComponent> (SkeleMeshComp->GetAttachParent ());

	if (nullptr == PNComp)
	{
		//PN_LOG(Warning, TEXT("The animation Instance`s owner is not a UMotionCaptureComponent Instance."));
		return;
	}

	if (0 == BoneMap.Num())
	{
		PN_LOG(Warning, TEXT("Bone map is empty."));
		return;
	}

	EHandMode curConfigHandMode = EHandMode::ModeUnknown;

#ifdef TRANCE
	curConfigHandMode = PNComp->HandMode;

	int32 HandModeToGet = -1;
	if (curConfigHandMode == EHandMode::LeftHand)
	{
		HandModeToGet = 1;
	}
	else if (curConfigHandMode == EHandMode::RightHand)
	{
		HandModeToGet = 2;
	}
	else
	{
		return;
	}

#endif // TRANCE

	FAnimationData AnimData;
	bool bFound = false;
	UBVHAsset* BVHAsset = PNComp->GetBVHAsset();
	if (nullptr != BVHAsset)
	{
		bFound = FAnimationDataManager::GetInstance()->GetAnimationData(BVHAsset, BVHFramePointer, AnimData);
	}
	else
	{
		bFound = FAnimationDataManager::GetInstance()->GetHandAnimationData(
#if WITH_AVATARID
			PNComp->GetAvatarID()
#else
			PNComp->GetAvatarName()
#endif
			, AnimData
			, HandModeToGet
			);
	}

	if (!bFound)
	{
#if WITH_AVATARID
		PN_LOG(Warning, TEXT("Can't find the actor's animation data. BVH file path:%s, Avatar ID:%d."), 
			(nullptr != BVHAsset) ? *(BVHAsset->GetBVHFilePath()) : TEXT(""), PNComp->GetAvatarID());
#else
		//PN_LOG(Warning, TEXT("Can't find the actor's animation data. BVH file path:%s, Avatar Name:%s."),
		//	(nullptr != BVHAsset) ? *(BVHAsset->GetBVHFilePath()) : TEXT(""), *(PNComp->GetAvatarName().ToString()));
#endif
		return;
	}

	if (AnimData.bUseRootEffect)
	{
		FVector CurActorPos = OwnerActor->GetActorLocation();
		FQuat CurActorQua = OwnerActor->GetActorQuat();
		if (!CurActorPos.Equals(AnimData.RootPosition) || !CurActorQua.Equals(AnimData.RootRotation))
		{
			// Move actor to destination
			FEventManager::GetInstance()->EnqueueEvent(MakeShareable(new FMoveToEvent(PNComp, AnimData.RootPosition, AnimData.RootRotation)));
		}
	}

	auto BoneNameComp = [&](const FName& LName, const FName& RName) -> bool{
		if (SkeleMeshComp && SkeleMeshComp->BoneIsChildOf(LName, RName))
		{
			return true;
		}
		return false;
	};

	{
		FPoseContainer Container(Output);
		int32 BoneCount = Container.GetNumBones();
		EMCBones::Type HandType;
		_GloveMod curDataMode = GM_Unknown;
		
		curDataMode = (_GloveMod)AnimData.GloveMode;//NetworkManager::GetInstance()->GetGloveMode();

		if ((curDataMode == GM_RightGlove && curConfigHandMode != EHandMode::RightHand) ||
			(curDataMode == GM_LeftGlove && curConfigHandMode != EHandMode::LeftHand))
		{
			return;
		}

		if (curDataMode == GM_Unknown)
		{
			PN_LOG (Error, TEXT ("Unknown glove mode in animation data."));
			return;
		}
		else if (curDataMode == GM_RightGlove)
		{
			HandType = EMCBones::RightHand;

			if (!BoneID2NameMap.Contains (HandType))
			{
				PN_LOG (Error, TEXT ("Can`t find the bone mapped with RightHand."));
				return;
			}
		}
		else if (curDataMode == GM_LeftGlove)
		{
			HandType = EMCBones::LeftHand;

			if (!BoneID2NameMap.Contains (HandType))
			{
				PN_LOG (Error, TEXT ("Can`t find the bone mapped with LeftHand."));
				return;
			}
		}
		else
		{
			PN_LOG (Error, TEXT ("Can`t support glove mode."));
			return;
		}

		TArray<FName> HandsNames;
		BoneID2NameMap.MultiFind (HandType, HandsNames);
		HandsNames.Sort (BoneNameComp);
		FName HandsName = HandsNames.Last ();
		int32 HandBoneIndex = SkeleMeshComp->GetBoneIndex (HandsName);
		if ((INDEX_NONE == HandBoneIndex) || (HandBoneIndex > BoneCount))
		{
			PN_LOG (Error, TEXT ("Get bone %s`s index failed, %s is not exist."), *(HandsName.ToString ()));
			return;
		}

		if (!bHaveInit)
		{
			WorldRotations.Init(FQuat::Identity, BoneCount);
			WorldPositions.Init(FVector::ZeroVector, BoneCount);
			for (int32 Index = 0; Index < BoneCount; Index++)
			{
				FQuat Rotation = Container[Index].GetRotation();
				FVector Position = Container[Index].GetLocation();

				int32 ParentBoneIndex = Container.GetParentBoneIndex(Index);
				if ((INDEX_NONE != ParentBoneIndex) && (ParentBoneIndex < BoneCount))
				{
					Rotation = WorldRotations[ParentBoneIndex] * Rotation;
					Position = WorldRotations[ParentBoneIndex] * Position + WorldPositions[ParentBoneIndex];
				}
				
				WorldRotations[Index] = Rotation;
				WorldPositions[Index] = Position;
			}
			bHaveInit = true;
		}

		// Preprocess the TopMost Bone and Hand data
		FVector RealtimeHandWorldPos = AnimData.BonePositions[HandType];
		FQuat RealtimeHandWorldQuat = AnimData.BoneRotations[HandType];

		EGlovePositionSource ePosSrc = UMotionCaptureFunctionLibrary::GetGlovePositionSource ();
		if (ePosSrc == EGlovePositionSource::VivePosition)
		{
			// apply glove offset relative to optical device
			FVector lOffset, rOffset;
			//??  
			hi5_getHandOffsetToOpticalDeviceInUE (lOffset, rOffset);
			{
				if (curDataMode == GM_LeftGlove)
				{
					FVector LeftPos;
					FRotator LeftRot;
					if (UMotionCaptureFunctionLibrary::GetBindedOpticalDeviceDataInUESpace (EGloveMod::GlM_LeftGlove, LeftPos, LeftRot))
					{
						RealtimeHandWorldPos = LeftPos + LeftRot.RotateVector (lOffset);
					}
				}
				else if (curDataMode == GM_RightGlove)
				{
					FVector RightPos;
					FRotator RightRot;
					if (UMotionCaptureFunctionLibrary::GetBindedOpticalDeviceDataInUESpace (EGloveMod::GlM_RightGlove, RightPos, RightRot))
					{
						RealtimeHandWorldPos = RightPos + RightRot.RotateVector (rOffset);
					}
				}
			}

			if (PNComp->LocationMode == ELocationMode::World)
			{
				PNComp->SetWorldLocation (RealtimeHandWorldPos);
				PNComp->SetWorldRotation (RealtimeHandWorldQuat);
			}
			else {
				PNComp->SetRelativeLocationAndRotation (RealtimeHandWorldPos, RealtimeHandWorldQuat);
			}
		}
		else if (ePosSrc == EGlovePositionSource::AlicePosition)
		{
#if USE_ALICE
			if (IHybridData::GetPtr ()) {
				EDataMode mode = AliceIns ().GetDataMode ();
				if (mode == EDataMode::World)
				{
					PNComp->SetWorldLocation (RealtimeHandWorldPos);
					PNComp->SetWorldRotation (RealtimeHandWorldQuat);
				}
				else if (mode == EDataMode::Relative)
				{
					PNComp->SetRelativeLocationAndRotation (RealtimeHandWorldPos, RealtimeHandWorldQuat);
				}
				else {
					PN_LOG (Error, TEXT ("Unknown Data Mode."));
				}
			}
#else
			PN_LOG (Error, TEXT ("Cannot work without Alice plugin! Please install Alice plugin first."));
#endif	// USE_ALICE
		}

		/*FVector HandToRootPositionVec = WorldPositions[0] - WorldPositions[HandBoneIndex];
		FVector FinalRootPos = RealtimeHandWorldPos + RealtimeHandWorldQuat.RotateVector (HandToRootPositionVec);*/

		/*Container[0].SetLocation(FinalRootPos);
		Container[0].SetRotation(RealtimeHandWorldQuat);*/

		//if (curDataMode == GM_RightGlove) {
		//	//PNComp->SetWorldLocation(FVector(0, 20, 50));
		//}
		//else {
		//	//PNComp->SetWorldLocation(FVector(0, -20, 50));

		//	auto cubes = OwnerActor->GetComponentsByTag (UStaticMeshComponent::StaticClass (), TEXT ("CUBE"));
		//	if (cubes.IsValidIndex (0))
		//	{
		//		UStaticMeshComponent* cube = Cast<UStaticMeshComponent> (cubes[0]);
		//		cube->SetRelativeRotation (RealtimeHandWorldQuat);
		//	}
		//}

		//PN_LOG (Warning, TEXT ("ReturnPos: %s"), *RealtimeHandWorldPos.ToString());
		//return;

		for (int32 i = 0; i < BoneMap.Num(); i++)
		{
			FName CTBoneName = BoneMap[i].CTName;
			uint8 PNBoneID = BoneMap[i].PNID;

			if (PNBoneID >= AnimData.BoneRotations.Num())
				continue;
			if (EMCBones::Invalid == PNBoneID)	// No bound bone
			{
				continue;
			}

			if (EMCBones::LeftForeArm == PNBoneID ||
				EMCBones::RightForeArm == PNBoneID
				) {
				continue;
			}

			int32 BoneIndex = SkeleMeshComp->GetBoneIndex(CTBoneName);	// Bound none bone
			if ((INDEX_NONE == BoneIndex) || (BoneIndex > BoneCount)) {
				continue;
			}

			if (EMCBones::LeftHand == PNBoneID ||
				EMCBones::RightHand == PNBoneID) {
				Container[0].SetLocation (Container[0].GetLocation () - WorldPositions[BoneIndex]);
				continue;
			}

			FQuat OrigRotation = Container[BoneIndex].GetRotation();	// T-Pose
			FQuat NewRotation = AnimData.BoneRotations[PNBoneID];	// Realtime Rotation Data
			FVector NewPosition = AnimData.BonePositions[PNBoneID];

			int32 CTParentBoneIndex = Container.GetParentBoneIndex(BoneIndex);
			if (INDEX_NONE != CTParentBoneIndex)
			{	// 父骨骼不为空，说明该骨骼是一根子骨骼
				FQuat CTParentWorldRotation = WorldRotations[CTParentBoneIndex];
				FVector	CTParentWorldPosition = WorldPositions[CTParentBoneIndex];

				FQuat PNParentWorldRotation = CTParentWorldRotation;
				FVector PNParentWorldPosition = CTParentWorldPosition;

				EMCBones::Type PNParentBoneID = FMotionCaptureHelper::PNBoneParentIndices[PNBoneID];
				if (EMCBones::Invalid != PNParentBoneID)	// not hips
				{
					if (BoneID2NameMap.Contains(PNParentBoneID))
					{
						TArray<FName> ParentNames;
						BoneID2NameMap.MultiFind(PNParentBoneID, ParentNames);
						ParentNames.Sort(BoneNameComp);
						FName UEParentBoneName = ParentNames[0];

						int32 PNParentBoneIndex = SkeleMeshComp->GetBoneIndex(UEParentBoneName);
						if ((INDEX_NONE == PNParentBoneIndex) || (PNParentBoneIndex > BoneCount))
						{
							continue;
						}
						PNParentWorldRotation = WorldRotations[PNParentBoneIndex];
						PNParentWorldPosition = WorldPositions[PNParentBoneIndex];
					}
				}

				// Final orientation is : T-Pose + RelationRotation
				// bone from ZERO orientation to T-Pose, then continue rotating [NewRotation].
				NewRotation = CTParentWorldRotation.Inverse() * NewRotation * WorldRotations[BoneIndex];
			}
			else//根骨骼的旋转
			{
 				NewRotation = NewRotation * OrigRotation;
			}

			Container[BoneIndex].SetRotation(NewRotation);

#ifdef TRANCE
			if (PNComp->UseDisplacement)//组件配置了使用位移数据
			{
				if ((HandType == PNBoneID) || AnimData.bWithDisplacement)//数据是否有位移
				{
					if (HandType == PNBoneID)
					{
						USceneComponent* PTrackingComp = SkeleMeshComp->GetAttachParent ();
						if (PTrackingComp != nullptr)
						{
							FVector rootPos = PTrackingComp->GetComponentLocation ();
							Container[BoneIndex].SetLocation (rootPos);
							//
						}
					}
				}
			}
#else
			if ((RootType == PNBoneID) || AnimData.bWithDisplacement)
			{
				Container[BoneIndex].SetLocation(NewPosition);
			}
#endif // 
		}

	}
}

void FAnimationNode_NewPoseCalc::GatherDebugData(FNodeDebugData& DebugData)
{
	
}

#if WITH_EDITOR
bool FAnimationNode_NewPoseCalc::InitBoneMap()
{
	TArray<FName> SkeleBoneNames;
	auto GetBoneNames = [&]() {
		if (nullptr == this->TargetSkeleton)
		{
			SkeleBoneNames.Empty();
		}
		else
		{
			// pre-size the array to avoid unnecessary reallocation
			SkeleBoneNames.Empty(this->TargetSkeleton->GetReferenceSkeleton().GetNum());
			SkeleBoneNames.AddUninitialized(this->TargetSkeleton->GetReferenceSkeleton().GetNum());
			for (int32 i = 0; i < this->TargetSkeleton->GetReferenceSkeleton().GetNum(); i++)
			{
				SkeleBoneNames[i] = this->TargetSkeleton->GetReferenceSkeleton().GetBoneName(i);
			}
		}
	};
	GetBoneNames();

	BoneMap.Empty(SkeleBoneNames.Num());
	BoneName2BoneMapIndex.Empty(SkeleBoneNames.Num());
	BoneID2NameMap.Empty(MCBONE_NUM);

	for (const FName& BoneName : SkeleBoneNames)
	{
		FName BoneNameWithoutPrefix (*(BoneName.GetPlainNameString ().Mid (SkeletonNamePrefix.Len ())));

		EMCBones::Type BoneID = EMCBones::Invalid;
		bool bContains = FMotionCaptureHelper::BoneName2IDMap.Contains(BoneNameWithoutPrefix);
		if (bContains)
		{
			BoneID = FMotionCaptureHelper::BoneName2IDMap.FindChecked(BoneNameWithoutPrefix);
			BoneID2NameMap.AddUnique(BoneID, BoneName);
			BoneName2BoneMapIndex.Add(BoneName, BoneMap.Num());
		}
		else
		{
			PN_LOG(Warning, TEXT("Can't find a matched PN bone for the bone %s."), *(BoneName.ToString()));
		}

		BoneMap.Add(FBoneMappingInfo(BoneID, BoneName));
	}

	BoneMap_Copy = BoneMap;

	return true;
}

void FAnimationNode_NewPoseCalc::SetTargetSkeleton(const USkeleton* Skeleton)
{
	TargetSkeleton = Skeleton;
}

void FAnimationNode_NewPoseCalc::ValueSet_Name(int32 ChangeAtIndex)
{
	FName NewCTName = BoneMap[ChangeAtIndex].CTName;
	if (BoneName2BoneMapIndex.Contains(NewCTName))
	{
		BoneMap[ChangeAtIndex] = BoneMap_Copy[ChangeAtIndex];
		return;
	}
	const FBoneMappingInfo* pElem = BoneMap_Copy.FindByPredicate([&](const FBoneMappingInfo& Elem)->bool{
		if (Elem.CTName == NewCTName)
		{
			return true;
		}
		return false;
	});
	if (pElem)
	{
		BoneMap[ChangeAtIndex] = BoneMap_Copy[ChangeAtIndex];
		return;
	}
	EMCBones::Type BoneID = BoneMap[ChangeAtIndex].PNID;
	if (EMCBones::Invalid != BoneID)
	{
		BoneID2NameMap.Remove(BoneID, BoneMap_Copy[ChangeAtIndex].CTName);
		BoneName2BoneMapIndex.Remove(BoneMap_Copy[ChangeAtIndex].CTName);
		BoneID2NameMap.AddUnique(BoneID, NewCTName);
		BoneName2BoneMapIndex.Add(NewCTName, ChangeAtIndex);
	}
	BoneMap_Copy[ChangeAtIndex] = BoneMap[ChangeAtIndex];
}

void FAnimationNode_NewPoseCalc::ValueSet_ID(int32 ChangeAtIndex)
{
	EMCBones::Type CurPNID = BoneMap[ChangeAtIndex].PNID;
	EMCBones::Type PrePNID = BoneMap_Copy[ChangeAtIndex].PNID;
	if ((EMCBones::Invalid == CurPNID) && (EMCBones::Invalid != PrePNID))
	{
		BoneID2NameMap.Remove(PrePNID, BoneMap_Copy[ChangeAtIndex].CTName);
		BoneName2BoneMapIndex.FindAndRemoveChecked(BoneMap_Copy[ChangeAtIndex].CTName);
	}
	else
	{
		if (EMCBones::Invalid != PrePNID)
		{
			BoneID2NameMap.Remove(PrePNID, BoneMap_Copy[ChangeAtIndex].CTName);
			BoneName2BoneMapIndex.FindAndRemoveChecked(BoneMap_Copy[ChangeAtIndex].CTName);
		}

		BoneID2NameMap.AddUnique(CurPNID, BoneMap[ChangeAtIndex].CTName);
		BoneName2BoneMapIndex.Add(BoneMap[ChangeAtIndex].CTName, ChangeAtIndex);
	}

	BoneMap_Copy[ChangeAtIndex] = BoneMap[ChangeAtIndex];
}

void FAnimationNode_NewPoseCalc::Add_BoneMap(int32 AddAtIndex)
{
	for (int32 Index = AddAtIndex + 1; Index < BoneMap.Num(); ++Index)
	{
		FName CTName = BoneMap[Index].CTName;
		if (BoneName2BoneMapIndex.Contains(CTName))
		{
			++BoneName2BoneMapIndex[CTName];
		}
	}

	BoneMap_Copy.Insert(BoneMap[AddAtIndex], AddAtIndex);
}

void FAnimationNode_NewPoseCalc::Del_BoneMap()
{
	int32 DelAtIndex = INDEX_NONE;
	for (int32 Index = 0; Index < BoneMap.Num(); ++Index)
	{
		if (BoneMap[Index] != BoneMap_Copy[Index])
		{
			DelAtIndex = Index;
			break;
		}
	}

	for (int32 Index = DelAtIndex; Index < BoneMap.Num(); ++Index)
	{
		FName CTName = BoneMap[Index].CTName;
		if (BoneName2BoneMapIndex.Contains(CTName))
		{
			--BoneName2BoneMapIndex[CTName];
		}
	}

	EMCBones::Type DelID = BoneMap_Copy[DelAtIndex].PNID;
	FName DelCTName = BoneMap_Copy[DelAtIndex].CTName;
	if (EMCBones::Invalid != DelID)
	{
		BoneID2NameMap.Remove(DelID, DelCTName);
		BoneName2BoneMapIndex.Remove(DelCTName);
	}

	BoneMap_Copy.RemoveAt(DelAtIndex);
}

void FAnimationNode_NewPoseCalc::Revert_BoneMap()
{
	BoneMap = BoneMap_Copy;
}
#endif


FQuat FAnimationNode_NewPoseCalc::StandardAxis = FQuat(FVector(1, 0, 0), FMath::DegreesToRadians(-90));
