// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "GlobalDefine.h"

enum class EEventType : char
{
	EVENT_TYPE_CREATE = 0,
	EVENT_TYPE_MOVE,
	EVENT_TYPE_INVALID = INDEX_NONE,
};

class AMotionCaptureManager;
#ifndef TRANCE
class UMotionCaptureComponent;
#else
class UMotionCaptureAvatarComponent;
#endif
class FEventBase
{
public:
	virtual ~FEventBase()
	{

	}

	FEventBase();

	FEventBase(EEventType Type);

	EEventType EventType = EEventType::EVENT_TYPE_INVALID;


	DECLARE_EVENT_OneParam(FEventBase, FPNActorEvent, const FEventBase*);

	void Fire() const
	{
		PNActorEvent.Broadcast(this);
	}

protected:
	FPNActorEvent& OnPNActorEvent()
	{
		return PNActorEvent;
	}

private:
	FPNActorEvent PNActorEvent;
};


class FCreateNewEvent : public FEventBase
{
public:
	FCreateNewEvent();
	FCreateNewEvent(const TWeakObjectPtr<AMotionCaptureManager>& Manager,
		const FName& Name, int32 ID, const FVector& InInitPos, const FQuat& InInitQua);

	FName AvatarName	= NAME_None;
	int32 AvatarID		= INDEX_NONE;
	FVector InitPos		= FVector::ZeroVector;
	FQuat InitQua		= FQuat::Identity;
};

#ifndef TRANCE
class FMoveToEvent : public FEventBase
{
public:
	FMoveToEvent();
	FMoveToEvent(const TWeakObjectPtr<UMotionCaptureComponent>& InComponent, const FVector& Pos, const FQuat& Qua);

	TWeakObjectPtr<UMotionCaptureComponent> Component;
	FVector DestPos = FVector::ZeroVector;
	FQuat DestQua	= FQuat::Identity;
};
#else 

class FMoveToEvent : public FEventBase
{
public:
	FMoveToEvent();
	FMoveToEvent(const TWeakObjectPtr<UMotionCaptureAvatarComponent>& InComponent, const FVector& Pos, const FQuat& Qua);

	TWeakObjectPtr<UMotionCaptureAvatarComponent> Component;
	FVector DestPos = FVector::ZeroVector;
	FQuat DestQua = FQuat::Identity;
};
#endif
