// Fill out your copyright notice in the Description page of Project Settings.
#include "GlovePacketParser.h"
#include "HI5Glove/GloveDataType.h"
#include "MotionCaptureFunctionLibrary.h"

namespace Hi5
{
	FGlovePacketParser::FGlovePacketParser( ) :
		FMotionCaptureHelper( ),
		CurMaxPacketID( 0 )
	{
	}

	FGlovePacketParser::~FGlovePacketParser( )
	{
	}


	bool FGlovePacketParser::Read( const FArrayReaderPtr& Data, FAnimationDataPtr AnimData, UObject* PawnObj )
	{
		if (!CheckHeader( Data ))
		{
			return false;
		}

		const uint8* RawData = Data->GetData( );
		int Offset = FGlovePacketHeader::HeaderSize;

		AnimData->GloveMode = PacketHeader.GloveMode;

		// 	if (0 != PacketHeader.WithReference)
		// 	{
		// 		FVector RootPosition = (RawData, Offset);
		// 		AnimData->RootPosition = FVector(RootPosition.X, RootPosition.Z, RootPosition.Y);
		// 		AnimData->RootRotation = GetQuaternion(RawData, Offset);
		// 		AnimData->bUseRootEffect = true;
		// 	}

		bool IsLeft;
		EMCBones::Type RootType = EMCBones::Hips;
		if (PacketHeader.GloveMode == HI5Device::GM_LeftGlove)
		{
			IsLeft = true;
			RootType = EMCBones::LeftForeArm;
		}
		else if (PacketHeader.GloveMode == HI5Device::GM_RightGlove)
		{
			IsLeft = false;
			RootType = EMCBones::RightForeArm;
		}
		else
		{
			return false;
		}

		FVector HipPos;
		FQuat HipQuat;
		///FMotionCaptureHelper::Test(RawData,Offset);
		EGlovePositionSource ePosSrc = UMotionCaptureFunctionLibrary::GetGlovePositionSource( );
		int tempoffset = Offset;
		if (ePosSrc == EGlovePositionSource::VivePosition)
		{
			tempoffset = Offset;
			HipPos = GetVivePosition( RawData, Offset );
			tempoffset = Offset;
			HipQuat = GetViveQuaternion( RawData, Offset );
			tempoffset = Offset;
		}
		else if (ePosSrc == EGlovePositionSource::AlicePosition)
		{
			HipPos = GetAlicePosition( RawData, Offset, PawnObj );
			HipQuat = GetAliceQuaternion( RawData, Offset, PawnObj );
		}

		AnimData->BonePositions[RootType] = HipPos;
		AnimData->BoneRotations[RootType] = HipQuat;

		if (0 != PacketHeader.bWithDisp)
		{
			AnimData->bWithDisplacement = true;
		}
		else
		{
			AnimData->bWithDisplacement = false;
		}

		//for (int i = 1; i < FGlovePacketParser::PNBoneCount; i++)
		for (int32 i = 1; i < HI5Device::LeftHandBoneMaxNumb; i++)
		{
			ParseAnimData( IsLeft, RawData, i, Offset, AnimData, PawnObj );
		}

		return true;
	}

	bool FGlovePacketParser::CheckHeader( const FArrayReaderPtr& Data )
	{
		if (FGlovePacketHeader::HeaderSize > Data->Num( ))
		{
			return false;
		}

		if (!ParseHeader( Data ))
		{
			PN_LOG( Error, TEXT( "Parse packet header failed." ) );
			return false;
		}

		return true;
	}

	void FGlovePacketParser::ParseAnimData( const bool IsLeft, const uint8* Data, const int32 Index, int32& Offset, FAnimationDataPtr AnimData, UObject* PawnObj )
	{
		int32 realindex = (int32)(IsLeft ? (EMCBones::LeftForeArm) : (EMCBones::RightForeArm)) + Index;

		uint8 PNBoneIndex = PNBoneIndices[realindex];
		if (AnimData->BonePositions.Num( ) <= PNBoneIndex)
			return;

		EGlovePositionSource ePosSrc = UMotionCaptureFunctionLibrary::GetGlovePositionSource( );

		if (0 != PacketHeader.bWithDisp)
		{
			if ((PNBoneIndex == EMCBones::LeftHand) || (PNBoneIndex == EMCBones::RightHand))
			{
				// 暂时维持现状，不对Vive数据应用GameSpace
				if (ePosSrc == EGlovePositionSource::VivePosition) {
					FVector BonePos = GetVivePosition( Data, Offset );
					AnimData->BonePositions[PNBoneIndex] = BonePos;
				}
				else if (ePosSrc == EGlovePositionSource::AlicePosition)
				{
					FVector BonePos = GetAlicePosition( Data, Offset, PawnObj );
					AnimData->BonePositions[PNBoneIndex] = BonePos;
				}
				else {
					PN_LOG( Error, TEXT( "Unknown Position Source!" ) );
					return;
				}
			}
			else
			{
				FVector BonePos = GetPosition( Data, Offset );
				//AnimData->BonePositions[PNBoneIndex] = BonePos;
			}
		}

		if ((PNBoneIndex == EMCBones::LeftHand) || (PNBoneIndex == EMCBones::RightHand))
		{
			if (ePosSrc == EGlovePositionSource::VivePosition)
			{
				FQuat BoneQuat = GetViveQuaternion( Data, Offset );
				AnimData->BoneRotations[PNBoneIndex] = BoneQuat;
			}
			else if (ePosSrc == EGlovePositionSource::AlicePosition)
			{
				FQuat BoneQuat = GetAliceQuaternion( Data, Offset, PawnObj );
				AnimData->BoneRotations[PNBoneIndex] = BoneQuat;
			}
			else
			{
				PN_LOG( Error, TEXT( "Unknown Position Source!" ) );
				return;
			}
		}
		else
		{
			FQuat BoneQuat = GetQuaternion( Data, Offset );
			AnimData->BoneRotations[PNBoneIndex] = BoneQuat;
		}
	}

	bool FGlovePacketParser::ParseHeader( const FArrayReaderPtr& Data )
	{
		const uint8* RawData = Data->GetData( );
		int32 Offset = 0;
		for (TFieldIterator<FProperty> PropIt( FGlovePacketHeader::StaticStruct( ), EFieldIteratorFlags::ExcludeSuper ); PropIt; ++PropIt)
		{
			FProperty* Property = *PropIt;

			if (Property->IsA( FByteProperty::StaticClass( ) ))
			{
				uint8 Value = GetUInt8( RawData, Offset );
				FByteProperty* ByteProp = CastField<FByteProperty>( Property );
				ByteProp->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<uint8>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FInt16Property::StaticClass( ) ))
			{
				int16 Value = GetUInt16( RawData, Offset );
				FInt16Property* Int16Prop = CastField<FInt16Property>( Property );
				Int16Prop->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<int16>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FUInt16Property::StaticClass( ) ))
			{
				uint16 Value = GetUInt16( RawData, Offset );
				FUInt16Property* UInt16Prop = CastField<FUInt16Property>( Property );
				UInt16Prop->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<uint16>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FIntProperty::StaticClass( ) ))
			{
				int32 Value = GetInt32( RawData, Offset );
				FIntProperty* IntProp = CastField<FIntProperty>( Property );
				IntProp->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<int32>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FUInt32Property::StaticClass( ) ))
			{
				uint32 Value = GetUInt32( RawData, Offset );
				FUInt32Property* UIntProp = CastField<FUInt32Property>( Property );
				UIntProp->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<uint32>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FFloatProperty::StaticClass( ) ))
			{
				float Value = GetFloat( RawData, Offset );
				FFloatProperty* FloatProp = CastField<FFloatProperty>( Property );
				FloatProp->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<float>( &PacketHeader )), Value );
			}
			else if (Property->IsA( FDoubleProperty::StaticClass( ) ))
			{
				double Value = GetDouble( RawData, Offset );
				FDoubleProperty* DoubleProp = CastField<FDoubleProperty>( Property );
				DoubleProp->SetPropertyValue( (void*)(Property->ContainerPtrToValuePtr<float>( &PacketHeader )), Value );
			}
			// 		else if (Property->IsA(FNameProperty::StaticClass()))
			// 		{
			// 			FString PropName = Property->GetName();
			// 			if (FString(TEXT("AvatarName")) == PropName)
			// 			{
			// 				PacketHeader.AvatarName = FName(*GetString(RawData, Offset, FPNPacketHeader::AvatarNameSize));	// temp: name length is 32 bytes.  We need to send name length within the protocol which can bear changes from server side.
			// 			}
			// 		}
		}

		return Offset == FGlovePacketHeader::HeaderSize;
	}

	int32 FGlovePacketParser::GetGloveMode( )
	{
		return PacketHeader.GloveMode;
	}
}
