// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "MotionCaptureHelper.h"
#include "AnimationData.h"
#include "Runtime/Networking/Public/Common/UdpSocketReceiver.h"

#include "GlovePacketParser.generated.h"


USTRUCT()
struct FGlovePacketHeader
{
	GENERATED_USTRUCT_BODY()

	static const uint16 HeaderSize = 36;

	UPROPERTY()
		int32			GloveMode = -1;

	UPROPERTY()
		double			TimeStamp_time = 0.0;

	UPROPERTY()
		double			TimeStamp_Latency = 0.0;

	UPROPERTY()
		uint32			bUpdate = 0;

	UPROPERTY()
		uint32			bWithDisp = 0;

	UPROPERTY()
		uint32			FrameIndex = 0;

	UPROPERTY()
		uint32			DataCount = 0;
};

namespace Hi5
{
	/**
	 *
	 */
	class FGlovePacketParser : public FMotionCaptureHelper
	{
	public:
		FGlovePacketParser( );
		virtual ~FGlovePacketParser( );

		virtual bool Read( const FArrayReaderPtr& Data, FAnimationDataPtr AnimData, UObject* PawnObj = nullptr );

		int32 GetGloveMode( );

	protected:
		virtual bool CheckHeader( const FArrayReaderPtr& Data );

		void ParseAnimData( const bool IsLeft, const uint8* Data, const int32 Index, int32& Offset, FAnimationDataPtr AnimData, UObject* PawnObj = nullptr );

		FGlovePacketHeader PacketHeader;
		uint32 CurMaxPacketID;

	private:
		bool ParseHeader( const FArrayReaderPtr& Data );
	};
}

