#pragma once

#include "CoreMinimal.h"
#include "HI5Glove/HI5Glove_C.h"
#include "Hi5GloveDataType.h"
#include <functional>
#include "SteamVRFunctionLibrary.h"
#include "MotionCaptureFunctionLibrary.h"

#if USE_ALICE
#include "IHybridData.h"

IHybridData& AliceIns ();
#endif

using namespace HI5Device;

enum AliceInputType
{
	AIT_None = 0,
	AIT_Wii,        // WII
	AIT_GameSir,    // GameSir
	AIT_Keyboard,   // Keyboard
	AIT_HI5Glove,   // Hi5 Glove
};

enum CmdGlove
{
	CG_None = 0,        // Unknown type
	CG_CalibPrepare,    // [send] calibration prepare
	CG_CalibBPose,      // [send] calibration b pose
	CG_CalibPPose,      // [send] calibration p pose
	CG_CalibBPosePer,   // [recv] calibration b pose percent
	CG_CalibPPosePer,   // [recv] calibration p pose percent
	CG_CalibError,      // [recv] calibration error
	CG_Max,             // Max type
};

void CaliInVive(CalibrationPose pose, int wait/*ms*/);
void CaliInAlice (CalibrationPose pose, int wait/*ms*/);

void tDeviceReadThead(/*void* param*/);

void tVivePushThread();

int StartReadGlove();

void StopReadGlove();

void Calibrationglove(CalibrationPose pos, int wait);

int GetCalibrationProgressInPercent();

void CalibrationPrepare();

void ModifyDataRotate(GloveBVHData& bvhData, int offset, float valueoffset);

int GetGloveData(GloveMod LeftOrRight, uint8 *data);

bool WCharToChar(wchar_t* strWChar, int WCharLen, char *OutValue, int& outsize);

void tCalibrateThread(/*void* param*/);

void pushGloveTrackerData(bool isLeft, float* pos/*xyz*/, float* quat/*sxyz*/);

FTransform GetChildWorldTransform(const FTransform& ParentWorldTransform, const FTransform& RelativeTransform);

void RegistViveTrackCallBack(bool IsLeftHand, std::function<bool(float*, float*) > func);

void hi5_enableVibratorOfLeft (unsigned int leftGloveTimeSpan/*ms*/);
void hi5_enableVibratorOfRight (unsigned int rightGloveTimeSpan/*ms*/);
void hi5_enableVibratorOfBoth (unsigned int leftGloveTimeSpan/*ms*/, unsigned int rightGloveTimeSpan/*ms*/);

bool hi5_getViveTrackerSN (const int32 index, FString& sn);
bool hi5_getViveTrackerId (const FString& sn, int32& id);

bool hi5_isDeviceMissing (int id);

// for ue4.14
bool hi5_getDeviceType (int ID, ESteamVRTrackedDeviceType& Type);	// vive tracker's enum type is [invalid] in current openvr library.

// The returned result is in UE space
bool hi5_getTrackedDevicePose (int unTrackedDevice, FVector& Position, FRotator& Rotator);

void hi5_getGlovePowerLevel (EGlovePowerLevel& left, EGlovePowerLevel& right);
void hi5_getGloveMag (EGloveMagneticedState& left, EGloveMagneticedState& right);
void __stdcall hi5_fnGloveStateChanged (void* userData, GloveMod glove, GloveEvent event);
void __stdcall hi5_fnDongleStateChanged (void *userData, const DongleIdentity *hid, DongleEvent action);
void hi5_initGloveStatus ();

FString __hi5_getTrackerBindState (bool left, OpticalSensorType& sensor_type);
FString hi5_getTrackerBindState (bool left, OpticalSensorType& sensor_type);
void hi5_loadTrackerBindState (bool left, const FString& sn, const OpticalSensorType& sensor_type);
bool hi5_loadCalibrationFile (char* filePath);

bool hi5_isDongleAvailable ();
bool hi5_isLeftGloveAvailable ();
bool hi5_isRightGloveAvailable ();

void __hi5_stopThreads ();

void __hi5_getHandOffsetToOpticalDeviceInUE (FVector& LeftPosition, FVector& RightPosition);
void hi5_getHandOffsetToOpticalDeviceInUE (FVector& LeftPosition, FVector& RightPosition);

void hi5_cali_mutex_lock ();
void hi5_cali_mutex_unlock ();

bool hi5_isInViveSystem ();
bool hi5_isInAliceSystem ();

// for test
void patch_wrist_offset ();
//Hi5_2
void hi5_2tReadOpticalDataBody();
void JudgeLeftHandAndRightHDeviceId(APawn* inPawn);
void hi5_2PreCalibration();
void hi5_2CalibrationEpilog(FString snLeft, FString snRight);
