// Fill out your copyright notice in the Description page of Project Settings.


#include "HI5_GloveStatus.h"
#include "Hi5_Manager_Thread.h"
const float FHI5_GloveStatus::delayCd = 15000.0f;
FHI5_GloveStatus::FHI5_GloveStatus()
{
	m_leftPL = EPowerLevel::EPowerLevelUnknown;
	m_RightPL = EPowerLevel::EPowerLevelUnknown;
	m_leftMag = EMagneticStatus::EMagneticStatusUnknown;
	m_rightMag = EMagneticStatus::EMagneticStatusUnknown;
	m_leftMagTemp = EMagneticStatus::EMagneticStatusUnknown;
	m_rightMagTemp = EMagneticStatus::EMagneticStatusUnknown;
	m_isLeftGloveAvailable = false;
	m_isRightGloveAvailable = false;
	m_StartleftTimer = false;
	m_StartrightTimer = false;
	m_currentStatus = EGloveStatus::EUnStatusknown;
	m_Hi5_StatueData = nullptr;
}

FHI5_GloveStatus::~FHI5_GloveStatus()
{
}

EPowerLevel FHI5_GloveStatus::GetLeftPower()
{
	return m_leftPL;
};
void FHI5_GloveStatus::SetLeftPower(EPowerLevel inValue)
{
	m_leftPL = inValue;
};
EPowerLevel FHI5_GloveStatus::GetRightPower()
{
	return m_RightPL;
};
void FHI5_GloveStatus::SetRightPower(EPowerLevel inValue)
{
	m_RightPL = inValue;
}

EMagneticStatus FHI5_GloveStatus::GetLeftMag()
{
	return m_leftMag;
}

void FHI5_GloveStatus::SetLeftMag(EMagneticStatus inValue)
{
	m_leftMag = inValue;
}

EMagneticStatus FHI5_GloveStatus::GetRightMag()
{
	return m_rightMag;
}

void FHI5_GloveStatus::SetRightMag(EMagneticStatus inValue)
{
	m_rightMag = inValue;
}

EMagneticStatus FHI5_GloveStatus::GetLeftMagTemp()
{
	return m_leftMagTemp;
}
void FHI5_GloveStatus::SetLeftMagTemp(EMagneticStatus inValue)
{
	m_leftMagTemp = inValue;
}
EMagneticStatus FHI5_GloveStatus::GetRightMagTemp()
{
	return m_rightMagTemp;
}
void FHI5_GloveStatus::SetRightMagTemp(EMagneticStatus inValue)
{
	m_rightMagTemp = inValue;
}

bool FHI5_GloveStatus::GetIsLeftGloveAvailable()
{
	return m_isLeftGloveAvailable;
}

void FHI5_GloveStatus::SetIsLeftGloveAvailable(bool isValue)
{
	m_isLeftGloveAvailable = isValue;
}

bool FHI5_GloveStatus::GetIsRightGloveAvailable()
{
	return m_isRightGloveAvailable;
}

void FHI5_GloveStatus::SetIsRightGloveAvailable(bool isValue)
{
	m_isRightGloveAvailable = isValue;
}

void FHI5_GloveStatus::Update(float DetaTime)
{
	DelayTime();
}

EGloveStatus FHI5_GloveStatus::Status()
{
	return m_currentStatus;
}

void FHI5_GloveStatus::StartLeftTimer()
{
	m_StartleftTimer = true;
	m_leftTimer = clock();  
}

void FHI5_GloveStatus::StartRightTimer()
{
	m_StartrightTimer = true;
	m_rightTimer = clock();  
}

void FHI5_GloveStatus::EndleftTimer()
{
	m_StartleftTimer = false;
}

void FHI5_GloveStatus::EndRightTimer()
{
	m_StartrightTimer = false;
}

void FHI5_GloveStatus::DelayTime()
{ 
	if(m_StartleftTimer)
	{
		if(clock()-m_leftTimer < delayCd)
		{
			
		}
		else
			m_StartleftTimer = false;
	}
	if(m_StartrightTimer)
	{
		if(clock()-m_rightTimer < delayCd)
		{
			
		}
		else
			m_StartrightTimer = false;
	}
}
void FHI5_GloveStatus::HI5_GloveStatusOnEventChanged(GloveMod gloveMod, GloveEvent gloveEvent)
{

	if (gloveMod == GloveMod::GM_LeftGlove)
	{
		switch (gloveEvent)
		{
		case GloveEvent::GE_FullPower:
			SetLeftPower(EPowerLevel::EPowerLevelFull);
			break;
		case GloveEvent::GE_NormalPower:
			SetLeftPower(EPowerLevel::EPowerLevelNormal);
			break;
		case GloveEvent::GE_LowPower:
			SetLeftPower(EPowerLevel::EPowerLevelLow);
			break;
		case GloveEvent::GE_Magneticed:
			SetLeftMagTemp(EMagneticStatus::EMagneticStatusBad);
			StartLeftTimer();
			SetLeftMag(EMagneticStatus::EMagneticStatusFair);
			break;
		case GloveEvent::GE_NotMagneticed:
			SetLeftMag(EMagneticStatus::EMagneticStatusGood);
			EndleftTimer();
			break;
		}
	}

	if (gloveMod == GloveMod::GM_RightGlove)
	{
		switch (gloveEvent)
		{
		case GloveEvent::GE_FullPower:
			SetRightPower(EPowerLevel::EPowerLevelFull);
			break;
		case GloveEvent::GE_NormalPower:
			SetRightPower(EPowerLevel::EPowerLevelNormal);
			break;
		case GloveEvent::GE_LowPower:
			SetRightPower(EPowerLevel::EPowerLevelLow);
			break;
		case GloveEvent::GE_Magneticed:
			SetRightMagTemp(EMagneticStatus::EMagneticStatusBad);
			StartRightTimer();
			SetRightMag(EMagneticStatus::EMagneticStatusFair);
			//Debug.Log(" set rightMS = " + rightMS.ToString());
			break;
		case GloveEvent::GE_NotMagneticed:
			SetRightMag(EMagneticStatus::EMagneticStatusGood);
			EndRightTimer();
			//rightTimer.Stop();
			//rightTimer.Dispose();
			break;
		}
	}
}

 void FHI5_GloveStatus::MainThreadUpdate()
{
	//2019 8 2
	if(m_Hi5_StatueData.get() && m_Hi5_StatueData->isChange())
	{
		//UpdateGloveStatus(m_Hi5_StatueData.GetstatusData());
                
	}
}
bool GetIsLeftGloveAvailable();
void SetIsLeftGloveAvailable(bool isValue);
bool GetIsRightGloveAvailable();
void SetIsRightGloveAvailable(bool isValue);
void FHI5_GloveStatus::UpdateGloveVisible(EGloveStatus status)
{
	if (status == EGloveStatus::EGloveStatusBothGloveAvailable)
	{
		SetIsLeftGloveAvailable(true);
		SetIsRightGloveAvailable(true);
		return;
	}

	if (status == EGloveStatus::EGloveStatusLeftGloveAvailable)
	{
		SetIsLeftGloveAvailable(true);
		SetIsRightGloveAvailable(false);
		EndRightTimer();
		return;
	}

	if (status == EGloveStatus::EGloveStatusRightGloveAvailable)
	{
		SetIsLeftGloveAvailable(false);
		SetIsRightGloveAvailable(true);
		EndleftTimer();
		return;
	}
	//ruige 2019 6 26
	if (status == EGloveStatus::EGloveStatusNoDongle || status == EGloveStatus::EGloveStatusNoGlove || status == EGloveStatus::EUnStatusknown
		|| (FHi5_Manager_Thread::getInstance() != nullptr && (!FHi5_Manager_Thread::getInstance()->Isconneted()) ))
	{
		SetIsLeftGloveAvailable(false);
		SetIsRightGloveAvailable(false);
	
		//ReleaseBothTimerThread();
		return;
	}
}

void FHI5_GloveStatus::SetHi5_StatueData(std::shared_ptr<FHi5_status> inValue)
{
	m_Hi5_StatueData = inValue;
}

void FHI5_GloveStatus::ReleaseTimer()
{
	EndRightTimer();
	EndleftTimer();
}
