// Fill out your copyright notice in the Description page of Project Settings.


#include "HI5_MagneticStatus.h"
float FHI5_MagneticStatus::cd = 5000.0f;
FHI5_MagneticStatus::FHI5_MagneticStatus()
{
	m_StatusLeft = EMagneticStatus::EMagneticStatusUnknown;
	m_StatusRight = EMagneticStatus::EMagneticStatusUnknown;
	leftCd = cd;
	rightCd = cd;
}

FHI5_MagneticStatus::~FHI5_MagneticStatus()
{
}

void FHI5_MagneticStatus::SetLeftStatus(EMagneticStatus inValue)
{
	std::lock_guard<std::mutex> lck(m_leftMut);
	m_StatusLeft = inValue;
}

EMagneticStatus FHI5_MagneticStatus::GetLeftStatus()
{
	std::lock_guard<std::mutex> lck(m_leftMut);
	return m_StatusLeft;
}

void FHI5_MagneticStatus::SetRightStatus(EMagneticStatus inValue)
{
	std::lock_guard<std::mutex> lck(m_rightMut);
	m_StatusRight = inValue;
}

EMagneticStatus FHI5_MagneticStatus::GetRightStatus()
{
	std::lock_guard<std::mutex> lck(m_rightMut);
	return m_StatusRight;
}

void FHI5_MagneticStatus::Update(float detatime)
{
	if (GetLeftStatus() == EMagneticStatus::EMagneticStatusFair)
	{
		leftCd -= (detatime*1000.0f);
		if (leftCd <= 0.0f)
		{
			SetLeftStatus(EMagneticStatus::EMagneticStatusBad);
			leftCd = cd;
		}
	}
	if (GetRightStatus() == EMagneticStatus::EMagneticStatusFair)
	{
		rightCd -= (detatime * 1000.0f); ;
		if (rightCd <= 0.0f)
		{
			SetRightStatus(EMagneticStatus::EMagneticStatusBad);
			rightCd = cd;
		}
	}
}
void FHI5_MagneticStatus::RemoveDongle()
{
	SetLeftStatus(EMagneticStatus::EMagneticStatusGood);
	SetRightStatus(EMagneticStatus::EMagneticStatusGood);
}

void FHI5_MagneticStatus::OnEventChanged(GloveMod gloveMod, GloveEvent gloveEvent)
{
	if (gloveMod == GloveMod::GM_LeftGlove)
	{
		switch (gloveEvent)
		{               
		case GloveEvent::GE_Magneticed:
			SetLeftStatus( EMagneticStatus::EMagneticStatusFair);
			leftCd = cd;
			break;
		case GloveEvent::GE_NotMagneticed:
			SetLeftStatus( EMagneticStatus::EMagneticStatusGood);
			break;
		default:
			break;
		}
	}
	
	if (gloveMod == GloveMod::GM_RightGlove)
	{
		switch (gloveEvent)
		{
		case GloveEvent::GE_Magneticed:
			SetRightStatus( EMagneticStatus::EMagneticStatusFair);
			rightCd = cd;
			break;
		case GloveEvent::GE_NotMagneticed:
			SetRightStatus( EMagneticStatus::EMagneticStatusGood);
			break;
		default:
			break;
		}
	}          
}



