// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "MotionCaptureHelper.h"
#include "AnimationData.h"
#include "Common/UdpSocketReceiver.h"

#include "Hi5PacketParser.generated.h"


USTRUCT()
struct FHi5PacketHeader
{
	GENERATED_USTRUCT_BODY()

	static const uint16 HeaderSize = 64;
	static const uint16 ValidHeaderBegin = 0xDDFF;
	static const uint16 ValidHeaderEnd = 0xEEFF;
	static const uint16 AvatarNameSize = 32;
	
	UPROPERTY()
	uint16			HeaderBegin = 0;

	UPROPERTY()
	uint32			DataVersion = 0;

	UPROPERTY()
	uint16			DataCount = 0;

	UPROPERTY() 
	uint8			WithDisplacement = 0;

	UPROPERTY()
	uint8			WithReference = 0;

	UPROPERTY()
	uint32			AvatarID = INDEX_NONE;

	UPROPERTY()
	FName			AvatarName = NAME_None;

	UPROPERTY()
	uint32			Sequence = 0;

	UPROPERTY()
	uint32			Reserve = 0;

	UPROPERTY()
	uint32			Reserve1 = 0;

	UPROPERTY()
	uint32			Reserve2 = 0;

	UPROPERTY()
	uint16			HeaderEnd = 0;
};

namespace Hi5
{
	class FPacketParser : public FMotionCaptureHelper
	{
	public:
		FPacketParser( );
		virtual ~FPacketParser( );

		virtual bool Read( const FArrayReaderPtr& Data, FAnimationDataPtr AnimData, int32& AvatarID, FName& AvatarName );

	protected:
		virtual bool CheckHeader( const FArrayReaderPtr& Data );

		void ParseAnimData( const uint8* Data, const int32 Index, int32& Offset, FAnimationDataPtr AnimData );

		FHi5PacketHeader PacketHeader;
		uint32 CurMaxPacketID;

	private:
		bool ParseHeader( const FArrayReaderPtr& Data );
	};
}

