// Fill out your copyright notice in the Description page of Project Settings.


#include "Hi5Source.h"

FHi5Source::FHi5Source()
{
	ResetData(L_BonePos);
	ResetData(R_BonePos);
	ResetData(L_BoneRot);
	ResetData(R_BoneRot);
}

FHi5Source::~FHi5Source()
{
}

void FHi5Source::ResetData(FVector* bones)
{
	if(bones != nullptr)
	{
		for(int i =0 ;i<EHi5Bones::EHi5NumOfHI5Bones;i++)
		{
			bones[i] = FVector::ZeroVector;
		}
	}
}

FVector FHi5Source::GetReceivedPosition(int boneIndex, EHI5Hand handType)
{
	return handType == EHI5Hand::ELEFT ? L_BonePos[boneIndex] : R_BonePos[boneIndex];
}
 FVector FHi5Source::GetReceivedRotation(int boneIndex, EHI5Hand handType)
{
	return handType == EHI5Hand::ELEFT ? L_BoneRot[boneIndex] : R_BoneRot[boneIndex];
}

 void FHi5Source::UpdateTrackingData(GloveBVHData& gloveData)
{
	withDisp = gloveData.bWithDisp == 1;
	float glove_data[160];
	memset(glove_data,0,sizeof(glove_data));
	memcpy(glove_data,gloveData.data,sizeof(float)*160);

	GloveMod mode = gloveData.glove;
	int offset = 0;
	if (mode == GloveMod::GM_LeftGlove)
	{
		for (int i = 0; i < L_BoneRot->Size(); i++)
		{
			offset = withDisp ? 3 + 6 * i : 3 + 3 * i;
			L_BoneRot[i].Set(glove_data[offset+0], glove_data[offset+1], glove_data[offset+2]);
			if(withDisp)
				L_BonePos[i].Set(glove_data[i*6+0], glove_data[i*6+1], glove_data[i*6+2]);
		}
	}
	else if (mode == GloveMod::GM_RightGlove)
	{
		for (int i = 0; i < R_BoneRot->Size(); i++)
		{
			offset = withDisp ? 3 + 6 * i : 3 + 3 * i;
			R_BoneRot[i].Set(glove_data[offset+0], glove_data[offset+1], glove_data[offset+2]);
			if(withDisp)
				R_BonePos[i].Set(glove_data[i*6+0], glove_data[i*6+1], glove_data[i*6+2]);
		}
	}
}