// Fill out your copyright notice in the Description page of Project Settings.
#include "Hi5_Manager_Thread.h"
#include <exception>
#include "HI5_Calibration.h"
#include "NetworkManager.h"
#include "HI5Glove.h"
#include "MotionCaptureFunctionLibrary.h"
#include "UnrealXML.h"
#include <iostream>
#include "HAL/PlatformFilemanager.h"

//#include "Windows/AllowWindowsPlatformTypes.h"
//#include <shlobj.h>

//#include "Windows/HideWindowsPlatformTypes.h"
 FVector FHi5_Manager_Thread::leftOffset = {-0.025f, 0.063f, -0.04f};
FVector FHi5_Manager_Thread::rightOffset  = {0.04f, -0.052f, -0.04f};
float FHi5_Manager_Thread::leftTrackerInitOffset[3]  = { -0.025f, 0.063f, 0.04f };
float FHi5_Manager_Thread::rightTrackerInitOffset[3] = { 0.04f, -0.052f, 0.04f };
std::once_flag FHi5_Manager_Thread::m_Flg;
FHi5_Manager_Thread* FHi5_Manager_Thread::m_instance;
 std::mutex   FHi5_Manager_Thread::m_mutexClone;
extern bool locateUserRoamingFolder(FString& path);
extern std::string& replace_str(std::string& str, const std::string& to_replaced, const std::string& newchars); 
FHi5_Manager_Thread::FHi5_Manager_Thread()
{
	m_Commands = std::make_shared<FHI5_Calibration_Send_Data>();
	m_magneticStatus  = std::make_shared<FHI5_MagneticStatus>();
	m_HI5_GloveStatus = std::make_shared<FHI5_GloveStatus>();
	m_FHi5_status = std::make_shared<FHi5_status>();
	m_HI5Source = std::make_shared<FHi5Source>();
	m_FSensorInforArray = std::make_shared<FSensorInforArray>();
	m_gloveData = new GloveBVHData();
	mleftData = new GloveBVHData();
	mrightData = new GloveBVHData();
	m_FHi5_BVHDataQuene = std::make_shared<FHi5_BVHDataQuene>();
	Init();
}

FHi5_Manager_Thread::~FHi5_Manager_Thread()
{
	if( m_gloveData != nullptr)
	{
		delete m_gloveData;
		m_gloveData = nullptr;
	}
	
	if(mleftData != nullptr)
	{
		delete mleftData;
		mleftData = nullptr;
	}
	
	if(mrightData != nullptr)
	{
		delete mrightData;
		mrightData = nullptr;
	}
}
FHi5_Manager_Thread* FHi5_Manager_Thread::getInstance()
{
	std::lock_guard<std::mutex> lck(m_mutexClone);
	if(m_instance != nullptr)
	{
		return m_instance;
	}
	else
	{
		return nullptr;
	}
}
void FHi5_Manager_Thread::DestorManagerThready()
{
	if(m_instance != nullptr)
	{
		delete m_instance;
		m_instance = nullptr; 
	}
}

void FHi5_Manager_Thread::SetEnableFingerAdbFixed(bool inParam)
{
	isEnableFingerAdbFixed = inParam;
	if (Isconneted())
	{
		enableFingerAdbFixed(true);
	}
}

void FHi5_Manager_Thread::Init()
{
	m_isStop = false;
	IsStartDongle = false;
	IsCalibrationComplete = false;
	IsLoadData = false;
	isInertia = false;
	IsStartDongle = false;
	isRequestBindhand = false;
	SetIsConnected(false);
	SetCalibrationState(ECalibrationState::ECalibrationNone);
	if(m_FSensorInforArray.get())
	{
		m_FSensorInforArray->Init();
	}
	if(m_HI5_GloveStatus.get())
	{
		m_HI5_GloveStatus->SetHi5_StatueData(m_FHi5_status); 
	}
	for(int i=0;i<12;i++)
	{
		SenserDatas[i].SetFSenserItemData(0,0,0,0);
	}
	isEnableFingerAdbFixed = true;
}
void FHi5_Manager_Thread::Update(float detaTime)
{
	 if (!isDongleAvailable())
	 {
 		SetIsConnected(false);
	 }
	 else
	 {
	     SetIsConnected(true);
	 }
	 BindHandInfor();
	//
    if(Isconneted() && IsStartDongle)
     {
		/*if (isRequestBindhand)
		{
#if PLATFORM_WINDOWS

			FString filename = "OpticalDeviceBindInfo.xml";
			LoadOptiDeviceBindStateFromFile(filename);
			///UMotionCaptureFunctionLibrary::LoadOptiDeviceBindStateFromFile(filename);
#endif

			isRequestBindhand = false;
		}*/
     	if(m_FSensorInforArray.get())
    	{
     		m_FSensorInforArray->SetSensorInforArray(SenserDatas);
     	}
     	hi5_2tReadOpticalDataBody();
    	if (m_HI5_GloveStatus != nullptr)
    	{
    		m_HI5_GloveStatus->MainThreadUpdate();
    	}
    	// if(m_HI5Source.get())
    	 {
    	 	bool isLeftUpdate = false;
    	 	//GloveBVHData* leftData = new GloveBVHData();
    	 	if(m_FHi5_BVHDataQuene.get())
    	 	{
    	 		isLeftUpdate = m_FHi5_BVHDataQuene->GetBvhtemLeft( mleftData);
    	 		if(isLeftUpdate)
    	 		{
    	 			mHi5DataTest.writeBvhData(true,mleftData->dataCount,mleftData->data);
    	 			//m_HI5Source->UpdateTrackingData(*leftData);
    	 			NetworkManager::GetInstance()->OnLocalPlayerBVHDataReceived((uint8*)mleftData, sizeof(*mleftData));
    	 		}
			
    	 		bool isRightUpdate = false;
    	 		//GloveBVHData* RightData = new GloveBVHData();
    	 	
    	 		isRightUpdate = m_FHi5_BVHDataQuene->GetBvhtemRight( mrightData);
    	 		if (isRightUpdate)
    	 		{
    	 			mHi5DataTest.writeBvhData(false,mrightData->dataCount,mrightData->data);
    	 			NetworkManager::GetInstance()->OnLocalPlayerBVHDataReceived((uint8*)mrightData, sizeof(*mrightData));
    	 		}
    	 	}
    	 }
    	if (m_magneticStatus.get())
    		m_magneticStatus->Update(detaTime);
	}
}
bool FHi5_Manager_Thread::Isconneted() const
{
    return m_Isconneted;
}
void FHi5_Manager_Thread::SetIsConnected(bool inValue)
{
	m_Isconneted = inValue;
}
void FHi5_Manager_Thread::AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command inValue)
{
    m_Calibrationpercent = 0;
	
    if (IsStartDongle && Isconneted() && m_Commands.get())
        m_Commands->AddCalibrationCommand(inValue);
}
void FHi5_Manager_Thread::AddOpticData(std::string sensorIdentity, OpticalSensorType type, float fpos[3], float frot[4])
{
    if (IsStartDongle && Isconneted() && m_Commands.get())
        m_Commands->AddOpticData( sensorIdentity,  type,  fpos,  frot);
}
void FHi5_Manager_Thread::AddOptiSensorBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType)
{
	
	if (IsStartDongle && Isconneted() && m_Commands.get())
         m_Commands->AddOptiSensorBindData(mod, trackerID, sensorType);
}
void FHi5_Manager_Thread::GamePause(bool isPause)
{
	if(isPause)
	{
		SynCloseConnectEquipment();
		IsStartDongle = false;
	}
	else
	{
		RegisterEvent();
		SynConnectEquipment(false);
		IsStartDongle = true;
	}
}
void FHi5_Manager_Thread::RequestConnect(bool inertia)
{
	if (!IsStartDongle)
	{
		StartThread();
		RegisterEvent();
		SynConnectEquipment(inertia);
		IsStartDongle = true;
	}              
}
void FHi5_Manager_Thread::RequestCloseConnect()
{
	if (Isconneted())
	{
		SetIsConnected(false);
	}   
	if (IsStartDongle)
	{
		SynCloseConnectEquipment();
		IsStartDongle = false;
		StopThread();
	}
}
void Run()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		(FHi5_Manager_Thread::getInstance())->AsynUpdateThread();
	}
	
}
void FHi5_Manager_Thread::StartThread()
{
	m_phread = std::thread(Run);
}

void  FHi5_Manager_Thread::GetSensorInfor(int& index,FSenerInforBluePr& information)
{
	if(index < 12)
	{
		SenserDatas[index];
		information.m_magneticValue =SenserDatas[index].m_magneticValue;
		information.m_energyValue = SenserDatas[index].m_energyValue;
		information.m_signalValue = SenserDatas[index].m_signalValue;
		information.m_online = SenserDatas[index].m_online;
	}
}

void FHi5_Manager_Thread::AsynUpdateThread()
{
	while (!m_isStop)
	{
		if (m_Commands.get())
		{
			if (Isconneted() && IsStartDongle)
			{
				//校准数据
				std::shared_ptr<FHI5_Calibration_Command_Data> pCalibarationData = m_Commands->GetSendCalibration();
				if (pCalibarationData.get() )
				{
					while(!pCalibarationData->Empty())
					{
						std::shared_ptr<FHI5_Calibration_Command> item_data =  pCalibarationData->Pop();
						if(item_data.get())
						{
							AsynOperateCalibrationThread(item_data);
						}
					}
				}
				else
				{
					//获取校准百分比
					if (_CalibrationState == ECalibrationState::ECalibrationVing
					|| _CalibrationState == ECalibrationState::ECalibrationBing
					|| _CalibrationState == ECalibrationState::ECalibrationPing)
					{
						m_Calibrationpercent = AsyGetClibrationPercent();
						
						//WriteCalibrationPercent();
					}
				}
						//发送光学数据
				std::shared_ptr<FHI5_Optic_DataArray> pHI5_Optic_DataArray = m_Commands->GetSendOpticData();
				if (pHI5_Optic_DataArray.get())
				{
							//发送左手光学数据
					while (!pHI5_Optic_DataArray->Empty())
					{
						std::shared_ptr<FHI5_Optic_Data> temp = pHI5_Optic_DataArray->Pop();
						if(temp.get())
						{
							AsynSendOpticData(temp->m_sensorIdentity,temp->m_type,temp->m_fpos,temp->m_frot);
						}
					}
				}
				std::shared_ptr<FOptiSensorBindDataArray> pOptiSensorBindDataArray = m_Commands->GetSenserBindOpticData();
				
				while (pOptiSensorBindDataArray != nullptr && !pOptiSensorBindDataArray->Empty())
				{
					std::shared_ptr<FOptiSensorBindData> temp = pOptiSensorBindDataArray->Pop();
					if (temp.get())
					{
						
					 	AsynSendOpticBindData(temp->m_mod, temp->m_trackerID, temp->m_sensorType);
					}
				}
				AsynReceivThreadData();
			}                          
		}
		else
		{
			if (Isconneted() && IsStartDongle)
			{
				AsynReceivThreadData();
			}
		}
		std::this_thread::sleep_for(std::chrono::milliseconds(17));
	}
}

void FHi5_Manager_Thread::StopThread()
{
	if (m_phread.joinable() )
	{
		m_isStop = true;
		m_phread.join();
		DestorManagerThready();
	}
}

void FHi5_Manager_Thread::Reconnected()
{
	isRequestBindhand = false;
	restartCapture();
}

void FHi5_Manager_Thread::InitManagerThread()
{
	if (m_instance == nullptr)
	{
		std::lock_guard<std::mutex> lck(m_mutexClone);
		if(m_instance == nullptr)
			m_instance = new FHi5_Manager_Thread();
	}
	//std::call_once(m_Flg, [&] ; }); // 只会调用一次
}

void FHi5_Manager_Thread::SetHandThickness(float param)
{
	setHandThickness(param);
}
void FHi5_Manager_Thread::RegisterEvent()
{
	RegisterGloveStateChangedCallback();
	RegisterDongleStateChangedHandle();
}
//         //连接设备
void FHi5_Manager_Thread::SynConnectEquipment(bool inertia)
{
	if(inertia)
		appendSupportFlags("WithoutTracker");
	hi5DongleStart();
	setReducedOutputDataFreq(1);
	setWristPositionInTrackerFrame(leftTrackerInitOffset, rightTrackerInitOffset);
	enableFingerAdbFixed(true);
             //只有android 发送 
	
	IsStartDongle = true;
	m_isStop = false;
	IsLoadData = true;
#if PLATFORM_ANDROID
	SetBindTrackedObjectInfo(true);
#endif

}
void FHi5_Manager_Thread::SynCloseConnectEquipment()
{
             //Debug.Log("SynCloseConnectEquipment");
	hi5DongleStop();
	if(m_HI5_GloveStatus.get())
		m_HI5_GloveStatus->ReleaseTimer();
	IsStartDongle = false;
}
//
//         //处理操作指令
void FHi5_Manager_Thread::AsynOperateCalibrationThread(std::shared_ptr<FHI5_Calibration_Command> inParam)
{ 
	if(inParam.get())
	{
		switch (inParam->m_Calibration_CommandType)
		{
		case FHI5_Calibration_Command::ECalibration_Command::EPreCalibration:
			m_Calibrationpercent = 0;
			calibrationPrepare();
			break;
		case FHI5_Calibration_Command::ECalibration_Command::EVCalibration:
			startCalibration(CalibrationPose::GCP_VPose);
			m_Calibrationpercent = 0;
			SetCalibrationState(ECalibrationState::ECalibrationVing);
			break;

		case FHI5_Calibration_Command::ECalibration_Command::EBCalibration:
			startCalibration(CalibrationPose::GCP_BPose);
			m_Calibrationpercent = 0;
			SetCalibrationState(ECalibrationState::ECalibrationBing);
			break;
		case FHI5_Calibration_Command::ECalibration_Command::EPCalibration:
			startCalibration(CalibrationPose::GCP_PPose);
			m_Calibrationpercent = 0;
			SetCalibrationState(ECalibrationState::ECalibrationPing);
			break;
		default:
			break;
		}
	}
}
int FHi5_Manager_Thread::AsyGetClibrationPercent()
{
	int percent = 0;
	switch(_CalibrationState)
	{
		case ECalibrationState::ECalibrationVing:
			
			percent = getCalibratingPercent(_CalibrationPose::GCP_VPose).percent;
			if (percent == 100)
			{
				SetCalibrationState(ECalibrationState::ECalibrationVend);
			}              
			break;
		case ECalibrationState::ECalibrationBing:
			percent = getCalibratingPercent(_CalibrationPose::GCP_BPose).percent;
			if (percent == 100)
			{
				SetCalibrationState(ECalibrationState::ECalibrationBend);
				//hi5_2CalibrationEpilog();
			}
				
			//GEngine->AddOnScreenDebugMessage(-1, 5.f, FColor::Green, FString::Printf(TEXT("%d"), percent));
			break;
		case ECalibrationState::ECalibrationPing:
			percent = getCalibratingPercent(_CalibrationPose::GCP_PPose).percent;
			//percent = FHI5_Calibration::GetCalibrationProgress(EHI5_Pose::PPose);
			if (percent == 100)
			{
				SetCalibrationState(ECalibrationState::ECalibrationPend);
				SaveCalibrationFile("CalibrationData.xml");
				IsCalibrationComplete = true;
			}
			break;
	}
	return percent;
}
void FHi5_Manager_Thread::AsynSendOpticData( std::string sensorIdentity,
                                  OpticalSensorType type,
                                  float fpos[3],
                                  float frot[4])
{
	size_t leng = strlen(sensorIdentity.c_str())+1;
	char* temp =  new char[leng];
	memset(temp,'\0',leng);
	strcpy_s(temp,leng,sensorIdentity.c_str());
	pushOpticalSensorData(temp, type, fpos, frot);
	delete[] temp;
}
void FHi5_Manager_Thread::AsynSendOpticBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType)
{
	size_t leng = strlen(trackerID.c_str())+1;
	char* temp =  new char[leng];
	memset(temp,'\0',leng);
	strcpy_s(temp,leng,trackerID.c_str());
	setOptiSensorBindState(mod, temp,sensorType);
	delete[] temp;
}
void FHi5_Manager_Thread::AsynReceivThreadData()
{
	m_gloveData->bUpdate = 0;
	//读取手姿态数据
	readBVHData( m_gloveData);
	//NetworkManager::GetInstance()->OnLocalPlayerBVHDataReceived((uint8*)&m_gloveData, sizeof(m_gloveData));
	while (m_gloveData->bUpdate == 1 && m_FHi5_BVHDataQuene.get()) 
	{
		m_FHi5_BVHDataQuene->WriteBvhData(m_gloveData);
		m_gloveData->bUpdate = 0;
		readBVHData( m_gloveData);
		
		if (m_gloveData->bUpdate == 0)
			break;
	}
	HI5GloveStatus status;
	readDeviceStatus (&status);
	
	EGloveStatus _newSatus = ConvertHI5GloveStatus(status);
	if (m_HI5_GloveStatus->m_currentStatus != _newSatus)
	{
		m_HI5_GloveStatus->m_currentStatus = _newSatus;
		m_FHi5_status->WriteStatueData(_newSatus);
	}
	m_HI5_GloveStatus->UpdateGloveVisible(_newSatus);
	SensorModulesOfDeviceStatus* SensorModulestatus = new SensorModulesOfDeviceStatus();
	int result = readSensorModulesOfDeviceStatus( SensorModulestatus);
	m_FSensorInforArray->Set(SensorModulestatus);
	delete SensorModulestatus;
}

void __stdcall hi5_TwoDongleStateChangedCallback(void* userData, const DongleIdentity* hid, DongleEvent action)
{
	
	if (action == DongleEvent::DE_Insert)
	{
		
	}
	else if (action == DongleEvent::DE_Removed)
	{
		//清楚数据
		if(FHi5_Manager_Thread::getInstance() != nullptr && (FHi5_Manager_Thread::getInstance()->GetSensorInforArray()).get())
		{
			FHi5_Manager_Thread::getInstance()->GetSensorInforArray()->Reset();
		}
	}
}

void __stdcall hi5_TwofnGloveStateChanged (void* userData, GloveMod glove, GloveEvent event)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		if(FHi5_Manager_Thread::getInstance()->GetMagneticStatus().get())
		{
			FHi5_Manager_Thread::getInstance()->GetMagneticStatus()->OnEventChanged(glove, event);
		}
		if(FHi5_Manager_Thread::getInstance()->GloveStatus().get())
		{
			FHi5_Manager_Thread::getInstance()->GloveStatus()->HI5_GloveStatusOnEventChanged(glove, event);
		}
	}
}

void FHi5_Manager_Thread::RegisterGloveStateChangedCallback()
{
	setGloveStateChangedHandle(nullptr, hi5_TwofnGloveStateChanged);
}

void FHi5_Manager_Thread::RegisterDongleStateChangedHandle()
{
	setDongleStateChangedHandle(nullptr, hi5_TwoDongleStateChangedCallback);
}

int FHi5_Manager_Thread::GetCalibrationpercent() const
{
	
	return m_Calibrationpercent;
}



bool FHi5_Manager_Thread::SaveCalibrationFile(const FString& fileName /*= TEXT("BCP.bin")*/)
{
	return UMotionCaptureFunctionLibrary::SaveCalibrationFile(fileName);
	/*FString filePath;
	if (UMotionCaptureFunctionLibrary::locateUserRoamingFolderTemp (filePath))
	{
		filePath = FPaths::Combine (filePath, fileName);
		return saveCalibrationData (TCHAR_TO_ANSI (*filePath)) != 0;
	}
	else
	{
		return false;
	}*/
}

bool FHi5_Manager_Thread::LoadCalibrationFileTemp(const FString& fileName /*= TEXT("BCP.bin")*/)
{
	return UMotionCaptureFunctionLibrary::LoadCalibrationData();
	/*FString filePath;
	if (UMotionCaptureFunctionLibrary::locateUserRoamingFolderTemp (filePath))
	{
		filePath = FPaths::Combine (filePath, fileName);
		return hi5_loadCalibrationFile (TCHAR_TO_ANSI (*filePath));
	}
	else
	{
		return false;
	}*/
}

EGloveStatus FHi5_Manager_Thread::ConvertHI5GloveStatus(HI5GloveStatus status)
{
	switch (status)
	{
		case HI5GloveStatus::DV_Unknown:
			return EGloveStatus::EUnStatusknown;
		case HI5GloveStatus::DV_NoDongle:
			return EGloveStatus::EGloveStatusNoDongle;
		case HI5GloveStatus::DV_NoGlove:
			return EGloveStatus::EGloveStatusNoGlove;
		case HI5GloveStatus::DV_LeftGloveAvailable:
			return EGloveStatus::EGloveStatusLeftGloveAvailable;
		case HI5GloveStatus::DV_RightGloveAvailable:
			return EGloveStatus::EGloveStatusRightGloveAvailable;
		case HI5GloveStatus::DV_BothGloveAvailable:
			return EGloveStatus::EGloveStatusBothGloveAvailable;
	}
	return EGloveStatus::EUnStatusknown;
}

void FHi5_Manager_Thread::TestOptiLog()
{
	mHi5DataTest.printOpticData();
}

void FHi5_Manager_Thread::TestLeftBvhLog()
{
	mHi5DataTest.printleftBvhData();
}

void FHi5_Manager_Thread::TestRightBvhLog()
{
	mHi5DataTest.printRightBvhData();
}

void FHi5_Manager_Thread::SetCalibrationState(ECalibrationState CalibrationState)
{
	_CalibrationState = CalibrationState;
	if(CalibrationState == ECalibrationState::ECalibrationNone)
		m_Calibrationpercent = 0;
}

void FHi5_Manager_Thread::BindHandInfor()
{
	if (!isRequestBindhand && Isconneted() && IsStartDongle)
	{
#if PLATFORM_WINDOWS
		isRequestBindhand = true;
		FString filename = "OpticalDeviceBindInfo.xml";
		LoadOptiDeviceBindStateFromFile(filename);
		LoadCalibrationFileTemp("CalibrationData.xml");
		///UMotionCaptureFunctionLibrary::LoadOptiDeviceBindStateFromFile(filename);
#endif
	}

}

bool FHi5_Manager_Thread::LoadOptiDeviceBindStateFromFile(const FString& fileName)
{
	FString filepath;
	if (locateUserRoamingFolder (filepath))
	{
		filepath = FPaths::Combine (filepath, fileName);
		if (!FPaths::FileExists (filepath))
		{
			return false;
		}

		MotionCaptureXML::UnrealXML xml_loader;
		void* node_root = xml_loader.LoadXML (filepath);

		xml_loader.SetOperateNodeByTag (_T ("Left"), 0);
		FString leftSN = xml_loader.GetContentByTag (_T ("SerialNumber"));
		FString leftDevTypeStr = xml_loader.GetContentByTag (_T ("DeviceType"));
		OpticalSensorType leftDevType;
		if (leftDevTypeStr == _T ("HTC_VIVE_Tracker"))
		{
			leftDevType = OpticalSensorType::OST_HTC_VIVE_Tracker;
		}
		else if (leftDevTypeStr == _T("HTC_VIVE_Controller"))
		{
			leftDevType = OpticalSensorType::OST_HTC_VIVE_Controller;
		}
		else
		{
			leftDevType = OpticalSensorType::OST_Unknown;
		}
		xml_loader.SetOperateNodeByTag (_T ("Right"), 0, node_root);
		FString rightSN = xml_loader.GetContentByTag (_T ("SerialNumber"));
		FString rightDevTypeStr = xml_loader.GetContentByTag (_T ("DeviceType"));
		OpticalSensorType rightDevType;
		if (rightDevTypeStr == _T ("HTC_VIVE_Tracker"))
		{
			rightDevType = OpticalSensorType::OST_HTC_VIVE_Tracker;
		}
		else if (rightDevTypeStr == _T ("HTC_VIVE_Controller"))
		{
			rightDevType = OpticalSensorType::OST_HTC_VIVE_Controller;
		}
		else {
			rightDevType = OpticalSensorType::OST_Unknown;
		}
		std::string left(TCHAR_TO_UTF8(*leftSN));
		std::string right(TCHAR_TO_UTF8(*rightSN));
	
	//	left.erase(std::remove(left.begin(), left.end(), '\"'), left.end());
	//	right.erase(std::remove(right.begin(), right.end(), '\"'), right.end());
		//str = "string:-name:-sponge-"
		// 执行之后结果：
		//str = "string:name:sponge"

	/*	std::string a1 = "-";
		std::string a2 = "_";*/
	/*	left =replace_str(left,a2 ,a1);
		right =replace_str(right,a2 ,a1);*/
		AddOptiSensorBindData(GloveMod::GM_LeftGlove,
			left, OpticalSensorType::OST_HTC_VIVE_Tracker);
		AddOptiSensorBindData(GloveMod::GM_RightGlove,
			right, OpticalSensorType::OST_HTC_VIVE_Tracker);

		//gGloveStatus[GloveMod::GM_LeftGlove] =
		//hi5_loadTrackerBindState (true, leftSN, leftDevType);
		//hi5_loadTrackerBindState (false, rightSN, rightDevType);

		/*TArray<int32> OutTrackedDeviceIds, OutControllerIds;
		USteamVRFunctionLibrary::GetValidTrackedDeviceIds (ESteamVRTrackedDeviceType::Invalid, OutTrackedDeviceIds);
		USteamVRFunctionLibrary::GetValidTrackedDeviceIds (ESteamVRTrackedDeviceType::Controller, OutControllerIds);
		OutTrackedDeviceIds.Append (OutControllerIds);

		for (int32& id : OutTrackedDeviceIds) {
			FString sn;
			if (GetViveTrackerSN (id, sn)) {
				if (sn == leftSN) {
					SetLeftViveTrackerId (id);
				}
				if (sn == rightSN) {
					SetRightViveTrackerId (id);
				}
			}
		}*/

		return true;
	}
	else
	{
		return false;
	}
}





