// Fill out your copyright notice in the Description page of Project Settings.


#include "Hi5_Thread_Data.h"
#include "Hi5Pawn.h"
#include "CoreMinimal.h"
#include "Hi5_Manager_Thread.h"
#include "Kismet/KismetSystemLibrary.h"
#include "Runtime/Engine/Classes/Kismet/GameplayStatics.h"
#include "UnrealXML.h"
#include "HI5Glove.h"
extern bool locateUserRoamingFolder(FString& path);
/*FSenserInfor*/
FSenserInfor::FSenserInfor()
{
	m_magneticValue = 0;
	m_energyValue = 0;
	m_signalValue = 0;
	m_online = 0;

}

FSenserInfor::~FSenserInfor()
{

}

void FSenserInfor::SetInfor(unsigned int magneticValue,
	unsigned int energyValue,
	unsigned int signalValue,
	int online)
{

	m_magneticValue = magneticValue;
	m_energyValue = energyValue;
	m_signalValue = signalValue;
	m_online = online;
}

void FSenserInfor::Reset()
{
	
	m_magneticValue = 0;
	m_energyValue = 0;
	m_signalValue = 0;
	m_online = 0;
}

/*FSensorInforArray*/
FSensorInforArray* FSensorInforArray::Create()
{
	FSensorInforArray* pSelf = new FSensorInforArray();
	pSelf->Init();
	return pSelf;
}

FSensorInforArray::FSensorInforArray()
{
	m_pSensors = nullptr;
}

FSensorInforArray::~FSensorInforArray()
{
	Release();
}

void FSensorInforArray::Init()
{
	if (m_pSensors == nullptr)
	{
		std::lock_guard<std::mutex> lck(m_mutex);
		m_pSensors = new std::vector<std::shared_ptr<FSenserInfor>>();
		int index = 12;
		while (index > 0)
		{
			m_pSensors->push_back(std::make_shared<FSenserInfor>());
			index--;
		}
	}
}

void FSensorInforArray::Release()
{
	std::lock_guard<std::mutex> lck(m_mutex);
	if (m_pSensors != nullptr)
	{
		delete m_pSensors;
		m_pSensors = nullptr;
	}
	
}

void FSensorInforArray::Reset()
{
	if (m_pSensors != nullptr)
	{
		std::lock_guard<std::mutex> lck(m_mutex);
		for (size_t i = 0; i < m_pSensors->size(); i++)
		{
			std::shared_ptr<FSenserInfor>& InRenderingData = (*m_pSensors)[i];
			if(InRenderingData.get())
				InRenderingData->Reset();
		}
	}
}

void FSensorInforArray::Set(SensorModulesOfDeviceStatus* SensorModulestatus)
{
	if (m_pSensors != nullptr && SensorModulestatus != nullptr)
	{
		std::lock_guard<std::mutex> lck(m_mutex);
		for (size_t i = 0; i < m_pSensors->size(); i++)
		{
			std::shared_ptr<FSenserInfor>& InRenderingData = (*m_pSensors)[i];
			if(InRenderingData.get())
			{
				InRenderingData->SetInfor(SensorModulestatus->sensorModuleStatus[i].magneticValue,
							SensorModulestatus->sensorModuleStatus[i].energyValue,
							SensorModulestatus->sensorModuleStatus[i].signalValue,
							SensorModulestatus->sensorModuleStatus[i].online);
			}
		}
	}
}

bool FSensorInforArray::GetItem(int index,FSenserInfor& outValue)
{
	if (m_pSensors != nullptr && index <12) 
	{
		std::lock_guard<std::mutex> lck(m_mutex);
		std::shared_ptr<FSenserInfor>& OutRenderingData = (*m_pSensors)[index];
		outValue.SetInfor(*OutRenderingData);
		return true;
	}
	else
	{
		return false;
	}
}

void FSensorInforArray::SetSensorInforArray(FSenserItemData* dataItem)
{
	if(dataItem != nullptr)
	{
		std::lock_guard<std::mutex> lck(m_mutex);
		for(int i=0;i<12;i++)
		{
			if (m_pSensors != nullptr )
			{
				std::shared_ptr<FSenserInfor>& OutRenderingData = (*m_pSensors)[i];
				dataItem[i].SetFSenserItemData(OutRenderingData->GetMagneticValue(),
					OutRenderingData->GetEnergyValue(),
					OutRenderingData->GetSignalValue(),
					OutRenderingData->GetOnline());
			}
		}
	}
}

/*FOptiSensorBindData*/
FOptiSensorBindData::FOptiSensorBindData()
{
	 m_mod = _GloveMod::GM_Unknown;
	 m_trackerID = "";
	 m_sensorType = _OpticalSensorType::OST_Unknown;
}

std::shared_ptr<FOptiSensorBindData> FOptiSensorBindData::Creat(GloveMod mod, std::string trackerID, OpticalSensorType sensorType)
{
	std::shared_ptr<FOptiSensorBindData> item = std::make_shared<FOptiSensorBindData>();
	item->m_mod = mod;
	item->m_trackerID = trackerID;
	item->m_sensorType = sensorType;
	return item;
}

void FOptiSensorBindDataArray::Push(std::shared_ptr<FOptiSensorBindData> inValue)
{
	if(inValue.get())
		data_queue.push(inValue);
}

std::shared_ptr<FOptiSensorBindData> FOptiSensorBindDataArray::Pop()
{
	if(!data_queue.empty())
	{
		std::shared_ptr<FOptiSensorBindData> res(std::shared_ptr<FOptiSensorBindData>(data_queue.front()));
		data_queue.pop();
		return res;
	}
	else
	{
		return nullptr;
	}
}

bool FOptiSensorBindDataArray::Empty() const
{
	return data_queue.empty();
}

void FOptiSensorBindDataArray::Clean()
{
	while (!data_queue.empty())
	{
		data_queue.pop();
	}
}


bool operator <(const FHI5_Calibration_Command& l, const FHI5_Calibration_Command& r)
{
	int lValve = (int)l.m_Calibration_CommandType;
	int rValve = (int)r.m_Calibration_CommandType;
	if (lValve != rValve)
	{
		return lValve < rValve;
	}
	return false;
}

bool operator <(const FHI5_Operate_Command& l, const FHI5_Operate_Command& r)
{
	int lValve = (int)l.m_Operate_CommandType;
	int rValve = (int)r.m_Operate_CommandType;
	if (lValve != rValve)
	{
		return lValve < rValve;
	}
	return false;

}

FHI5_Optic_DataArray::FHI5_Optic_DataArray()
{
	
}

void FHI5_Optic_DataArray::Push(std::shared_ptr<FHI5_Optic_Data> inValue)
{
	if(inValue.get())
		data_queue.push(inValue);
}

std::shared_ptr<FHI5_Optic_Data> FHI5_Optic_DataArray::Pop()
{
	if(!data_queue.empty())
	{
		std::shared_ptr<FHI5_Optic_Data> res(std::shared_ptr<FHI5_Optic_Data>(data_queue.front()));
		data_queue.pop();
		return res;
	}
	else
	{
		return nullptr;
	}
}

bool FHI5_Optic_DataArray::Empty() const
{
	return data_queue.empty();
}

void FHI5_Optic_DataArray::Clean()
{
	while (!data_queue.empty())
	{
		data_queue.pop();
	}
}

void FHI5_Calibration_Command_Data::Push(std::shared_ptr<FHI5_Calibration_Command> inValue)
{
	if(inValue.get())
		_CalibrationQue.push(inValue);
}

std::shared_ptr<FHI5_Calibration_Command> FHI5_Calibration_Command_Data::Pop()
{
	if(!_CalibrationQue.empty())
	{
		std::shared_ptr<FHI5_Calibration_Command> res(_CalibrationQue.front());
		_CalibrationQue.pop();
		return res;
	}
	else
	{
		return nullptr;
	}
}

bool FHI5_Calibration_Command_Data::Empty() const
{
	return _CalibrationQue.empty();
}

void FHI5_Calibration_Command_Data::Clean()
{
	while (!_CalibrationQue.empty())
	{
		_CalibrationQue.pop();
	}
}

FHI5_Operate_Base_Content::FHI5_Operate_Base_Content()
{
	m_HI5_Optic_DataArray = std::make_shared<FHI5_Optic_DataArray>();
	m_OptiSensorBindDataArray = std::make_shared<FOptiSensorBindDataArray>();
	m_Calibration_Send_Data = std::make_shared<FHI5_Calibration_Command_Data>();

}
std::shared_ptr<FHI5_Optic_DataArray> FHI5_Operate_Base_Content::GetHI5_Optic_DataArray()
{
	if(m_HI5_Optic_DataArray.get())
		return m_HI5_Optic_DataArray;
	else
		return nullptr;
}

std::shared_ptr<FOptiSensorBindDataArray> FHI5_Operate_Base_Content::GetOptiSensorBindDataArray()
{
	if(m_OptiSensorBindDataArray.get())
		return m_OptiSensorBindDataArray;
	else
		return nullptr;
}

std::shared_ptr<FHI5_Calibration_Command_Data> FHI5_Operate_Base_Content::GetCalibration_Send_Data()
{
	if(m_Calibration_Send_Data.get())
		return m_Calibration_Send_Data;
	else
		return nullptr;
}

FHI5_Operate_Data::FHI5_Operate_Data()
{
	m_Data = std::make_shared<FHI5_Operate_Base_Content>();
}

std::shared_ptr<FHI5_Operate_Base_Content> FHI5_Operate_Data::Get_Data()
{
	if(m_Data.get())
		return m_Data;
	else
		return nullptr;
}

FHI5_Operate_Command::EOperate_Command FHI5_Operate_Data::Get_Command()
{
	return m_Command;
}

void FHI5_Operate_Data::SetCommand(FHI5_Operate_Command::EOperate_Command inCommand)
{
	m_Command = inCommand;
}

/*FHI5_Calibration_Send_Data*/
FHI5_Calibration_Send_Data::FHI5_Calibration_Send_Data()
{
	init();
}

void FHI5_Calibration_Send_Data::clean()
{
	std::lock_guard<std::mutex> lck(mut);
	m_commandDir.clear();
}

void FHI5_Calibration_Send_Data::init()
{
	std::lock_guard<std::mutex> lck(mut);
	m_commandDir.clear();
}

std::shared_ptr<FHI5_Operate_Data> FHI5_Calibration_Send_Data::GetCommand(FHI5_Operate_Command inKeyComamnd)
{
	std::lock_guard<std::mutex> lck(mut);
	std::shared_ptr<FHI5_Operate_Data> temp = nullptr;
	if (m_commandDir.find(inKeyComamnd) != m_commandDir.end())
	{
		temp = m_commandDir[inKeyComamnd];
		m_commandDir.erase(inKeyComamnd);
	}
	return temp;
}

std::shared_ptr<FHI5_Operate_Data> FHI5_Calibration_Send_Data::GetConnectEquipmentCommand()
{
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::EConnectEquipment;
	return GetCommand(keyComamnd);
}

std::shared_ptr<FHI5_Operate_Data> FHI5_Calibration_Send_Data::GetCloseConnectEquipmentCommand()
{
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::ECloseConnectEquipment;
	return GetCommand(keyComamnd);
}

std::shared_ptr<FHI5_Optic_DataArray> FHI5_Calibration_Send_Data::GetSendOpticData()
{
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::ESendOpticData;
	std::shared_ptr<FHI5_Operate_Data> item  = GetCommand( keyComamnd);
	if(item.get() && item->Get_Data().get() && item->Get_Data()-> GetHI5_Optic_DataArray().get())
	{
		std::shared_ptr<FHI5_Optic_DataArray> item_data = item->Get_Data()-> GetHI5_Optic_DataArray();
		return item_data;
	}
	else
	{
		return nullptr;
	}
}

std::shared_ptr<FHI5_Calibration_Command_Data> FHI5_Calibration_Send_Data::GetSendCalibration()
{
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::ECalibrationOper;
	std::shared_ptr<FHI5_Operate_Data> item  = GetCommand( keyComamnd);
	if(item.get() && item->Get_Data().get() && item->Get_Data()-> GetCalibration_Send_Data().get())
	{
		std::shared_ptr<FHI5_Calibration_Command_Data> item_data = item->Get_Data()-> GetCalibration_Send_Data();
		return item_data;
	}
	else
	{
		return nullptr;
	}
	
}

std::shared_ptr<FOptiSensorBindDataArray> FHI5_Calibration_Send_Data::GetSenserBindOpticData()
{
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::EOptiSensorBindData;
	std::shared_ptr<FHI5_Operate_Data> item  = GetCommand( keyComamnd);
	if(item.get() && item->Get_Data().get() && item->Get_Data()-> GetOptiSensorBindDataArray().get())
	{
		std::shared_ptr<FOptiSensorBindDataArray> item_data = item->Get_Data()-> GetOptiSensorBindDataArray();
		return item_data;
	}
	else
	{
		return nullptr;
	}
}

void FHI5_Calibration_Send_Data::AddOpticData(std::string sensorIdentity, OpticalSensorType type, float fpos[3], float frot[4])
{
	std::lock_guard<std::mutex> lck(mut);
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::ESendOpticData;
	if (m_commandDir.find(keyComamnd) != m_commandDir.end())
	{
		std::shared_ptr<FHI5_Operate_Data> item = m_commandDir[keyComamnd];
		if(item.get())
		{
			std::shared_ptr<FHI5_Operate_Base_Content> content = item->Get_Data();
			if(content.get())
			{
				std::shared_ptr<FHI5_Optic_DataArray> _SendData = content->GetHI5_Optic_DataArray();
				std::shared_ptr<FHI5_Optic_Data> temp = FHI5_Optic_Data::Creat(sensorIdentity, type, fpos, frot);
				if(FHi5_Manager_Thread::getInstance() != nullptr)
				{
					FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeOptiData(sensorIdentity,fpos,frot);
				}
				if(_SendData.get())
					_SendData->Push(temp);
			}
		}
	}
	else
	{
		std::shared_ptr<FHI5_Operate_Data> item = std::make_shared<FHI5_Operate_Data>();
		if(item != nullptr)
		{
			std::shared_ptr<FHI5_Operate_Base_Content> content = item->Get_Data();
			item->SetCommand(FHI5_Operate_Command::EOperate_Command::ECalibrationOper);
			if(content.get())
			{
				std::shared_ptr<FHI5_Optic_DataArray> _SendData = content->GetHI5_Optic_DataArray();
				std::shared_ptr<FHI5_Optic_Data> temp = FHI5_Optic_Data::Creat(sensorIdentity, type, fpos, frot);
				if(FHi5_Manager_Thread::getInstance() != nullptr)
				{
					FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeOptiData(sensorIdentity,fpos,frot);
				}
				_SendData->Push(temp);
			}
		}
		m_commandDir.insert(std::pair<FHI5_Operate_Command,std::shared_ptr<FHI5_Operate_Data>>(keyComamnd,item));
	}
}

void FHI5_Calibration_Send_Data::AddOptiSensorBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType)
{
	std::lock_guard<std::mutex> lck(mut);
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::EOptiSensorBindData;
	if (m_commandDir.find(keyComamnd) != m_commandDir.end())
	{
		std::shared_ptr<FHI5_Operate_Data> item = m_commandDir[keyComamnd];
		if(item.get())
		{
			std::shared_ptr<FHI5_Operate_Base_Content> content = item->Get_Data();
			if(content.get())
			{
				std::shared_ptr<FOptiSensorBindDataArray> _SendData = content->GetOptiSensorBindDataArray();
				std::shared_ptr<FOptiSensorBindData> temp = FOptiSensorBindData::Creat(mod, trackerID, sensorType);
				OpticalSensorType data1 = temp->m_sensorType;
				std::string data2 = temp->m_trackerID;
				if(temp->m_mod == GloveMod::GM_LeftGlove)
				{
					if(FHi5_Manager_Thread::getInstance() != nullptr)
					{
						FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeBindOptic(true,trackerID);
					}		
				}
				if(temp->m_mod == GloveMod::GM_RightGlove)
				{
					if(FHi5_Manager_Thread::getInstance() != nullptr)
					{
						FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeBindOptic(false,trackerID);
					}		
				}
				_SendData->Push(temp);
			}
		}
	}
	else
	{
		std::shared_ptr<FHI5_Operate_Data>  item = std::make_shared<FHI5_Operate_Data>();
		if(item.get())
		{
			std::shared_ptr<FHI5_Operate_Base_Content>  content = item->Get_Data();
			item->SetCommand(FHI5_Operate_Command::EOperate_Command::EOptiSensorBindData);
			if(content.get())
			{
				std::shared_ptr<FOptiSensorBindDataArray> _SendData = content->GetOptiSensorBindDataArray();
				std::shared_ptr<FOptiSensorBindData> temp = FOptiSensorBindData::Creat(mod, trackerID, sensorType);
				OpticalSensorType data1 = temp->m_sensorType;
				std::string data2 = temp->m_trackerID;
				if(temp->m_mod == GloveMod::GM_LeftGlove)
				{
					if(FHi5_Manager_Thread::getInstance() != nullptr)
					{
						FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeBindOptic(true,trackerID);
					}		
				}
				if(temp->m_mod == GloveMod::GM_RightGlove)
				{
					if(FHi5_Manager_Thread::getInstance() != nullptr)
					{
						FHi5_Manager_Thread::getInstance()->mHi5DataTest.writeBindOptic(false,trackerID);
					}		
				}
				_SendData->Push(temp);
			}
		}
		m_commandDir.insert(std::pair<FHI5_Operate_Command,std::shared_ptr<FHI5_Operate_Data>>(keyComamnd,item));
	}
}
	
void FHI5_Calibration_Send_Data::AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command param)
{
	std::lock_guard<std::mutex> lck(mut);
	FHI5_Operate_Command keyComamnd;
	keyComamnd.m_Operate_CommandType = FHI5_Operate_Command::EOperate_Command::ECalibrationOper;
	if (m_commandDir.find(keyComamnd) != m_commandDir.end())
	{
		std::shared_ptr<FHI5_Operate_Data> item = m_commandDir[keyComamnd];
		if(item.get())
		{
			std::shared_ptr<FHI5_Operate_Base_Content>  content = item->Get_Data();
			if(content.get())
			{
				std::shared_ptr<FHI5_Calibration_Command_Data>  _SendData = content->GetCalibration_Send_Data();
				FHI5_Calibration_Command* _commandItem = new FHI5_Calibration_Command(param);
				std::shared_ptr<FHI5_Calibration_Command> ptr(_commandItem);
				_SendData->Push(ptr);
			}
		}
	}
	else
	{
		std::shared_ptr<FHI5_Operate_Data> item = std::make_shared<FHI5_Operate_Data>();;
		if(item.get())
		{
			std::shared_ptr<FHI5_Operate_Base_Content>  content = item->Get_Data();
			item->SetCommand(FHI5_Operate_Command::EOperate_Command::ECalibrationOper);
			if(content != nullptr)
			{
				std::shared_ptr<FHI5_Calibration_Command_Data>  _SendData = content->GetCalibration_Send_Data();
				FHI5_Calibration_Command* _commandItem = new FHI5_Calibration_Command(param);
				std::shared_ptr<FHI5_Calibration_Command> ptr(_commandItem);
				if(_SendData.get())
				{
					_SendData->Push(ptr);
					m_commandDir.insert(std::pair<FHI5_Operate_Command,std::shared_ptr<FHI5_Operate_Data>>(keyComamnd,item));
				}
				
			}
		}
	}
}

FVive_Tracker_Infor_Map::FVive_Tracker_Infor_Map()
{
	m_LeftTrackerIndex = -1;
	m_RightTrackerIndex = -1;
	m_trackermap.Empty();
}

void FVive_Tracker_Infor_Map::AddItem(FVive_Tracker_Infor& inparam)
{
	if(!m_trackermap.Contains(inparam.trackerId))
	{
		m_trackermap.Add(inparam.trackerId,inparam);
	}
}

bool FVive_Tracker_Infor_Map::IsContain(int inIndex)
{
	return m_trackermap.Contains(inIndex);
}

void FVive_Tracker_Infor_Map::SetPosAndQua(int inIndex,FVector inPos, FRotator inQua)
{
	if(m_trackermap.Contains(inIndex))
	{
		m_trackermap[inIndex].position = inPos;
		m_trackermap[inIndex].rotation = inQua;
	}
}

bool FVive_Tracker_Infor_Map::GetPosAndQua(int inIndex,FVector& outPos, FRotator& outQua)
{
	if(m_trackermap.Contains(inIndex))
	{
		outPos = m_trackermap[inIndex].position;
		outQua = m_trackermap[inIndex].rotation;
		return true;
	}
	else
	{
		return false;
	}
}
std::string& replace_str(std::string& str, const std::string& to_replaced, const std::string& newchars)     
{
	for(std::string::size_type pos(0); pos != std::string::npos; pos += newchars.length())   
	{
			pos = str.find(to_replaced,pos);
			if(pos!=std::string::npos)     
				str.replace(pos,to_replaced.length(),newchars);     
			else  
				 break;
	}     
	return   str;     
}
bool FVive_Tracker_Infor_Map::FindTwoDevices(APawn* inPawn)
{
	FString _Device1 ="";
	FString _Device2 = "";

	float _distance = 100.0f;
	int device1 = -1;
	int device2 = -2;
	FVive_Tracker_Infor deviceTracker1;
	FVive_Tracker_Infor deviceTracker2;
	TArray<FVive_Tracker_Infor> arrays;
	for (auto It = m_trackermap.CreateConstIterator(); It; ++It)
	{
		arrays.Add(It.Value());
	}
	
	for (int i = 0; i < arrays.Num(); i++)
	{
		int j = i+1;
		FVive_Tracker_Infor itemtemp = arrays[i];
		for(; j< arrays.Num(); j++)
		{
			// vive空间
			float tempHeight = abs(arrays[i].position.Y - arrays[j].position.Y);
			{
				FVive_Tracker_Infor temp = arrays[j];
				float dis = FVector::Distance(itemtemp.position, temp.position);
				if (_distance > dis)
				{
					_Device1 = itemtemp.trackerIdentity;
					_Device2 = temp.trackerIdentity;
					deviceTracker1 = itemtemp ;
					deviceTracker2 = temp;
					device1 = itemtemp.trackerId;
					device2 = temp.trackerId;
				}
			}                      
		}
	}
	
	// UE4笔记---获取 PlayerController 对象
	APlayerController* PlayerController = UGameplayStatics::GetPlayerController(GWorld, 0);
	if(PlayerController == nullptr)
	{
		int a =10;
	}
	//APawn* myPawn1 = GetWorld()->GetFirstPlayerController()->GetPawn();
	//ATutorialsPlayerController* TutorialsPlayerController = Cast<ATutorialsPlayerController>(PlayerController);
	
	////UGameplayStatics::Get
	FVector temp = deviceTracker2.position - deviceTracker1.position;
	//if(GEngine->GetWorld() != nullptr)
	//{
	//	if(GEngine->GetWorld()->GetFirstPlayerController() != nullptr)
	//	{
	//		if(GEngine->GetWorld()->GetFirstPlayerController()->GetPawn() != nullptr)
	//		{
	//			int a = 10;
	//		}
	//	}
	//}
	APawn* myPawn = PlayerController->GetPawn();
	if(myPawn != nullptr)
	{
		AHi5Pawn* hi5Pawn = Cast<AHi5Pawn>(myPawn);
		if(hi5Pawn != nullptr)
		{
			UCameraComponent* cameraComponent = hi5Pawn->GetCameraComponentImplementableEvent();
			if(cameraComponent != nullptr)
			{
				if (FVector::DotProduct(cameraComponent->GetRightVector(), temp) > 0.0f)
				{
					m_LeftTrackerIndex = device1;
					m_LeftTrackerIdentity = deviceTracker1.trackerIdentity;
					m_RightTrackerIndex = device2;
					m_RightTrackerIdentity  = deviceTracker2.trackerIdentity;
				}
				else
				{
					m_LeftTrackerIndex = device2;
					m_RightTrackerIndex = device1;
					m_LeftTrackerIdentity = deviceTracker2.trackerIdentity;
					m_RightTrackerIdentity  = deviceTracker1.trackerIdentity;
				}
			}
		}
	}
	
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		std::string LeftTrackertemp(TCHAR_TO_UTF8(*m_LeftTrackerIdentity));
		std::string RightTrackerTemp(TCHAR_TO_UTF8(*m_RightTrackerIdentity));
		std::string  LeftTracker = LeftTrackertemp;//"\"" + LeftTrackertemp + "\"";
		std::string  RightTracker = RightTrackerTemp;//"\"" + RightTrackerTemp + "\"";
		/*std::string a1 = "-";
		std::string a2 = "_";
		LeftTracker =replace_str(LeftTracker,a1 ,a2);
		RightTracker =replace_str(RightTracker,a1 ,a2);*/
		hi5_2CalibrationEpilog(FString(LeftTrackertemp.c_str()), FString(RightTrackerTemp.c_str()));
		FHi5_Manager_Thread::getInstance()->AddOptiSensorBindData(GloveMod::GM_LeftGlove,
			LeftTrackertemp, OpticalSensorType::OST_HTC_VIVE_Tracker);
		FHi5_Manager_Thread::getInstance()->AddOptiSensorBindData(GloveMod::GM_RightGlove,
			RightTrackerTemp, OpticalSensorType::OST_HTC_VIVE_Tracker);
		FString FileName = TEXT ("OpticalDeviceBindInfo.xml");
		SaveOptiDeviceBindStateToFile(FileName,FString(LeftTracker.c_str()),FString(RightTracker.c_str()));
	}
	//
	return true;
}
bool FVive_Tracker_Infor_Map::SaveOptiDeviceBindStateToFile(const FString& fileName,FString leftSN,FString rightSN)
{
	FString filepath;
	if (locateUserRoamingFolder (filepath))
	{
		filepath = FPaths::Combine (filepath, fileName);

		MotionCaptureXML::UnrealXML xml_saver;
		if (FPaths::FileExists (filepath))
		{
			xml_saver.LoadXML (filepath);
		}
		else
		{
			xml_saver.UseXMLString (_T ("<HI5_BindInfo></HI5_BindInfo>"));
		}
		//OpticalSensorType leftDevType, rightDevType;
		FString leftDevTypeStr, rightDevTypeStr;
		leftDevTypeStr = "HTC_VIVE_Tracker";
		rightDevTypeStr  = "HTC_VIVE_Tracker";
		{
			void* node_root = xml_saver.GetOperateNode ();
			void* node_left = xml_saver.SetOperateNodeByTag (_T ("Left"), 0);
			xml_saver.SetOperateNodeByTag (_T ("Type"), 0);
			xml_saver.SetContent (_T ("LEFT"));
			xml_saver.SetOperateNodeByTag (_T ("SerialNumber"), 0, node_left);
			xml_saver.SetContent (leftSN);
			xml_saver.SetOperateNodeByTag (_T ("DeviceType"), 0, node_left);
			xml_saver.SetContent (leftDevTypeStr);

			void* node_right = xml_saver.SetOperateNodeByTag (_T ("Right"), 0, node_root);
			xml_saver.SetOperateNodeByTag (_T ("Type"), 0);
			xml_saver.SetContent (_T ("RIGHT"));
			xml_saver.SetOperateNodeByTag (_T ("SerialNumber"), 0, node_right);
			xml_saver.SetContent (rightSN);
			xml_saver.SetOperateNodeByTag (_T ("DeviceType"), 0, node_right);
			xml_saver.SetContent (rightDevTypeStr);
		}
		return xml_saver.SaveXML (filepath);
	}
	else
	{
		return false;
	}
}
/*FHI5_Calibration_Send_Data*/
// FHi5_Thread_Data::FHi5_Thread_Data()
// {
// }
//
// FHi5_Thread_Data::~FHi5_Thread_Data()
// {
// }
