// Fill out your copyright notice in the Description page of Project Settings.

#pragma once
#include "CoreMinimal.h"
#include "mutex"
#include <map>
#include <queue>
#include "HI5Glove/HI5Glove_C.h"
#include "Hi5GloveDataType.h"
//#include "Hi5Glove_C.h"
using namespace HI5Device;
// template<typename T>
// class Hi5_ThreadSafe_Queue
// {
// private:
// 	mutable  std::mutex mut;
// 	std::queue<std::shared_ptr<T>> data_queue;
// 	std::condition_variable data_cond;
// public:
// 	Hi5_ThreadSafe_Queue(){}
// 	virtual ~Hi5_ThreadSafe_Queue(){}
// 	void Push(std::shared_ptr <T> inValue)
// 	{
// 		std::unique_lock<std::mutex> lk(mut);
// 		data_queue.push(inValue);
// 		data_cond.notify_one();
// 	}
// 	void Pop(std::list<std::shared_ptr<T>>& PopArray)
// 	{
// 		std::unique_lock<std::mutex> lk(mut);
// 		data_cond.wait(lk,[this]{return !data_queue.empty();});
// 		while(!data_queue.empty())
// 		{
// 			std::shared_ptr<T> res(std::shared_ptr<T>(data_queue.front()));
// 			PopArray.push_back(res);
// 			data_queue.pop();
// 		}
// 	}
// 	bool Empty() const
// 	{
// 		std::lock_guard<std::mutex> lk(mut);
// 		return data_queue.empty();
// 	}
// 	void Clean()
// 	{
// 		std::unique_lock<std::mutex> lk(mut);
// 		while (!data_queue.empty())
// 		{
// 			data_queue.pop();
// 		}
// 	}
// };

struct FSenserItemData
{
	FSenserItemData()
	{
		m_magneticValue = 0;
		m_energyValue = 0;
		m_signalValue = 0;
		m_online = 0;
	}
	void SetFSenserItemData(int magneticValue,
						int energyValue,
						int signalValue,
						int online)
	{
		m_magneticValue = magneticValue;
		m_energyValue = energyValue;
		m_signalValue = signalValue;
		m_online = online;
	}
	int m_magneticValue;  // best : 100
	int m_energyValue;    // best : 100
	int m_signalValue;    // best : 100
	int m_online;         // 1 : online
};
//传感器信息
class MOTIONCAPTURERUNTIME_API  FSenserInfor : public std::enable_shared_from_this<FSenserInfor>  
{
public:
	FSenserInfor();
	virtual ~FSenserInfor();
	void SetInfor(unsigned int magneticValue,
		unsigned int energyValue,
		unsigned int signalValue,
		int online);
	void SetInfor( FSenserInfor& inValue)
	{
		m_magneticValue = inValue.m_magneticValue; 
		m_energyValue = inValue.m_energyValue;    
		m_signalValue = inValue.m_signalValue;   
		m_online = inValue.m_online;      
	}
	void GetInfor(unsigned int& outMagneticValue,
				unsigned int& outEnergyValue,
				unsigned int& outSingalValue)
	{
		outMagneticValue = m_magneticValue;
		outEnergyValue =  m_energyValue;
		outSingalValue = m_signalValue;
 	}
	void Reset();
	 int GetMagneticValue() const
	{
		return m_magneticValue;
	}
	 int GetEnergyValue() const
	{
		return m_energyValue;
	}
	 int GetSignalValue() const
	{
		return m_signalValue;
	}
	 int GetOnline() const
	{
		return m_online;
	}
protected:
	unsigned int m_magneticValue;  // best : 100
	unsigned int m_energyValue;    // best : 100
	unsigned int m_signalValue;    // best : 100
	int m_online;         // 1 : online
};
class MOTIONCAPTURERUNTIME_API FSensorInforArray : public std::enable_shared_from_this<FSensorInforArray>  
{
public:
	static FSensorInforArray* Create();
	FSensorInforArray();
	virtual ~FSensorInforArray();
	void Init();
	void Release();
	void Reset();
	void Set(SensorModulesOfDeviceStatus* SensorModulestatus);
	bool GetItem(int index,FSenserInfor& outValue);
	void SetSensorInforArray(FSenserItemData* dataItem);
	
protected:
	std::mutex					  m_mutex;
	std::vector< std::shared_ptr<FSenserInfor>>* m_pSensors;
};

//光学绑定手腕信息
class MOTIONCAPTURERUNTIME_API  FOptiSensorBindData : public std::enable_shared_from_this<FOptiSensorBindData>
{
public:
	FOptiSensorBindData();
	static std::shared_ptr<FOptiSensorBindData> Creat(GloveMod mod, std::string trackerID,OpticalSensorType sensorType);
	GloveMod m_mod;
	std::string m_trackerID;
	OpticalSensorType m_sensorType;
 };

class MOTIONCAPTURERUNTIME_API FOptiSensorBindDataArray  : public std::enable_shared_from_this<FOptiSensorBindDataArray>
{
 public:
 	FOptiSensorBindDataArray()
 	{
 		
 	}
 	void Push(std::shared_ptr <FOptiSensorBindData> inValue);
 	std::shared_ptr<FOptiSensorBindData> Pop();
 	bool Empty() const;
 	void Clean();
	
 protected:
 	std::queue<std::shared_ptr<FOptiSensorBindData>> data_queue;
};

//光学数据
class MOTIONCAPTURERUNTIME_API FHI5_Optic_Data  : public std::enable_shared_from_this<FHI5_Optic_Data>
{
public:
	std::string m_sensorIdentity;
	 OpticalSensorType m_type;
	 float m_fpos[3];
	 float m_frot[4];
	
	FHI5_Optic_Data()
	{
		m_sensorIdentity ="";
		m_type = OpticalSensorType::OST_Unknown;
		m_fpos[0] = 0.0f;
		m_fpos[1] = 0.0f;
		m_fpos[2] = 0.0f;
		m_frot[0] = 0.0f;
		m_frot[1] = 0.0f;
		m_frot[2] = 0.0f;
		m_frot[3] = 0.0f;
	}
	
	static std::shared_ptr<FHI5_Optic_Data> Creat(std::string sensorIdentity,
		OpticalSensorType type,
		float fpos[3],
		float frot[4])
	{
		std::shared_ptr<FHI5_Optic_Data> item = std::make_shared<FHI5_Optic_Data>();
		item->m_sensorIdentity = sensorIdentity;
		item->m_type = type;
		item->m_fpos[0] = fpos[0];
		item->m_fpos[1] = fpos[1];
		item->m_fpos[2] = fpos[2];
		item->m_frot[0] = frot[0];
		item->m_frot[1] = frot[1];
		item->m_frot[2] = frot[2];
		item->m_frot[3] = frot[3];
		return item;
	}
};
class MOTIONCAPTURERUNTIME_API FHI5_Optic_DataArray : public std::enable_shared_from_this<FHI5_Optic_DataArray>
{
public:
	FHI5_Optic_DataArray();
	void Push(std::shared_ptr <FHI5_Optic_Data> inValue);
	std::shared_ptr<FHI5_Optic_Data> Pop();
	bool Empty() const;
	void Clean();
protected:
	std::queue<std::shared_ptr<FHI5_Optic_Data>> data_queue;
};

/*************************校准指令****************************************/
class MOTIONCAPTURERUNTIME_API FHI5_Calibration_Command : public std::enable_shared_from_this<FHI5_Calibration_Command>
{
public:
	enum ECalibration_Command
	{
		ENon = 0,
		EPreCalibration = 1,     //Pre calibration
		EVCalibration = 2,      //V pos
		EBCalibration = 3,      //B pos
		EPCalibration = 4,     //P pos
	};
	FHI5_Calibration_Command(ECalibration_Command inValue)
	{
		m_Calibration_CommandType = inValue;
	}
	ECalibration_Command m_Calibration_CommandType;
	friend bool operator <(const FHI5_Calibration_Command& l, const FHI5_Calibration_Command& r);
};

class MOTIONCAPTURERUNTIME_API FHI5_Calibration_Command_Data
{
public:
	void Push(std::shared_ptr<FHI5_Calibration_Command> inValue);
	std::shared_ptr<FHI5_Calibration_Command> Pop();
	bool Empty() const;
	void Clean();
private:
	std::queue<std::shared_ptr<FHI5_Calibration_Command>> _CalibrationQue;
};
/*************************校准指令****************************************/

/**************数据类型********************/
class MOTIONCAPTURERUNTIME_API FHI5_Operate_Base_Content : public std::enable_shared_from_this<FHI5_Operate_Base_Content>
{
public:
	FHI5_Operate_Base_Content();
	std::shared_ptr<FHI5_Optic_DataArray> GetHI5_Optic_DataArray();
	std::shared_ptr<FOptiSensorBindDataArray> GetOptiSensorBindDataArray();
	std::shared_ptr<FHI5_Calibration_Command_Data> GetCalibration_Send_Data();
private:
	std::shared_ptr<FHI5_Optic_DataArray> m_HI5_Optic_DataArray;
	std::shared_ptr<FOptiSensorBindDataArray> m_OptiSensorBindDataArray;
	std::shared_ptr<FHI5_Calibration_Command_Data> m_Calibration_Send_Data;
};

/*************************Operate_Command ******************************************/
class MOTIONCAPTURERUNTIME_API FHI5_Operate_Command
{
public:
	 enum EOperate_Command
	{
	 	ENon = 0,
		EConnectEquipment = 1,  //连接设备
		ECalibrationOper = 2,   //校准操作
		ESendOpticData = 3,     //传送手光学数据
		EOptiSensorBindData = 4,     //传送手传感器绑定光学数据
		ECloseConnectEquipment = 5, // 断开连接设备
	};
	EOperate_Command m_Operate_CommandType;
	friend bool operator <(const FHI5_Operate_Command& l, const FHI5_Operate_Command& r);
};

class MOTIONCAPTURERUNTIME_API FHI5_Operate_Data : public std::enable_shared_from_this<FHI5_Operate_Data>
{
public:
	FHI5_Operate_Data();
	std::shared_ptr<FHI5_Operate_Base_Content>  Get_Data();
	FHI5_Operate_Command::EOperate_Command Get_Command();
	void SetCommand(FHI5_Operate_Command::EOperate_Command inCommand);
private:
	FHI5_Operate_Command::EOperate_Command m_Command;
	std::shared_ptr<FHI5_Operate_Base_Content> m_Data;
};

class MOTIONCAPTURERUNTIME_API FHI5_Calibration_Send_Data : public std::enable_shared_from_this<FHI5_Calibration_Send_Data>
{
public:
	FHI5_Calibration_Send_Data();
	void clean();
	//获取连接设备指令
	std::shared_ptr<FHI5_Operate_Data> GetConnectEquipmentCommand();
	//获取关闭设备指令
	std::shared_ptr<FHI5_Operate_Data> GetCloseConnectEquipmentCommand();
	//获取左手光学数据指令
	std::shared_ptr<FHI5_Optic_DataArray> GetSendOpticData();
	std::shared_ptr<FHI5_Calibration_Command_Data> GetSendCalibration();
	std::shared_ptr<FOptiSensorBindDataArray> GetSenserBindOpticData();
	void AddOpticData(std::string sensorIdentity, OpticalSensorType type, float fpos[3], float frot[4]);
	void AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command param);
	void AddOptiSensorBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType);
private:
	void init();
	std::shared_ptr<FHI5_Operate_Data> GetCommand(FHI5_Operate_Command keyComamnd);
private:
	std::map<FHI5_Operate_Command,std::shared_ptr<FHI5_Operate_Data>> m_commandDir;
	mutable  std::mutex mut;
};


struct  MOTIONCAPTURERUNTIME_API FVive_Tracker_Infor
{
	FString		trackerIdentity;
	int			trackerId;
	FVector		position;
	FRotator	rotation;
};

class MOTIONCAPTURERUNTIME_API FVive_Tracker_Infor_Map
{
public:
	FVive_Tracker_Infor_Map();
	void AddItem(FVive_Tracker_Infor& inparam);
	bool IsContain(int inIndex);
	void SetPosAndQua(int inIndex,FVector inPos, FRotator inQua);
	bool GetPosAndQua(int inIndex,FVector& outPos, FRotator& outQua);
	bool FindTwoDevices(APawn* inPawn);
	bool SaveOptiDeviceBindStateToFile(const FString& fileName,FString leftSN,FString rightSN);
protected:
	TMap<int,FVive_Tracker_Infor>   m_trackermap;
public:
	int								m_LeftTrackerIndex;
	int								m_RightTrackerIndex;
	FString							m_LeftTrackerIdentity;
	FString							m_RightTrackerIdentity;
};
/*******************************************************************/
// class MOTIONCAPTURERUNTIME_API FHi5_Thread_Data
// {
// 	public:
// 		FHi5_Thread_Data();
// 		~FHi5_Thread_Data();
// };




