// Fill out your copyright notice in the Description page of Project Settings.
#include "LocalDataReader.h"

#include "EventManager.h"
#include "AnimationDataManager.h"
#include "MotionCaptureManager.h"

using namespace Hi5;

ULocalDataReader::ULocalDataReader()
{
	CurAnimData = MakeShareable(new FAnimationData());

	CurRemoteAnimData = MakeShareable(new FAnimationData());

	PacketParser = MakeShareable(new FGlovePacketParser());
}

ULocalDataReader::~ULocalDataReader()
{
	PacketParser.Reset();
}

void ULocalDataReader::OnLocalPlayerBVHDataReceived(const FArrayReaderPtr& Data)
{
	int32 AvatarID = INDEX_NONE;
	FName AvatarName = NAME_None;

	//AvatarName = "RightHand";
	AvatarName = FAnimationDataManager::GetInstance()->GetLocalAvatarName();
	FString aname = AvatarName.ToString();
	if (AvatarName == NAME_None)
	{
		PN_LOG(Display, TEXT("Get local avatar name failed."));
		return;
	}

	if (PacketParser.IsValid() && PacketParser->Read(Data, CurAnimData))
	{

		bool bFound = FAnimationDataManager::GetInstance()->IsHandAnimationDataExist(
#if WITH_AVATARID
			AvatarID
#else
			AvatarName
			,CurAnimData->GloveMode
#endif
		);
		if (!bFound)
		{
			if (FAnimationDataManager::GetInstance()->AddHandAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurAnimData
				, CurAnimData->GloveMode
			))
			{
#ifndef TRANCE
				// Create new actor in game thread.
				//FEventManager::GetInstance()->EnqueueEvent(MakeShareable(new FCreateNewEvent(PNManager,
				//	AvatarName, AvatarID, FVector::ZeroVector, FQuat::Identity)));
#endif
			}
			else
			{
				PN_LOG(Error, TEXT("Add new animation data failed."));
			}
		}
		else
		{
			FAnimationDataManager::GetInstance()->UpdateHandAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurAnimData, CurAnimData->GloveMode);
		}
	}
	else
	{
		PN_LOG(Error, TEXT("Parse received data failed. Please check the server version."));
	}
}

void ULocalDataReader::OnRemotePlayerBVHDataReceived(const FArrayReaderPtr& Data,const FName& AName)
{
	if (AName.IsEqual(FAnimationDataManager::GetInstance()->GetLocalAvatarName()))
	{
		return;
	}

	UObject* PawnObj = nullptr;
	if (!FAnimationDataManager::GetInstance ()->IsAvatarValid (AName, &PawnObj))
	{
		return;
	}

	int32 AvatarID = INDEX_NONE;
	FName AvatarName = AName;
	if (PacketParser.IsValid() && PacketParser->Read(Data, CurRemoteAnimData, PawnObj))
	{
		bool bFound = FAnimationDataManager::GetInstance()->IsHandAnimationDataExist(
#if WITH_AVATARID
			AvatarID
#else
			AvatarName
			, CurRemoteAnimData->GloveMode
#endif
		);
		if (!bFound)
		{
			if (FAnimationDataManager::GetInstance()->AddHandAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurRemoteAnimData
				, CurRemoteAnimData->GloveMode
			))
			{
#ifndef TRANCE
				// Create new actor in game thread.
				//FEventManager::GetInstance()->EnqueueEvent(MakeShareable(new FCreateNewEvent(PNManager,
				//	AvatarName, AvatarID, FVector::ZeroVector, FQuat::Identity)));
#endif
			}
			else
			{
				PN_LOG(Error, TEXT("Add new animation data failed."));
			}
		}
		else
		{
			FAnimationDataManager::GetInstance()->UpdateHandAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurRemoteAnimData
			, CurRemoteAnimData->GloveMode);
		}
	}
	else
	{
		PN_LOG(Error, TEXT("Parse received data failed. Please check the server version."));
	}
}

int32 ULocalDataReader::GetGloveMode()
{
	if (PacketParser.IsValid())
	{
		return PacketParser->GetGloveMode();
	}

	return -1;
}


