// Fill out your copyright notice in the Description page of Project Settings.
#include "MotionCaptureAvatarComponent.h"
#include "AnimationDataManager.h"
#include "AssetRegistryModule.h"
#include "NetworkManager.h"
#include "BVHAsset.h"
#include "HI5Glove.h"
#include "Net/UnrealNetwork.h"
#include "MotionCaptureFunctionLibrary.h"

#if USE_ALICE
#include "../../../../HybridData/Source/HybridData/Public/IHybridData.h"
#endif

// Sets default values for this component's properties
UMotionCaptureAvatarComponent::UMotionCaptureAvatarComponent()
{
	// Set this component to be initialized when the game starts, and to be ticked every frame.  You can turn these features
	// off to improve performance if you don't need them.
	PrimaryComponentTick.bCanEverTick = true;

	
	// ...
}

UMotionCaptureAvatarComponent::~UMotionCaptureAvatarComponent()
{
	
}


void UMotionCaptureAvatarComponent::PostLoad()
{
	Super::PostLoad();

	LoadBVHAsset();

}
	

void UMotionCaptureAvatarComponent::BeginPlay()
{
	Super::BeginPlay();
	FAnimationDataManager::GetInstance()->AddMotionCaptureComponent(this);
}

void UMotionCaptureAvatarComponent::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);
	FAnimationDataManager::GetInstance()->RemoveMotionCaptureComponent(this);
}

#if WITH_EDITOR
void UMotionCaptureAvatarComponent::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent)
{
	const FName PropertyName = (PropertyChangedEvent.Property != nullptr) ? PropertyChangedEvent.Property->GetFName() : NAME_None;

	if (PropertyName == GET_MEMBER_NAME_CHECKED(UMotionCaptureComponent, BVHAsset))
	{
		LoadBVHAsset();
	}

	Super::PostEditChangeProperty(PropertyChangedEvent);
}
#endif

#if WITH_AVATARID
int32 UMotionCaptureAvatarComponent::GetAvatarID() const
{
	return AvatarID;
}
#else
const FName& UMotionCaptureAvatarComponent::GetAvatarName() const
{
	return AvatarName;
}

const FString& UMotionCaptureAvatarComponent::GetDeviceName() const
{
	return DeviceName;
}

bool UMotionCaptureAvatarComponent::GetIsLocalAvatar()const
{
	return IsLocalAvatar;
}

void UMotionCaptureAvatarComponent::SetIsLocalAvatar(bool IsLocal)
{
	IsLocalAvatar = IsLocal;
}

#endif

UBVHAsset* UMotionCaptureAvatarComponent::GetBVHAsset()
{
	return BVHAsset;
}

#if WITH_AVATARID
void UMotionCaptureAvatarComponent::SetAvatarID(int32 InAvatarID)
{
	AvatarID = InAvatarID;
}
#else
void UMotionCaptureAvatarComponent::SetAvatarName(const FName& InAvatarName)
{
	AvatarName = InAvatarName;
}
#endif

void UMotionCaptureAvatarComponent::SetBVHAsset(UBVHAsset* InNewBVHAsset)
{
	BVHAsset = InNewBVHAsset;
}

void UMotionCaptureAvatarComponent::MoveActorTo(const FEventBase* Event)
{
	const FMoveToEvent* TargetEvent = static_cast<const FMoveToEvent*>(Event);
	if (TargetEvent)
	{
		if (TargetEvent->Component.IsValid())
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 8)
			TargetEvent->Component->GetOwner()->SetActorLocationAndRotation(TargetEvent->DestPos, TargetEvent->DestQua);
#else
			TargetEvent->Component->GetOwner()->SetActorLocationAndRotation(TargetEvent->DestPos, TargetEvent->DestQua.Rotator());
#endif
		}
		else
		{
			PN_LOG(Error, TEXT("The actor has been destroyed before move it to target position."));
		}
	}
}

void UMotionCaptureAvatarComponent::LoadBVHAsset()
{
	if (nullptr != BVHAsset)
	{
		if (!BVHAsset->IsLoaded())
		{
			if (!BVHAsset->Load())
			{
				PN_LOG(Verbose, TEXT("BVH asset load failed."));
				BVHAsset = nullptr;
			}
		}
	}
}

void UMotionCaptureAvatarComponent::TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction)
{
	Super::TickComponent(DeltaTime, TickType, ThisTickFunction);

	AutoSetDeviceName ();
}

void UMotionCaptureAvatarComponent::AutoSetDeviceName ()
{
	if (AvatarName != NAME_None) {
		return;
	}

	EGlovePositionSource PosSrc = UMotionCaptureFunctionLibrary::GetGlovePositionSource ();
	if (PosSrc == EGlovePositionSource::AlicePosition) {
		ENetMode gamemode = GWorld->GetNetMode ();
		if (gamemode == NM_ListenServer || gamemode == NM_DedicatedServer || gamemode == NM_Client)
		{
			SetIsReplicated (true);

#if USE_ALICE
			if (IHybridData::GetPtr ()) {
				if (AvatarName == NAME_None)
				{
					if (HandMode == EHandMode::LeftHand)
					{
						FString LeftAvatarName;
						if (AliceIns ().GetLeftGloveDeviceName (LeftAvatarName))
						{
							AvatarName = FName (*LeftAvatarName);
							if (gamemode == NM_Client)
							{
								SetAvatarNameOnServer (FName (*LeftAvatarName));
							}
						}
					}
					else if (HandMode == EHandMode::RightHand)
					{
						FString RightAvatarName;
						if (AliceIns ().GetRightGloveDeviceName (RightAvatarName))
						{
							AvatarName = FName (*RightAvatarName);
							if (gamemode == NM_Client)
							{
								SetAvatarNameOnServer (FName (*RightAvatarName));
							}
						}
					}
					else
					{
					}
				}
				else
				{
					if (gamemode == NM_Client)
					{
						SetAvatarNameOnServer (AvatarName);
					}
				}
			}
#endif // USE_ALICE
		}

		if (gamemode == NM_Standalone)
		{
#if USE_ALICE
			if (IHybridData::GetPtr ()) {
				if (AvatarName == NAME_None)
				{
					if (HandMode == EHandMode::LeftHand)
					{
						FString LeftAvatarName;
						if (AliceIns ().GetLeftGloveDeviceName (LeftAvatarName))
						{
							AvatarName = FName (*LeftAvatarName);
						}
					}
					else if (HandMode == EHandMode::RightHand)
					{
						FString RightAvatarName;
						if (AliceIns ().GetRightGloveDeviceName (RightAvatarName))
						{
							AvatarName = FName (*RightAvatarName);
						}
					}
					else
					{
					}
				}
			}
#endif // USE_ALICE
		}
	}
	else if (PosSrc == EGlovePositionSource::VivePosition) {
		AvatarName = TEXT ("DefaultGlove");
	}
}

void UMotionCaptureAvatarComponent::GetLifetimeReplicatedProps(TArray< FLifetimeProperty >& OutLifetimeProps) const
{
	// Intentionally not calling super: We do not want to replicate bActive which controls ticking. We sometimes need to tick on client predictively.
	Super::GetLifetimeReplicatedProps(OutLifetimeProps);

	DOREPLIFETIME(UMotionCaptureAvatarComponent, DeviceName);
	DOREPLIFETIME(UMotionCaptureAvatarComponent, AvatarName);
}

void UMotionCaptureAvatarComponent::SetAvatarNameOnServer_Implementation(const FName& _AvatarName)
{
	this->AvatarName = _AvatarName;
}

bool UMotionCaptureAvatarComponent::SetAvatarNameOnServer_Validate(const FName& _AvatarName)
{
	return (_AvatarName != NAME_None);
}
