// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"
#include "MotionCaptureAvatarComponent.generated.h"


UENUM(BlueprintType)
enum class EHandMode :uint8
{
	ModeUnknown = 0,
	LeftHand,
	RightHand ,
};

UENUM(BlueprintType)
enum class ELocationMode :uint8
{
	World,
	Relative,
};


class FEventBase;
class UBVHAsset;

UCLASS( ClassGroup= MotionCapture, meta=(BlueprintSpawnableComponent) )
class MOTIONCAPTURERUNTIME_API UMotionCaptureAvatarComponent : public USceneComponent
{
	GENERATED_BODY()

public:	
	// Sets default values for this component's properties
	UMotionCaptureAvatarComponent();
	~UMotionCaptureAvatarComponent();


	virtual void PostLoad() override;

	virtual void BeginPlay() override;

	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	void AutoSetDeviceName ();

#if WITH_EDITOR
	virtual void PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent) override;
#endif

	// 	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	// 	int32 GetAvatarID() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	const FName& GetAvatarName() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	const FString& GetDeviceName() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	UBVHAsset* GetBVHAsset();

	// 	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	// 	void SetAvatarID(int32 InAvatarID);

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	void SetAvatarName(const FName& InAvatarName);

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	void SetBVHAsset(UBVHAsset* InNewBVHAsset);

	void MoveActorTo(const FEventBase* Event);

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	bool GetIsLocalAvatar() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	void SetIsLocalAvatar(bool IsLocal);

public:
	/** Avatar ID is used to receive animation data, this will be disabled when BVH asset is set. */
	UPROPERTY(VisibleInstanceOnly, Category = "MotionCapture", meta = (DisplayName = "Avatar ID"))
	int32 AvatarID = INDEX_NONE;

	/** Avatar name is used to receive animation data, this will be disabled when BVH asset is set. */
	UPROPERTY (EditAnywhere, BlueprintReadWrite, Replicated, Category = "MotionCapture", meta = (DisplayName = "Avatar Name"))
	FName AvatarName = NAME_None;
	//FName AvatarName = "DefaultName";	// NAME_None;

	/** Device name is used to receive Alice data, this will be disabled when BVH asset is set. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Replicated, Category = "MotionCapture", meta = (DisplayName = "Device Name"))
	FString DeviceName;

	/** Avatar name is used to receive animation data, this will be disabled when BVH asset is set. */
	UPROPERTY(EditAnywhere, Category = "MotionCapture", meta = (DisplayName = "Is Local Avatar"))
	bool IsLocalAvatar = false;

	UPROPERTY(EditAnywhere, Category = "MotionCapture", meta = (DisplayName = "Use Displacement"))
	bool UseDisplacement = false;

	UPROPERTY(EditAnywhere, Category = "MotionCapture", meta = (DisplayName = "Hand Mode"))
	EHandMode HandMode = EHandMode::ModeUnknown;

	UPROPERTY (EditAnywhere, Category = "MotionCapture", meta = (DisplayName = "UseLocationAs"))
	ELocationMode LocationMode = ELocationMode::World;

public:
	/** The BVH asset which is used to read animation data from. */
	UPROPERTY(EditInstanceOnly, BlueprintReadWrite, Category = "MotionCapture", meta = (DisplayName = "BVH Asset"))
	UBVHAsset* BVHAsset = nullptr;

	virtual void GetLifetimeReplicatedProps(TArray< FLifetimeProperty >& OutLifetimeProps) const override;

	UFUNCTION (BlueprintCallable, Reliable, Server, WithValidation, Category = "MotionCapture | Server")
	void SetAvatarNameOnServer (const FName& _AvatarName);
	void SetAvatarNameOnServer_Implementation (const FName& _AvatarName);
	bool SetAvatarNameOnServer_Validate (const FName& _AvatarName);
private:
	void LoadBVHAsset();
	
};
