// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "MotionCaptureComponent.h"
#include "AnimationDataManager.h"
#include "AssetRegistryModule.h"
#include "BVHAsset.h"
#include "MotionCaptureSpawner.h"
#include "MotionCaptureManager.h"


// Sets default values
UMotionCaptureComponent::UMotionCaptureComponent(const FObjectInitializer& ObjectInitializer) :
	Super(ObjectInitializer)/*,
	bAutoStart(true)*/
{
}

UMotionCaptureComponent::~UMotionCaptureComponent()
{
// 	FAnimationDataManager::GetInstance()->RemoveAnimationData(
// #if WITH_AVATARID
// 		GetAvatarID()
// #else
// 		GetAvatarName()
// #endif
// 		);
}

void UMotionCaptureComponent::PostLoad()
{
	Super::PostLoad();

	LoadBVHAsset();
}

#if WITH_EDITOR
void UMotionCaptureComponent::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent)
{
	const FName PropertyName = (PropertyChangedEvent.Property != nullptr) ? PropertyChangedEvent.Property->GetFName() : NAME_None;

	if (PropertyName == GET_MEMBER_NAME_CHECKED(UMotionCaptureComponent, BVHAsset))
	{
		LoadBVHAsset();
	}
	
	Super::PostEditChangeProperty(PropertyChangedEvent);
}
#endif

#if WITH_AVATARID
int32 UMotionCaptureComponent::GetAvatarID() const
{
	return AvatarID;
}
#else
const FName& UMotionCaptureComponent::GetAvatarName() const
{
	return AvatarName;
}
#endif

UBVHAsset* UMotionCaptureComponent::GetBVHAsset()
{
	return BVHAsset;
}

#if WITH_AVATARID
void UMotionCaptureComponent::SetAvatarID(int32 InAvatarID)
{
	
	AvatarID = InAvatarID;
}
#else
void UMotionCaptureComponent::SetAvatarName(const FName& InAvatarName)
{
	AvatarName = InAvatarName;
}
#endif

void UMotionCaptureComponent::SetBVHAsset(UBVHAsset* InNewBVHAsset)
{
	BVHAsset = InNewBVHAsset;
}

void UMotionCaptureComponent::MoveActorTo(const FEventBase* Event)
{
	const FMoveToEvent* TargetEvent = static_cast<const FMoveToEvent*>(Event);
	if (TargetEvent)
	{
		if (TargetEvent->Component.IsValid())
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 8)
			TargetEvent->Component->GetOwner()->SetActorLocationAndRotation(TargetEvent->DestPos, TargetEvent->DestQua);
#else
			TargetEvent->Component->GetOwner()->SetActorLocationAndRotation(TargetEvent->DestPos, TargetEvent->DestQua.Rotator());
#endif
		}
		else
		{
			PN_LOG(Error, TEXT("The actor has been destroyed before move it to target position."));
		}
	}
}

void UMotionCaptureComponent::LoadBVHAsset()
{
	if (nullptr != BVHAsset)
	{
		if (!BVHAsset->IsLoaded())
		{
			if (!BVHAsset->Load())
			{
				PN_LOG(Verbose, TEXT("BVH asset load failed."));
				BVHAsset = nullptr;
			}
		}
	}
}
