// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "MotionCaptureComponent.generated.h"


class FEventBase;
class UBVHAsset;


UCLASS(ClassGroup = MotionCapture, hidecategories = Object, editinlinenew/*, meta = (BlueprintSpawnableComponent)*/)
class UMotionCaptureComponent : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this actor's properties
	UMotionCaptureComponent(const FObjectInitializer& ObjectInitializer);
	virtual ~UMotionCaptureComponent();

	virtual void PostLoad() override;

#if WITH_EDITOR
	virtual void PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent) override;
#endif

// 	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
// 	int32 GetAvatarID() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	const FName& GetAvatarName() const;

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	UBVHAsset* GetBVHAsset();

// 	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
// 	void SetAvatarID(int32 InAvatarID);

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	void SetAvatarName(const FName& InAvatarName);

	UFUNCTION(BlueprintCallable, Category = "MotionCapture")
	void SetBVHAsset(UBVHAsset* InNewBVHAsset);

	void MoveActorTo(const FEventBase* Event);

private:
	/** Avatar ID is used to receive animation data, this will be disabled when BVH asset is set. */
	UPROPERTY(VisibleInstanceOnly, Category = "MotionCapture", meta = (DisplayName = "Avatar ID"))
	int32 AvatarID = INDEX_NONE;

	/** Avatar name is used to receive animation data, this will be disabled when BVH asset is set. */
	UPROPERTY(EditInstanceOnly, Category = "MotionCapture", meta = (DisplayName = "Avatar Name"))
	FName AvatarName = NAME_None;

public:
	/** The BVH asset which is used to read animation data from. */
	UPROPERTY(EditInstanceOnly, BlueprintReadWrite, Category = "MotionCapture", meta = (DisplayName = "BVH Asset"))
	UBVHAsset* BVHAsset = nullptr;

// 	UPROPERTY(EditAnywhere, Category = "MotionCapture", /*replicated,*/ meta = (DisplayName = "Auto Start BVH anim"))
// 	uint8  bAutoStart : 1;

private:
	void LoadBVHAsset();

};
