// Fill out your copyright notice in the Description page of Project Settings.
#include "MotionCaptureFunctionLibrary.h"
#include "NetworkManager.h"
#include "HI5Glove.h"
#include "UnrealXML.h"
#include "SteamVRFunctionLibrary.h"
#include "AnimationData.h"
#include "AnimationDataManager.h"
#include "HAL/PlatformFilemanager.h"

#include "Windows/AllowWindowsPlatformTypes.h"
#include <shlobj.h>
#include "Hi5_Manager_Thread.h"
#include "Windows/HideWindowsPlatformTypes.h"

EGlovePositionSource UMotionCaptureFunctionLibrary::GlovePositionSource                             = EGlovePositionSource::VivePosition;
int UMotionCaptureFunctionLibrary::LeftViveTrackerId                                                = -1;
int UMotionCaptureFunctionLibrary::RightViveTrackerId                                               = -1;

bool locateUserRoamingFolder(FString& path)
{
	WCHAR* roaming_dir;
	if (SUCCEEDED (SHGetKnownFolderPath (FOLDERID_RoamingAppData, 0, NULL, &roaming_dir)))
	{
		path = FPaths::Combine (roaming_dir, TEXT ("Hi5"));
		return FPlatformFileManager::Get ().GetPlatformFile ().CreateDirectory (*path);
	}
	else
	{
		return false;
	}
}

UMotionCaptureFunctionLibrary::UMotionCaptureFunctionLibrary()
{
}

bool UMotionCaptureFunctionLibrary::StartMocapService (
	EGlovePositionSource posSrc /*= EGlovePositionSource::VivePosition*/,
	bool ReadLocal/*=true*/)
{
	GlovePositionSource = posSrc;
	FHi5_Manager_Thread::InitManagerThread();
	if (ReadLocal)
	{
		if (GlovePositionSource == EGlovePositionSource::VivePosition)
		{
			StartReadGlove ();
		}
		else if (GlovePositionSource == EGlovePositionSource::AlicePosition)
		{
			
		}
	}
	NetworkManager::GetInstance ()->InitLocalDataReader ();
	return true;
}

void UMotionCaptureFunctionLibrary::StopMocapService ()
{
	NetworkManager::GetInstance()->Disconnect();

	if (GlovePositionSource == EGlovePositionSource::VivePosition)
	{
		
	}
	else if (GlovePositionSource == EGlovePositionSource::AlicePosition)
	{

	}
	StopReadGlove ();
}

void UMotionCaptureFunctionLibrary::CalibrationGlove(ECalibrationPose Pose, int32 TimeOut)
{
	//rg
	if (Pose == ECalibrationPose::GCP_BPose && FHi5_Manager_Thread::getInstance() != nullptr)
		FHi5_Manager_Thread::getInstance()->AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command::EBCalibration);
	if (Pose== ECalibrationPose::GCP_PPose && FHi5_Manager_Thread::getInstance()!= nullptr)
		FHi5_Manager_Thread::getInstance()->AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command::EPCalibration);
	if (Pose ==ECalibrationPose::GCP_VPose && FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command::EPreCalibration);
		FHi5_Manager_Thread::getInstance()->AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command::EVCalibration);
	}
	return;
	if (hi5_isInViveSystem ())
	{
		if (hi5_isLeftGloveAvailable () || hi5_isRightGloveAvailable ())
		{
			hi5_cali_mutex_lock ();
			if (Pose == ECalibrationPose::GCP_APose || Pose == ECalibrationPose::GCP_BPose)
			{
				CalibrationPrepare ();
			}
			Calibrationglove ((CalibrationPose)Pose, TimeOut);
		}
	}
	else if (hi5_isInAliceSystem())
	{
		if (Pose == ECalibrationPose::GCP_APose || Pose == ECalibrationPose::GCP_BPose)
		{
			CalibrationPrepare ();
		}
		Calibrationglove ((CalibrationPose)Pose, TimeOut);
	}
}

int32 UMotionCaptureFunctionLibrary::GetCalibrationProgress()
{
	return GetCalibrationProgressInPercent();
}

bool UMotionCaptureFunctionLibrary::GetParsedLocalJointsData(EGloveMod Glove, TArray<FVector>& Positions, TArray<FRotator>& Orientations)
{
	FName AvatarName = FAnimationDataManager::GetInstance ()->GetLocalAvatarName ();

	FAnimationData Data;
	if (FAnimationDataManager::GetInstance ()->GetHandAnimationData (AvatarName, Data, (Glove == EGloveMod::GlM_LeftGlove) ? 1 : 2))
	{
		Positions = Data.BonePositions;
		Orientations.SetNumUninitialized (Data.BoneRotations.Num ());
		for (int i = 0; i < Data.BoneRotations.Num (); i++) {
			Orientations[i] = Data.BoneRotations[i].Rotator ();
		}
		return true;
	}
	else
	{
		return false;
	}
}

bool UMotionCaptureFunctionLibrary::GetLocalJointData (EGloveMod Glove, EMCBones::Type Bone, FVector& Position, FRotator& Orientation)
{
	if (Bone == EMCBones::Invalid) {
		return false;
	}
	FName AvatarName = FAnimationDataManager::GetInstance ()->GetLocalAvatarName ();
	FAnimationData Data;
	if (FAnimationDataManager::GetInstance ()->GetHandAnimationData (AvatarName, Data, (Glove == EGloveMod::GlM_LeftGlove) ? 1 : 2))
	{
		Position = Data.BonePositions[Bone];
		Orientation = Data.BoneRotations[Bone].Rotator();
		return true;
	}
	else
	{
		return false;
	}
}

bool UMotionCaptureFunctionLibrary::GetLocalGloveData(EGloveMod Glove, const FString &AvatarName, TArray<uint8>& Data)
{
	GloveBVHData GetData;

	int len = GetGloveData((GloveMod)Glove, (uint8*)&GetData);
	if (len == sizeof(GetData))
	{
		HI5BVHData bvhData;
		memset(&bvhData, 0, sizeof(bvhData));
		bvhData.header.deviceType = (int)AIT_HI5Glove;
		bvhData.header.version = 1;
		int NameLen = 0;
		WCharToChar((wchar_t*)*AvatarName, AvatarName.Len(), bvhData.avatarName, NameLen);
		if (Glove == EGloveMod::GlM_LeftGlove)
		{
			bvhData.bLeftHand = true;
		}
		else
		{
			bvhData.bLeftHand = false;
		}
		bvhData.bWithDisp = GetData.bWithDisp;
		bvhData.frameIndex = GetData.frameIndex;
		bvhData.nAvatarIndex = 1;
		bvhData.dataCount = GetData.dataCount;
		memcpy(bvhData.data, GetData.data, sizeof(GetData.data));
		Data.Append((uint8*)&bvhData, sizeof(bvhData));
		return true;
	}
	return false;
}

void UMotionCaptureFunctionLibrary::OnReceiveGloveData(const FString& AvartarName, const TArray<uint8>& Data)
{
	int OldDataLen = Data.Num();
	
	GloveBVHData *pbvhData = (GloveBVHData *)Data.GetData();
	int DataLen = sizeof(GloveBVHData);
	if (OldDataLen != DataLen)
	{
		return;
	}
	FArrayReaderPtr Datagram = MakeShareable(new FArrayReader(true));
	Datagram->SetNumUninitialized(DataLen);
	Datagram->Insert((uint8*)pbvhData, DataLen, 0);
	FName AName(*AvartarName);
	NetworkManager::GetInstance()->OnRemotePlayerBVHDataReceived(Datagram, AName);
}

void UMotionCaptureFunctionLibrary::GetGloveOffsetToOpticalDevice(FVector& LeftOffset, FVector& RightOffset)
{
	hi5_getHandOffsetToOpticalDeviceInUE (LeftOffset, RightOffset);
}

void UMotionCaptureFunctionLibrary::ResetDeviceId()
{
	LeftViveTrackerId = RightViveTrackerId = -1;
}

void UMotionCaptureFunctionLibrary::CacheDeviceId()
{
	OpticalSensorType sensor_type;
	FString leftSN = hi5_getTrackerBindState (true, sensor_type);
	hi5_getViveTrackerId (leftSN, LeftViveTrackerId);
	FString rightSN = hi5_getTrackerBindState (false, sensor_type);
	hi5_getViveTrackerId (rightSN, RightViveTrackerId);
}

bool UMotionCaptureFunctionLibrary::GetBindedOpticalDeviceDataInUESpace(EGloveMod Mod, FVector& Position, FRotator& Rotation)
{
	if (Mod == EGloveMod::GlM_LeftGlove)
	{
		return GetTrackedDeviceDataInUESpace (LeftViveTrackerId, Position, Rotation);
	}
	else if (Mod == EGloveMod::GlM_RightGlove)
	{
		return GetTrackedDeviceDataInUESpace (RightViveTrackerId, Position, Rotation);
	}
	return false;
}

FSenerInforBluePr UMotionCaptureFunctionLibrary::GetSensorInfor( int Index)
{
	FSenerInforBluePr information;
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->GetSensorInfor(Index,information);
	}
	return information;
}

int UMotionCaptureFunctionLibrary::GetSensorSingal(int Index)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FSenerInforBluePr information;
		FHi5_Manager_Thread::getInstance()->GetSensorInfor(Index,information);
		return information.m_signalValue;
	}
	else
	{
		return 0;
	}
}

int UMotionCaptureFunctionLibrary::GetSensorMagnetic(int Index)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FSenerInforBluePr information;
		FHi5_Manager_Thread::getInstance()->GetSensorInfor(Index,information);
		return information.m_magneticValue;
	}
	else
	{
		return 0;
	}
}
 int UMotionCaptureFunctionLibrary::GetSensorEnergyValue(int Index)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FSenerInforBluePr information;
		FHi5_Manager_Thread::getInstance()->GetSensorInfor(Index,information);
		return information.m_energyValue;
	}
	else
	{
		return 0;
	}
}

bool UMotionCaptureFunctionLibrary::GetSensorOnLine(int Index)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FSenerInforBluePr information;
		FHi5_Manager_Thread::getInstance()->GetSensorInfor(Index,information);
		if(information.m_signalValue  >= 1)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else
	{
		return false;
	}
}

void UMotionCaptureFunctionLibrary::Hi5Tick(float deltaTime)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->Update(deltaTime);
	}
}

void UMotionCaptureFunctionLibrary::JudgeLeftHandAndRightHand(APawn* inPawn)
{
	hi5_2PreCalibration();
	JudgeLeftHandAndRightHDeviceId(inPawn);
}

void UMotionCaptureFunctionLibrary::PrintTestOptData()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->mHi5DataTest.printOpticData();
	}
}

void UMotionCaptureFunctionLibrary::PrintTestLeftBvhData()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->mHi5DataTest.printleftBvhData();
	}
}

void UMotionCaptureFunctionLibrary::PrintTestRightBvhData()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->mHi5DataTest.printRightBvhData();
	}
}

void UMotionCaptureFunctionLibrary::Reconnect()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->Reconnected();
	}
}

void UMotionCaptureFunctionLibrary::SetEnableFingerAdbFixed(bool isFingerFixed)
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->SetEnableFingerAdbFixed(isFingerFixed);
	}
}

void UMotionCaptureFunctionLibrary::ClibrationingClickHome()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		FHi5_Manager_Thread::getInstance()->SetCalibrationState(ECalibrationState::ECalibrationNone);
	}
}

ECalibrationState UMotionCaptureFunctionLibrary::GetCalibrationState()
{
	if(FHi5_Manager_Thread::getInstance() != nullptr)
	{
		return FHi5_Manager_Thread::getInstance()->GetCalibrationState();
	}
	return ECalibrationState::ECalibrationNone;
}

EGlovePositionSource UMotionCaptureFunctionLibrary::GetGlovePositionSource ()
{
	return GlovePositionSource;
}

bool UMotionCaptureFunctionLibrary::LoadCalibrationData ()
{
	
	//bool bind_ret = LoadOptiDeviceBindStateFromFile ();
	bool cali_ret = LoadCalibrationFile ();
	return (cali_ret);
}

bool UMotionCaptureFunctionLibrary::SaveCalibrationData ()
{
	bool cali_ret = SaveCalibrationFile ();
	//bool bind_ret = SaveOptiDeviceBindStateToFile ();
	return (cali_ret );
}

bool UMotionCaptureFunctionLibrary::SaveCalibrationFile(const FString& fileName /*= TEXT("BCP.bin")*/)
{
	FString filePath;
	if (locateUserRoamingFolder (filePath))
	{
		filePath = FPaths::Combine (filePath, fileName);
		return saveCalibrationData (TCHAR_TO_ANSI (*filePath)) != 0;
	}
	else
	{
		return false;
	}
}

bool UMotionCaptureFunctionLibrary::LoadCalibrationFile(const FString& fileName /*= TEXT("BCP.bin")*/)
{
	FString filePath;
	if (locateUserRoamingFolder (filePath))
	{
		FString fileNameTemp = "CalibrationData.xml";
		filePath = FPaths::Combine (filePath, fileNameTemp);
		return hi5_loadCalibrationFile (TCHAR_TO_ANSI (*filePath));
	}
	else
	{
		return false;
	}
}

void UMotionCaptureFunctionLibrary::VibrateLeftGlove (const int GloveTimeSpan)
{
	if (GloveTimeSpan >= 0)
	{
		/*if (GloveTimeSpan > 5000)
		{
			hi5_enableVibrator (5000, 0xffff);
		}
		else
		{
			hi5_enableVibrator (GloveTimeSpan, 0xffff);
		}*/
		hi5_enableVibratorOfLeft (GloveTimeSpan);
	}
}

void UMotionCaptureFunctionLibrary::VibrateRightGlove (const int GloveTimeSpan)
{
	if (GloveTimeSpan >= 0)
	{
		/*if (GloveTimeSpan > 5000)
		{
			hi5_enableVibrator (0xffff, 5000);
		}
		else
		{
			hi5_enableVibrator (0xffff, GloveTimeSpan);
		}*/
		hi5_enableVibratorOfRight (GloveTimeSpan);
	}
}

void UMotionCaptureFunctionLibrary::VibrateGloves (const int LeftGloveTimeSpan, const int RightGloveTimeSpan)
{
	unsigned int ltime, rtime;
	if (LeftGloveTimeSpan < 0)
	{
		ltime = 0xffff;
	}
	else if (LeftGloveTimeSpan > 5000)
	{
		ltime = 5000;
	}
	else
	{
		ltime = LeftGloveTimeSpan;
	}

	if (RightGloveTimeSpan < 0)
	{
		rtime = 0xffff;
	}
	else if (RightGloveTimeSpan > 5000)
	{
		rtime = 5000;
	}
	else
	{
		rtime = RightGloveTimeSpan;
	}

	//hi5_enableVibrator (ltime, rtime);
	hi5_enableVibratorOfBoth (ltime, rtime);
}

bool UMotionCaptureFunctionLibrary::GetOptiDeviceSN(int32 id, FString& sn)
{
	return hi5_getViveTrackerSN(id, sn);
}

bool UMotionCaptureFunctionLibrary::SaveOptiDeviceBindStateToFile(const FString& fileName)
{
	return true;
	FString filepath;
	if (locateUserRoamingFolder (filepath))
	{
		filepath = FPaths::Combine (filepath, fileName);

		MotionCaptureXML::UnrealXML xml_saver;
		if (FPaths::FileExists (filepath))
		{
			xml_saver.LoadXML (filepath);
		}
		else
		{
			xml_saver.UseXMLString (_T ("<HI5_BindInfo></HI5_BindInfo>"));
		}

		OpticalSensorType leftDevType, rightDevType;
		FString leftSN = hi5_getTrackerBindState (true, leftDevType);
		FString rightSN = hi5_getTrackerBindState (false, rightDevType);

		FString leftDevTypeStr, rightDevTypeStr;

		switch (leftDevType)
		{
		case OpticalSensorType::OST_HTC_VIVE_Tracker:
			leftDevTypeStr = _T ("HTC_VIVE_Tracker");
			break;
		case OpticalSensorType::OST_HTC_VIVE_Controller:
			leftDevTypeStr = _T ("HTC_VIVE_Controller");
			break;
		default:
			break;
		}

		switch (rightDevType)
		{
		case OpticalSensorType::OST_HTC_VIVE_Tracker:
			rightDevTypeStr = _T ("HTC_VIVE_Tracker");
			break;
		case OpticalSensorType::OST_HTC_VIVE_Controller:
			rightDevTypeStr = _T ("HTC_VIVE_Controller");
			break;
		default:
			break;
		}

		{
			void* node_root = xml_saver.GetOperateNode ();

			void* node_left = xml_saver.SetOperateNodeByTag (_T ("Left"), 0);
			xml_saver.SetOperateNodeByTag (_T ("Type"), 0);
			xml_saver.SetContent (_T ("LEFT"));
			xml_saver.SetOperateNodeByTag (_T ("SerialNumber"), 0, node_left);
			xml_saver.SetContent (leftSN);
			xml_saver.SetOperateNodeByTag (_T ("DeviceType"), 0, node_left);
			xml_saver.SetContent (leftDevTypeStr);

			void* node_right = xml_saver.SetOperateNodeByTag (_T ("Right"), 0, node_root);
			xml_saver.SetOperateNodeByTag (_T ("Type"), 0);
			xml_saver.SetContent (_T ("RIGHT"));
			xml_saver.SetOperateNodeByTag (_T ("SerialNumber"), 0, node_right);
			xml_saver.SetContent (rightSN);
			xml_saver.SetOperateNodeByTag (_T ("DeviceType"), 0, node_right);
			xml_saver.SetContent (rightDevTypeStr);
		}

		return xml_saver.SaveXML (filepath);
	}
	else
	{
		return false;
	}
}

 bool UMotionCaptureFunctionLibrary::LoadOptiDeviceBindStateFromFile(const FString& fileName)
{
	 return true;
	FString filepath;
	if (locateUserRoamingFolder (filepath))
	{
		filepath = FPaths::Combine (filepath, fileName);
		if (!FPaths::FileExists (filepath))
		{
			return false;
		}

		MotionCaptureXML::UnrealXML xml_loader;
		void* node_root = xml_loader.LoadXML (filepath);

		xml_loader.SetOperateNodeByTag (_T ("Left"), 0);
		FString leftSN = xml_loader.GetContentByTag (_T ("SerialNumber"));
		FString leftDevTypeStr = xml_loader.GetContentByTag (_T ("DeviceType"));
		OpticalSensorType leftDevType;
		if (leftDevTypeStr == _T ("HTC_VIVE_Tracker"))
		{
			leftDevType = OpticalSensorType::OST_HTC_VIVE_Tracker;
		}
		else if (leftDevTypeStr == _T("HTC_VIVE_Controller"))
		{
			leftDevType = OpticalSensorType::OST_HTC_VIVE_Controller;
		}
		else
		{
			leftDevType = OpticalSensorType::OST_Unknown;
		}
		xml_loader.SetOperateNodeByTag (_T ("Right"), 0, node_root);
		FString rightSN = xml_loader.GetContentByTag (_T ("SerialNumber"));
		FString rightDevTypeStr = xml_loader.GetContentByTag (_T ("DeviceType"));
		OpticalSensorType rightDevType;
		if (rightDevTypeStr == _T ("HTC_VIVE_Tracker"))
		{
			rightDevType = OpticalSensorType::OST_HTC_VIVE_Tracker;
		}
		else if (rightDevTypeStr == _T ("HTC_VIVE_Controller"))
		{
			rightDevType = OpticalSensorType::OST_HTC_VIVE_Controller;
		}
		else {
			rightDevType = OpticalSensorType::OST_Unknown;
		}

		hi5_loadTrackerBindState (true, leftSN, leftDevType);
		hi5_loadTrackerBindState (false, rightSN, rightDevType);

		/*TArray<int32> OutTrackedDeviceIds, OutControllerIds;
		USteamVRFunctionLibrary::GetValidTrackedDeviceIds (ESteamVRTrackedDeviceType::Invalid, OutTrackedDeviceIds);
		USteamVRFunctionLibrary::GetValidTrackedDeviceIds (ESteamVRTrackedDeviceType::Controller, OutControllerIds);
		OutTrackedDeviceIds.Append (OutControllerIds);

		for (int32& id : OutTrackedDeviceIds) {
			FString sn;
			if (GetViveTrackerSN (id, sn)) {
				if (sn == leftSN) {
					SetLeftViveTrackerId (id);
				}
				if (sn == rightSN) {
					SetRightViveTrackerId (id);
				}
			}
		}*/

		return true;
	}
	else
	{
		return false;
	}
}

bool UMotionCaptureFunctionLibrary::GetOptiDeviceBindState(EGloveMod hand, FString& tracker_sn)
{
	bool isLeftHand = (hand == EGloveMod::GlM_LeftGlove);
	OpticalSensorType sensor_type;
	tracker_sn = hi5_getTrackerBindState (isLeftHand, sensor_type);
	return !tracker_sn.IsEmpty ();
}

bool UMotionCaptureFunctionLibrary::GetTrackedDeviceDataInViveSpace (int DeviceId, float* pos, float* rot)
{
	FVector Position;
	FRotator Orientation;
	/*if (USteamVRFunctionLibrary::GetTrackedDevicePositionAndOrientation (DeviceId, Position, Orientation)) */
	if (hi5_getTrackedDevicePose (DeviceId, Position, Orientation))
	{
        UESpace2ViveSpace(Position, Orientation, pos, rot);

		//pos[0] = Position[1] / 100;	// vive_x <= ue_y
		//pos[1] = Position[2] / 100;	// vive_y <= ue_z
		//pos[2] = -Position[0] / 100;// vive_z <= -ue_x

		//FQuat q = Orientation.Quaternion();
		//rot[0] = q.W;	// vive_w <= ue_w
		//rot[1] = -q.Y;	// vive_x <= -ue_y
		//rot[2] = -q.Z;	// vive_y <= -ue_z
		//rot[3] = q.X;	// vive_z <= ue_x
		//PN_LOG (Warning, TEXT ("VivePos: %s"), *Position.ToString ());
		return true;
	}
	return false;
}

void UMotionCaptureFunctionLibrary::UESpace2ViveSpace(const FVector& Position, const FRotator& Rotation, float* pos, float* rot)
{
	pos[0] = Position[1] / 100;	// vive_x <= ue_y
	pos[1] = Position[2] / 100;	// vive_y <= ue_z
	pos[2] = -Position[0] / 100;// vive_z <= -ue_x

	FQuat q = Rotation.Quaternion ();
	rot[0] = q.W;	// vive_w <= ue_w
	rot[1] = -q.Y;	// vive_x <= -ue_y
	rot[2] = -q.Z;	// vive_y <= -ue_z
	rot[3] = q.X;	// vive_z <= ue_x
}

void UMotionCaptureFunctionLibrary::GetGlovePowerLevel(EGlovePowerLevel& leftGlove, EGlovePowerLevel& rightGlove)
{
	hi5_getGlovePowerLevel (leftGlove, rightGlove);
}

void UMotionCaptureFunctionLibrary::GetGloveMagneticed(EGloveMagneticedState& leftGlove, EGloveMagneticedState& rightGlove)
{
	hi5_getGloveMag (leftGlove, rightGlove);
}

bool UMotionCaptureFunctionLibrary::IsGloveAvailable(EGloveMod mod)
{
	switch (mod)
	{
	case EGloveMod::GlM_LeftGlove:
		return hi5_isLeftGloveAvailable ();

	case EGloveMod::GlM_RightGlove:
		return hi5_isRightGloveAvailable ();

	case EGloveMod::GlM_BothGloves:
		return hi5_isLeftGloveAvailable () && hi5_isRightGloveAvailable ();

	default:
		return false;
	}
}

bool UMotionCaptureFunctionLibrary::IsDongleAvailable()
{
	return hi5_isDongleAvailable ();
}

bool UMotionCaptureFunctionLibrary::GetTrackedDeviceDataInUESpace(int DeviceId, FVector& Position, FRotator& Rotation)
{
	return hi5_getTrackedDevicePose (DeviceId, Position, Rotation);
}

int UMotionCaptureFunctionLibrary::GetLeftTrackerId ()
{
	return LeftViveTrackerId;
}

int UMotionCaptureFunctionLibrary::GetRightTrackerId ()
{
	return RightViveTrackerId;
}

