// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MotionCaptureManager.generated.h"


class UMotionCaptureNetwork;
class AMotionCaptureSpawner;

UCLASS(ClassGroup = MotionCapture/*, hidecategories = (Rendering, Physics, Actor)*/)
class AMotionCaptureManager : public AActor
{
	GENERATED_BODY()

public:
 	/** Host IP address. */
 	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Network")
 	FString IP = TEXT("0.0.0.0");

	/** Bounding port. */
	UPROPERTY(EditAnywhere, Category = "Network")
	int32 Port = 7002;

public:
	AMotionCaptureManager(const FObjectInitializer& ObjectInitializer);

	virtual void BeginPlay() override;
	virtual void BeginDestroy() override;
	virtual void Tick(float DeltaSeconds) override;

	static AMotionCaptureManager* GetInstance();
	bool Connect(const FString& IP, int32 Port);
	void Disconnect();

	bool FindSpawner(int32 AvatarID, const FName& AvatarName, FVector& InitPos, FQuat& InitQua, UClass** ActorClass) const;
	void CreatePNActor(const FEventBase* Event) const;

#if WITH_EDITORONLY_DATA
public:
	/** Returns SpriteComponent subobject **/
	class UBillboardComponent* GetSpriteComponent() const;
private:
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Display, meta = (AllowPrivateAccess = "true"))
	class UBillboardComponent* SpriteComponent = nullptr;
#endif

private:
	UPROPERTY()
	UMotionCaptureNetwork*			Network = nullptr;

	mutable TArray<TWeakObjectPtr<AMotionCaptureSpawner>>	SpawnerInstances;

	static AMotionCaptureManager* Instance;
};
