// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#include "MotionCaptureNetwork.h"
#include "EventManager.h"
#include "AnimationDataManager.h"
#include "MotionCaptureManager.h"

using namespace Hi5;

UMotionCaptureNetwork::UMotionCaptureNetwork() :
CurAnimData(nullptr)
{
	
}

UMotionCaptureNetwork::~UMotionCaptureNetwork()
{

}

bool UMotionCaptureNetwork::Connect()
{
	if (!UDPSocket.IsValid())
	{
		AMotionCaptureManager* PNManager = Cast<AMotionCaptureManager>(GetOuter());
		check(PNManager);

		int32 Port = PNManager->Port;
		TSharedPtr<class FInternetAddr> HostAddr = ISocketSubsystem::Get()->GetLocalBindAddr(*GLog);
		if (!HostAddr.IsValid())
		{
			PN_LOG(Error, TEXT("Get local bind addr failed."));
			return false;
		}
		PNManager->IP = HostAddr->ToString(false);
		uint32 HostIp = 0;
		HostAddr->GetIp(HostIp);
		FUdpSocketBuilder Builder = FUdpSocketBuilder(TEXT("Motion Capture UDP Network")).
			BoundToAddress(FIPv4Address(HostIp)).
			BoundToPort(Port);

		UDPSocket = MakeShareable(Builder.Build());
	}

	if (UDPSocket.IsValid())
	{
		CurAnimData = MakeShareable(new FAnimationData());

		PacketParser = MakeShareable(new FPacketParser());

		UDPReceiver = MakeShareable(new FUdpSocketReceiver(UDPSocket.Get(), FTimespan(0, 0, 1), TEXT("Motion Capture UDP receiver")));
		UDPReceiver->OnDataReceived().BindUObject(this, &UMotionCaptureNetwork::UdpReceivedCallback);
//#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 11)
		UDPReceiver->Start();
//#endif
		return true;
	}
	
	return false;
}

bool UMotionCaptureNetwork::Connect(const FString& IP, int32 Port)
{
	if (!UDPSocket.IsValid())
	{
		TSharedPtr<class FInternetAddr> HostAddr = ISocketSubsystem::Get()->GetLocalBindAddr(*GLog);
		if (!HostAddr.IsValid())
		{
			PN_LOG(Error, TEXT("Get local bind addr failed."));
			return false;
		}

		uint32 HostIp = 0;
		HostAddr->GetIp(HostIp);
		FUdpSocketBuilder Builder = FUdpSocketBuilder(TEXT("Motion Capture UDP Network")).
			BoundToAddress(FIPv4Address(HostIp)).
			BoundToPort(Port);

		UDPSocket = MakeShareable(Builder.Build());
	}

	if (UDPSocket.IsValid())
	{
		CurAnimData = MakeShareable(new FAnimationData());

		PacketParser = MakeShareable(new FPacketParser());

		UDPReceiver = MakeShareable(new FUdpSocketReceiver(UDPSocket.Get(), FTimespan(0, 0, 1), TEXT("Motion Capture UDP receiver")));
		UDPReceiver->OnDataReceived().BindUObject(this, &UMotionCaptureNetwork::UdpReceivedCallback);
//#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 11)
		UDPReceiver->Start();
//#endif
		return true;
	}

	return false;
}
	

void UMotionCaptureNetwork::DisConnect()
{
	if (UDPReceiver.IsValid())
	{
		UDPReceiver->Exit();
		UDPReceiver.Reset();
	}

	if (UDPSocket.IsValid())
	{
		UDPSocket->Close();
		UDPSocket.Reset();
	}

	PacketParser.Reset();
}

void UMotionCaptureNetwork::UdpReceivedCallback(const FArrayReaderPtr& Data, const FIPv4Endpoint& IP)
{
#ifndef TRANCE
	AMotionCaptureManager* PNManager = Cast<AMotionCaptureManager>(GetOuter());
	check(PNManager);
#endif

	int32 AvatarID = INDEX_NONE;
	FName AvatarName = NAME_None;
	if (PacketParser.IsValid() && PacketParser->Read(Data, CurAnimData, AvatarID, AvatarName))
	{
		bool bFound = FAnimationDataManager::GetInstance()->IsAnimationDataExist(
#if WITH_AVATARID
			AvatarID
#else
			AvatarName
#endif
			);
		if (!bFound)
		{
			if (FAnimationDataManager::GetInstance()->AddAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurAnimData
				))
			{
#ifndef TRANCE
				// Create new actor in game thread.
				FEventManager::GetInstance()->EnqueueEvent(MakeShareable(new FCreateNewEvent(PNManager, 
					AvatarName, AvatarID, FVector::ZeroVector, FQuat::Identity)));
#endif
			}
			else
			{
				PN_LOG(Error, TEXT("Add new animation data failed."));
			}
		}
		else
		{
			FAnimationDataManager::GetInstance()->UpdateAnimationData(
#if WITH_AVATARID
				AvatarID
#else
				AvatarName
#endif
				, CurAnimData);
		}
	}
	else
	{
		PN_LOG(Error, TEXT("Parse received data failed. Please check the server version."));
	}
}
