// Copyright 2015 Noitom Technology Co., Ltd. All Rights Reserved.

#pragma once
 
#include "CoreMinimal.h"
#include "MotionCaptureSpawner.generated.h"


/**
 * The dynamically created Avatar`s spawn point.
 */
UCLASS(ClassGroup = MotionCapture, MinimalAPI)
class AMotionCaptureSpawner : public AActor
{
	GENERATED_BODY()

#if WITH_EDITORONLY_DATA
public:
	/** Returns SpriteComponent subobject **/
	class UBillboardComponent* GetSpriteComponent() const;

	/** Returns TextRenderComponent subobject **/
	class UTextRenderComponent* GetTextRenderComponent() const;

private:
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Display, meta = (AllowPrivateAccess = "true"))
	class UBillboardComponent* SpriteComponent;

	UPROPERTY()
	class UTextRenderComponent* TextRenderComponent;
#endif

public:
	AMotionCaptureSpawner(const FObjectInitializer& ObjectInitializer);

	virtual ~AMotionCaptureSpawner();

public:
	/** The ID of the avatar that will be spawned at the location. */
	UPROPERTY(VisibleInstanceOnly, Category = "Actor Binding", meta = (DisplayName = "Avatar ID"))
	int32 AvatarID = INDEX_NONE;

	/** The name of the avatar that will be spawned at the location. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Actor Binding", meta = (DisplayName = "Avatar Name"))
	FName AvatarName = NAME_None;

	/** The actual class of the avatar that will be spawned at the location. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Actor Binding", meta = (DisplayName = "Dynamic Actor Class"))
	TSubclassOf<AActor> DynamicActorClass;

private:
	AMotionCaptureManager* PNManager = nullptr;

	friend class AMotionCaptureManager;
};
