// Fill out your copyright notice in the Description page of Project Settings.

#include "NetworkManager.h"
#include "MotionCaptureNetwork.h"
#include "LocalDataReader.h"
#include "HI5Glove.h"

#if USE_ALICE
#include "../../../../HybridData/Source/HybridData/Public/IHybridData.h"
#endif

NetworkManager* NetworkManager::Instance = nullptr;

NetworkManager::NetworkManager()
{
}

NetworkManager::~NetworkManager()
{
	Disconnect();
}

NetworkManager* NetworkManager::GetInstance()
{
	if (Instance == nullptr)
	{
		Instance = new NetworkManager();
	}

	return Instance;
}

void NetworkManager::InitLocalDataReader()
{
	if (LocalDataReader == nullptr)
	{
		LocalDataReader = NewObject<ULocalDataReader>();
		LocalDataReader->AddToRoot();
	}
}

bool NetworkManager::Connect(const FString& IP/* ="0.0.0.0" */, int32 Port/* =7012 */)
{
	if (Network != nullptr)
	{
		Network->DisConnect();
	}
	else
	{
		Network = NewObject<UMotionCaptureNetwork>();
		Network->AddToRoot();
	}

	// Setup PNNetwork
	if (nullptr != Network)
	{
		if (!Network->Connect(IP,Port))
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	return false;
}

void NetworkManager::Disconnect()
{
	if (Network != nullptr)
	{
		Network->DisConnect();
		Network = nullptr;
	}
}

void NetworkManager::OnLocalPlayerBVHDataReceived(const FArrayReaderPtr& Data)
{
	if (LocalDataReader!= nullptr)
	{
		LocalDataReader->OnLocalPlayerBVHDataReceived(Data);
	}
}

void NetworkManager::OnLocalPlayerBVHDataReceived(const uint8* pData,int DataLen)
{
	FArrayReaderPtr Datagram = MakeShareable(new FArrayReader(true));
	Datagram->SetNumUninitialized(DataLen);
	Datagram->Insert(pData, DataLen, 0);
	OnLocalPlayerBVHDataReceived(Datagram);
}

///< remote均认为是alice数据
void NetworkManager::OnRemotePlayerBVHDataReceived(const FArrayReaderPtr& Data,const FName& AvatarName)
{
	if (LocalDataReader!=nullptr)
	{
#if USE_ALICE
		if (IHybridData::GetPtr ()) {
			bool isLeftGlove;
			FString RealAvatarName, DeviceId, RigidName;
			AliceIns ().GetGloveInfoByID (AvatarName.ToString (), RealAvatarName, DeviceId, RigidName, isLeftGlove);
			LocalDataReader->OnRemotePlayerBVHDataReceived (Data, FName (*RealAvatarName));
		}
#else
		LocalDataReader->OnRemotePlayerBVHDataReceived (Data, AvatarName);
#endif // USE_ALICE
	}
}

int32 NetworkManager::GetGloveMode()
{
	if (LocalDataReader!=NULL)
	{
		return LocalDataReader->GetGloveMode();
	}

	return -1;
}
