#pragma once

#include "CoreMinimal.h"
#include "AnimationData.h"

namespace Hi5
{
	class FPoseContainer
	{
	public:
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
		FPoseContainer( FPoseContext& Output ) : InternalPose( Output.Pose )
		{
			BoneCount = InternalPose.GetNumBones( );
		}
#else
		FPoseContainer( FPoseContext& Output ) : PoseRef( Output.Pose )
		{
			InternalPose.AllocateLocalPoses( Output.AnimInstance->RequiredBones, Output.Pose );
			BoneCount = InternalPose.Bones.Num( );
		}
#endif

		~FPoseContainer( )
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
#else
			InternalPose.ConvertToLocalPoses( PoseRef );
#endif
		}

		int32 GetNumBones( ) const
		{
			return BoneCount;
		}

		FVector GetLocation( int32 Index ) const
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			return InternalPose[FCompactPoseBoneIndex( Index )].GetLocation( );
#else
			return InternalPose.Bones[Index].GetLocation( );
#endif
		}

		FQuat GetRotation( int32 Index ) const
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			return InternalPose[FCompactPoseBoneIndex( Index )].GetRotation( );
#else
			return InternalPose.Bones[Index].GetRotation( );
#endif
		}

		void SetLocation( int32 Index, const FVector& Pos )
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			InternalPose[FCompactPoseBoneIndex( Index )].SetLocation( Pos );
#else
			InternalPose.Bones[Index].SetLocation( Pos );
#endif
		}

		void SetRotation( int32 Index, const FQuat& Quat )
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			InternalPose[FCompactPoseBoneIndex( Index )].SetRotation( Quat );
#else
			InternalPose.Bones[Index].SetRotation( Quat );
#endif
		}

		int32 GetParentBoneIndex( int32 Index )
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			return InternalPose.GetParentBoneIndex( FCompactPoseBoneIndex( Index ) ).GetInt( );
#else
			return InternalPose.GetParentBoneIndex( Index );
#endif
		}

		FTransform& operator[]( int32 Index )
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			return InternalPose[FCompactPoseBoneIndex( Index )];
#else
			return InternalPose.Bones[Index];
#endif
		}

		const FTransform& operator[]( int32 Index ) const
		{
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
			return InternalPose[FCompactPoseBoneIndex( Index )];
#else
			return InternalPose.Bones[Index];
#endif
		}

	private:
#if (ENGINE_MAJOR_VERSION >= 4) && (ENGINE_MINOR_VERSION >= 9)
		FCompactPose& InternalPose;
#else
		FA2Pose& PoseRef;
		FA2CSPose InternalPose;
#endif

		void* operator new(std::size_t) = delete;

		int32 BoneCount;
	};


	class FAnimDataContainer
	{
	public:
		FAnimDataContainer( ) = default;

		FAnimDataContainer& operator=( FAnimationData&& InData )
		{
			Data = MoveTemp( InData );
			bUseData = true;
			return *this;
		}

		FAnimDataContainer& operator=( const FAnimationDataPtr& InDataPtr )
		{
			DataPtr = InDataPtr;
			return *this;
		}

		const FAnimationData* operator->( ) const
		{
			return bUseData ? &Data : DataPtr.operator->( );
		}

		bool IsValid( ) const
		{
			return bUseData ? true : DataPtr.IsValid( );
		}

	private:
		bool bUseData = false;
		FAnimationData Data;
		FAnimationDataPtr DataPtr;
	};
}
