// Fill out your copyright notice in the Description page of Project Settings.

#include "UnrealXML.h"

namespace MotionCaptureXML
{

	UnrealXML::UnrealXML ()
	{
	}

	UnrealXML::~UnrealXML ()
	{
		if (nullptr != spXmlFile)
		{
			spXmlFile->Clear ();
			delete spXmlFile;
			spXmlFile = nullptr;
			spRootNode = nullptr;
		}
	}

	void * UnrealXML::LoadXML (const FString& FilePath)
	{
		if (nullptr != spXmlFile)
		{
			spXmlFile->Clear ();
			delete spXmlFile;
			spXmlFile = nullptr;
			spRootNode = nullptr;
		}

		spXmlFile = new FXmlFile (FilePath);
		if (nullptr != spXmlFile)
		{
			spRootNode = spXmlFile->GetRootNode ();
			spOperateNode = spRootNode;
		}

		UE_LOG (LogTemp, Warning, TEXT ("FilePath = %s"), *FilePath);

		return (void *)spRootNode;
	}

	bool UnrealXML::SaveXML (const FString& FilePath)
	{
		if (nullptr == spXmlFile)
		{
			return false;
		}

		bool ret = spXmlFile->Save (FilePath);
		//ret = spXmlFile->Save("testFind.xml");
		return ret;
	}

	FString UnrealXML::GetContentByTag (const FString& NodeTag, void * parentNode)
	{
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return "";

		FXmlNode* pNode = pParentNode->FindChildNode (NodeTag);
		if (nullptr != pNode)
		{
			return pNode->GetContent ();
		}

		return "";
	}

	int UnrealXML::GetContentNumByTag (const FString& NodeTag, void * parentNode)
	{
		FString content = GetContentByTag (NodeTag, parentNode);
		if (content.IsNumeric ())
		{
			return FCString::Atoi (*content);
		}
		return 0;
	}

	bool UnrealXML::GetContentBoolByTag (const FString& NodeTag, void * parentNode)
	{
		FString content = GetContentByTag (NodeTag, parentNode);
		content = content.ToUpper ();
		if (content == "0" || content == "FALSE")
		{
			return false;
		}
		return true;
	}

	FString UnrealXML::GetAttributeByTag (const FString& NodeTag, const FString& AttrName, void * parentNode)
	{
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return "";

		FXmlNode* pNode = pParentNode->FindChildNode (NodeTag);
		if (nullptr != pNode)
		{
			return pNode->GetAttribute (AttrName);
		}

		return "";
	}

	FString UnrealXML::SetContentByTag (const FString& NodeTag, const FString& newContent, void * parentNode)
	{
		FString outContent;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return "";

		FXmlNode* pNode = pParentNode->FindChildNode (NodeTag);
		if (nullptr != pNode)
		{
			outContent = pNode->GetContent ();
			pNode->SetContent (newContent);
		}
		else
		{
			pParentNode->AppendChildNode (NodeTag, newContent);
		}
		return outContent;
	}

	FString UnrealXML::SetAttributeByTag (const FString& NodeTag, const FString& AttrName, const FString &AttrValue, void * parentNode)
	{
		FString outValue;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return "";

		FXmlNode* pNode = pParentNode->FindChildNode (NodeTag);
		bool FindValue = false;
		int i = 0;
		TArray<FXmlAttribute>* attrArray = (TArray<FXmlAttribute>*)&pParentNode->GetAttributes ();
		if (nullptr != pNode)
		{
			for (auto Iter (attrArray->CreateConstIterator ()); Iter; Iter++)
			{
				if (Iter->GetTag () == AttrName)
				{
					outValue = Iter->GetValue ();
					FXmlAttribute newAttr1 (AttrName, AttrValue);
					(*attrArray)[i] = newAttr1;
					FindValue = true;
				}
				i++;
			}
		}
		else
		{
			pParentNode->AppendChildNode (NodeTag, "");
		}
		if (!FindValue)
		{
			FXmlAttribute newAttr (AttrName, AttrValue);
			attrArray->Add (newAttr);
		}
		return outValue;
	}

	FString UnrealXML::GetContent (void * operateNode)
	{
		FXmlNode* pOperateNode = (FXmlNode*)operateNode;
		if (nullptr == pOperateNode)
		{
			pOperateNode = spOperateNode;
		}
		if (pOperateNode == NULL)
			return "";

		FXmlNode* pNode = pOperateNode;
		if (nullptr != pNode)
		{
			return pNode->GetContent ();
		}

		return "";
	}

	void * UnrealXML::UseXMLString (const FString& xml_string)
	{
		spXmlFile = new FXmlFile (xml_string, EConstructMethod::ConstructFromBuffer);
		if (nullptr != spXmlFile)
		{
			spRootNode = spXmlFile->GetRootNode ();
			spOperateNode = spRootNode;
			return (void*)spOperateNode;
		}

		return nullptr;
	}

	FString UnrealXML::GetName (void * operateNode)
	{
		FXmlNode* pOperateNode = (FXmlNode*)operateNode;
		if (nullptr == pOperateNode)
		{
			pOperateNode = spOperateNode;
		}
		if (pOperateNode == NULL)
			return "";

		FXmlNode* pNode = pOperateNode;
		if (nullptr != pNode)
		{
			return pNode->GetTag ();
		}

		return "";
	}

	FString UnrealXML::GetAttribute (const FString& AttrName, void * operateNode)
	{
		FXmlNode* pOperateNode = (FXmlNode*)operateNode;
		if (nullptr == pOperateNode)
		{
			pOperateNode = spOperateNode;
		}
		if (pOperateNode == NULL)
			return "";

		FXmlNode* pNode = pOperateNode;
		if (nullptr != pNode)
		{
			return pNode->GetAttribute (AttrName);
		}

		return "";
	}

	FString UnrealXML::SetContent (const FString& newContent, void * operateNode)
	{
		FString outContent;
		FXmlNode* pOperateNode = (FXmlNode*)operateNode;
		if (nullptr == pOperateNode)
		{
			pOperateNode = spOperateNode;
		}
		if (pOperateNode == NULL)
			return "";

		FXmlNode* pNode = pOperateNode;
		if (nullptr != pNode)
		{
			outContent = pNode->GetContent ();
			pNode->SetContent (newContent);
		}

		return outContent;
	}

	FString UnrealXML::SetAttribute (const FString& AttrName, const FString &AttrValue, void * operateNode)
	{
		FString outValue;
		FXmlNode* pOperateNode = (FXmlNode*)operateNode;
		if (nullptr == pOperateNode)
		{
			pOperateNode = spOperateNode;
		}
		if (pOperateNode == NULL)
			return "";

		FXmlNode* pNode = pOperateNode;
		bool FindValue = false;
		int i = 0;
		TArray<FXmlAttribute> *attrArray = (TArray<FXmlAttribute> *)&(pNode->GetAttributes ());
		for (auto Iter (attrArray->CreateConstIterator ()); Iter; Iter++)
		{
			if (Iter->GetTag () == AttrName)
			{
				outValue = Iter->GetValue ();
				FXmlAttribute newAttr1 (AttrName, AttrValue);
				(*attrArray)[i] = newAttr1;
				FindValue = true;
			}
			i++;
		}

		if (!FindValue)
		{
			FXmlAttribute newAttr (AttrName, AttrValue);
			attrArray->Add (newAttr);
		}
		//TArray<FXmlAttribute> attrArray1 = spOperateNode->GetAttributes();
		return outValue;
	}

	void * UnrealXML::SetOperateNodeByTag (const FString& NodeTag, int startindex, void * parentNode)
	{
		FString outValue;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return 0;
		TArray<FXmlNode*> Children = pParentNode->GetChildrenNodes ();
		int32 ChildCount = Children.Num ();
		for (int32 ChildIndex = startindex; ChildIndex < ChildCount; ++ChildIndex)
		{
			if (Children[ChildIndex] != nullptr && Children[ChildIndex]->GetTag () == NodeTag)
			{
				spOperateNode = Children[ChildIndex];
				return (void *)spOperateNode;
			}
		}
		pParentNode->AppendChildNode (NodeTag, "");
		Children = pParentNode->GetChildrenNodes ();
		ChildCount = Children.Num ();
		if (ChildCount > 0)
		{
			if (Children[ChildCount - 1] != nullptr && Children[ChildCount - 1]->GetTag () == NodeTag)
			{
				spOperateNode = Children[ChildCount - 1];
				return (void *)spOperateNode;
			}
		}

		return NULL;
	}

	void * UnrealXML::SetOperateNodeByIndex (int index, void * parentNode)
	{
		FString outValue;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return 0;
		TArray<FXmlNode*> Children = pParentNode->GetChildrenNodes ();
		const int32 ChildCount = Children.Num ();
		if (index >= ChildCount)
		{
			return NULL;
		}
		if (Children[index])
		{
			spOperateNode = Children[index];
			return (void *)spOperateNode;
		}
		return NULL;
	}

	bool UnrealXML::ResetOperateNodeToRoot (void * rootxml)
	{
		FXmlNode* pRootNode = (FXmlNode*)rootxml;
		if (pRootNode == NULL)
		{
			pRootNode = spRootNode;
		}
		spOperateNode = pRootNode;
		if (spOperateNode)
			return true;
		return false;
	}

	void * UnrealXML::SetOperateNode (void * newnode)
	{
		void * outval = (void *)spOperateNode;
		spOperateNode = (FXmlNode*)newnode;
		return outval;
	}

	int UnrealXML::GetChildCount (void * parentNode)
	{
		FString outValue;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return -1;
		TArray<FXmlNode*> Children = pParentNode->GetChildrenNodes ();
		int ChildCount = Children.Num ();
		return ChildCount;
	}

	void * UnrealXML::GetChild (void * parentNode, int index)
	{
		FString outValue;
		FXmlNode* pParentNode = (FXmlNode*)parentNode;
		if (nullptr == pParentNode)
		{
			pParentNode = spOperateNode;
		}
		if (pParentNode == NULL)
			return 0;
		TArray<FXmlNode*> Children = pParentNode->GetChildrenNodes ();
		const int32 ChildCount = Children.Num ();

		spOperateNode = Children[index];
		return (void *)spOperateNode;
	}

	int UnrealXML::SplitDir (FString oldpath, FString * &outpath)
	{
		FString pString[1000];
		FString foldpath1 = oldpath;
		FString oldstring = "\\";
		FString placestring = "/";
		FString foldpath = foldpath1.Replace (*oldstring, *placestring);
		FString fleft, fright;
		int offset = 0;
		bool ret = foldpath.Split ("/", &fleft, &fright);
		while (ret)
		{
			pString[offset] = fleft;
			offset++;
			foldpath = fright;
			ret = foldpath.Split ("/", &fleft, &fright);
		}
		pString[offset] = foldpath;
		offset++;
		outpath = new FString[offset];
		int i;
		for (i = 0; i < offset; i++)
		{
			outpath[i] = pString[i];
		}
		return offset;
	}

	bool UnrealXML::CompressDir (FString *path, FString *&outpath, int& max)
	{
		int i, j;
		if (path == NULL || max < 1)
			return false;
		FString temppath[1000];
		for (i = 0; i < max; i++)
		{
			temppath[i] = path[i];
		}
		bool findit = true;
		while (findit)
		{
			findit = false;
			for (i = 0; i < max - 1; i++)
			{
				// if find the current directory string is not .. and the net directory string is ..,then the two directory relative diretory is null. 
				if (temppath[i] != ".."&&temppath[i + 1] == "..")
				{
					for (j = i; j < max - 2; j++)
					{
						temppath[j] = temppath[j + 2];
					}
					max -= 2;
					findit = true;
					break;
				}
			}
		}
		if (outpath)
		{
			delete[] outpath;
		}
		outpath = new FString[max];
		for (i = 0; i < max; i++)
		{
			outpath[i] = temppath[i];
		}
		return true;
	}

	bool UnrealXML::GetRealPath (const FString oldpath, FString& realpath)
	{
		bool ret = true;
		FString * outpath = NULL;

		FString fsoldpath = oldpath;

		int dircount = SplitDir (fsoldpath, outpath);

		//MessageBox(NULL, (XMLCHARPTR )*outpath[0], NULL, 0);
		FString * outpath1 = NULL;
		CompressDir (outpath, outpath1, dircount);

		//MessageBox(NULL, (XMLCHARPTR )*(*outpath1), NULL, 0);
		FString outrealpath;
		if (dircount > 0)
		{
			int i;
			for (i = 0; i < dircount; i++)
			{
				if (outpath1[i].Len () > 0)
				{
					outrealpath = outrealpath + outpath1[i] + "/";
				}
			}
		}
		realpath = outrealpath;
		return ret;
	}
}