// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "XMlFile.h"

class FXmlFile;
class FXmlNode;

namespace MotionCaptureXML
{

	class UnrealXML
	{
	public:
		UnrealXML ();
		~UnrealXML ();

		// Open a xml file and return the void* file hand.
		void * LoadXML (const FString& FilePath);

		// Use the default xml structure constructed by FXmlFile
		void * UseXMLString (const FString& xml_string);

		// Save the recent opened xml file to the file which filename is the FilePath Param. 
		bool SaveXML (const FString& FilePath);

		// Get the xml element content by the element name
		FString GetContentByTag (const FString& NodeTag, void * parentNode = 0);

		// Get the xml element content and convert it to number by the element name
		int GetContentNumByTag (const FString& NodeTag, void * parentNode = 0);

		// Get the xml element content and convert it to boolean by the element name
		bool GetContentBoolByTag (const FString& NodeTag, void * parentNode = 0);

		// Get the xml element attribute by the element name and attribute name
		FString GetAttributeByTag (const FString& NodeTag, const FString& AttrName, void * parentNode = 0);

		// Set the xml element content by the element name
		FString SetContentByTag (const FString& NodeTag, const FString& newContent, void * parentNode = 0);

		// Set the xml element attribute by the element name and attribute name
		FString SetAttributeByTag (const FString& NodeTag, const FString& AttrName, const FString &AttrValue, void * parentNode = 0);

		// Get the current operate xml element content
		FString GetContent (void * operateNode = 0);

		// Get the current operate xml element name
		FString GetName (void * operateNode = 0);

		// Get the current operate xml element attribute by the attribute name
		FString GetAttribute (const FString& AttrName, void * operateNode = 0);

		// Set the current operate xml element content
		FString SetContent (const FString& newContent, void * operateNode = 0);

		// Set the current operate xml element attribute by the attribute name
		FString SetAttribute (const FString& AttrName, const FString &AttrValue, void * operateNode = 0);

		// Set the current operate xml element by the element name which is child element for parentNode element
		void * SetOperateNodeByTag (const FString& NodeTag, int startindex, void * parentNode = 0);

		// Set the current operate xml element by the child number which is child element for parentNode element
		void * SetOperateNodeByIndex (int index, void * parentNode = 0);

		// Set the current operate xml element to the file root element
		bool ResetOperateNodeToRoot (void * rootxml = 0);

		// Set the current operate xml element to newnode element
		void * SetOperateNode (void * newnode);

		// Get the count of the current operate xml element chilren elements
		int GetChildCount (void * parentNode);

		// Get the child element of the parentNode element by the child number
		void * GetChild (void * parentNode, int index);

		// Get the current operate xml element node handle
		void * GetOperateNode ()
		{
			return (void *)spOperateNode;
		}

		// Convert the directory string to be more simplified,such as "/dir2/dir1/../dir3" can be simplify to "/dir2/dir3"
		bool GetRealPath (const FString oldpath, FString& realpath);
	private:
		// Split the directory string with the "/" or "\" to a string array
		int SplitDir (FString oldpath, FString * &outpath);
		// Combine the directory string array to directory
		bool CompressDir (FString *path, FString *&outpath, int& max);
		FXmlFile* spXmlFile = NULL;
		FXmlNode* spRootNode = NULL;
		FXmlNode* spOperateNode = NULL;
		bool mbLoadAliceConfig = false;
	};

}
