#include "prop.h"

prop::prop( const char* filename )
{
	state = false;
	name = filename;
	parse();
}

prop::~prop()
{
}

const char* prop::getstring( const char* _key, const char* default_val/*=""*/ ) const
{
	if (!isKeyExists(_key))
		return default_val;
	std::map<std::string, std::string>::const_iterator it = propmap.find(_key);
	if (it == propmap.end())
		return default_val;
	return it->second.c_str();
}

int prop::getint( const char* _key, int default_val /*= 0*/ ) const
{
	if (!isKeyExists(_key))
		return default_val;
	const char* s = getstring(_key);
	return s ? atoi(s) : 0;
}

double prop::getdouble( const char* _key, double default_val /*= 0.0*/ ) const
{
	if (!isKeyExists(_key))
		return default_val;
	const char* s = getstring(_key);
	return s ? atof(s) : 0;
}

void prop::parse()
{
	FILE* pfile;
	fopen_s(&pfile, name.c_str(), "r");
	if (!pfile)
		return;

	while (fgets(line, 1023, pfile))
		parseline(line);
	fclose(pfile);
	state = true;
}

void prop::parseline(char* _line)
{
	trimright(_line);
	while (1)
	{
		switch (*_line)
		{
		case ' ':
		case '\t':
		case '\r':
		case '\n':
			++_line;
			continue;
		case '#':
		case ';':
		case 0:
			return;
		}
		char* pos = strchr(_line, '=');
		strncpy_s(key, 256, _line, pos - _line);
		key[pos-_line] = 0;
		trimright(key);
		propmap[key] = trimleft(pos+1);
		return;
	}
}

char* prop::trimleft( char* str )
{
	while (1)
	{
		switch (*str)
		{
		case ' ':
		case '\t':
		case '\r':
		case '\n':
			++str;
			continue;
		default:
			return str;
		}
	}
}

void prop::trimright( char* str )
{
	int len = strlen(str);
	while (len)
	{
		switch (str[len-1])
		{
		case ' ':
		case '\t':
		case '\r':
		case '\n':
			--len;
			continue;
		default:
			str[len] = 0;
			return;
		}
	}
}

bool prop::isKeyExists( const char* _key ) const
{
	std::map<std::string, std::string>::const_iterator it = propmap.find(_key);
	return (it != propmap.end());
}

