// Fill out your copyright notice in the Description page of Project Settings.

#pragma once
#include <atomic>
#include <ctime>
#include "HI5_MagneticStatus.h"
#include "CoreMinimal.h"
#include "Hi5_BVHDataQuene.h"

enum EPowerLevel
{
    EPowerLevelUnknown = -1,
    EPowerLevelFull = 0,
    EPowerLevelGood,
    EPowerLevelNormal,
    EPowerLevelLow,
    EPowerLevelNone,
};


class MOTIONCAPTURERUNTIME_API FHI5_GloveStatus : public std::enable_shared_from_this<FHI5_GloveStatus>
{
public:
	FHI5_GloveStatus();
	~FHI5_GloveStatus();
    EPowerLevel GetLeftPower();
    void SetLeftPower(EPowerLevel inValue);
    EPowerLevel GetRightPower();
    void SetRightPower(EPowerLevel inValue);
    EMagneticStatus GetLeftMag();
    void SetLeftMag(EMagneticStatus inValue);
    EMagneticStatus GetRightMag();
    void SetRightMag(EMagneticStatus inValue);
    EMagneticStatus GetLeftMagTemp();
    void SetLeftMagTemp(EMagneticStatus inValue);
    EMagneticStatus GetRightMagTemp();
    void SetRightMagTemp(EMagneticStatus inValue);
    bool GetIsLeftGloveAvailable();
    void SetIsLeftGloveAvailable(bool isValue);
    bool GetIsRightGloveAvailable();
    void SetIsRightGloveAvailable(bool isValue);
    void Update(float DetaTime);
    EGloveStatus Status();
    void HI5_GloveStatusOnEventChanged(GloveMod gloveMod, GloveEvent gloveEvent);
    void MainThreadUpdate();
    void UpdateGloveVisible(EGloveStatus  status);
    void SetHi5_StatueData(std::shared_ptr<FHi5_status> inValue);
    void ReleaseTimer();
    EGloveStatus m_currentStatus;
private:
    void StartLeftTimer();
    void StartRightTimer();
    void EndleftTimer();
    void EndRightTimer();
    void DelayTime();
private:
    std::atomic<EPowerLevel> m_leftPL;
    std::atomic<EPowerLevel> m_RightPL;
    std::atomic<EMagneticStatus> m_leftMag;
    std::atomic<EMagneticStatus> m_rightMag;
    std::atomic<EMagneticStatus> m_leftMagTemp;
    std::atomic<EMagneticStatus> m_rightMagTemp;
    std::atomic<bool> m_isLeftGloveAvailable;
    std::atomic<bool> m_isRightGloveAvailable;
    bool m_StartleftTimer;
    clock_t m_leftTimer;  
    bool m_StartrightTimer;
    clock_t m_rightTimer; 
    static const float delayCd;
    std::shared_ptr<FHi5_status> m_Hi5_StatueData; 
};
