// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Hi5_Thread_Data.h"
#include "HI5_MagneticStatus.h"
#include "HI5_GloveStatus.h"
#include "Hi5_BVHDataQuene.h"
#include "HI5Glove/HI5Glove_C.h"
#include "Hi5Source.h"
#include <iostream>
#include "mutex"
#include "SteamVRFunctionLibrary.h"
#include "MotionCaptureFunctionLibrary.h"
#include "Hi5DataTest.h"
using namespace HI5Device;
class MOTIONCAPTURERUNTIME_API FHi5_Manager_Thread
{
public:

	
	FHi5_Manager_Thread();
	virtual ~FHi5_Manager_Thread();
	void Update(float detaTime);
	bool Isconneted() const;
	void SetIsConnected(bool inValue);
	void AddCalibrationCommand(FHI5_Calibration_Command::ECalibration_Command inValue);
	void AddOpticData(std::string sensorIdentity, OpticalSensorType type, float fpos[3], float frot[4]);
	void AddOptiSensorBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType);
	void GamePause(bool isPause);
	void RequestConnect(bool inertia);
	void RequestCloseConnect();
	void StartThread();
	void StopThread();
	void Reconnected();
	static void InitManagerThread();
	static FHi5_Manager_Thread* getInstance();
	static void DestorManagerThready();
	int GetCalibrationpercent() const;
	std::shared_ptr<FHI5_MagneticStatus> GetMagneticStatus() const
	{
		return m_magneticStatus;
	}
	std::shared_ptr<FHI5_GloveStatus>  GloveStatus() const
	{
		return m_HI5_GloveStatus;
	}
	void GetSensorInfor( int& index,FSenerInforBluePr& information);
	int GetSendsorInfor(int index);
	std::shared_ptr<FSensorInforArray> GetSensorInforArray() const
	{
		return m_FSensorInforArray;
	}
	void AsynUpdateThread();
	void SetInertia(bool inParam)
	{
		isInertia = inParam;
	}
	bool GetInertia() const
	{
		return isInertia;
	}
	void SetEnableFingerAdbFixed(bool inParam);

	
	FVive_Tracker_Infor_Map& GetVive_Tracker_Infor_Map()
	{
		return  mFViveTrackerInfor;
	}
	void SetCalibrationState(ECalibrationState CalibrationState);
	ECalibrationState GetCalibrationState()
	{
		return _CalibrationState;
	}
private:
	void Init();
	void SetHandThickness(float param);
	void RegisterEvent();
	void SynConnectEquipment(bool inertia);
	void SynCloseConnectEquipment();
	void AsynOperateCalibrationThread(std::shared_ptr<FHI5_Calibration_Command> inParam);
	int AsyGetClibrationPercent();
	void AsynSendOpticData( std::string sensorIdentity,
								  OpticalSensorType type,
								  float fpos[3],
								  float frot[4]);
	void AsynSendOpticBindData(GloveMod mod, std::string trackerID, OpticalSensorType sensorType);
	void AsynReceivThreadData();
	void RegisterGloveStateChangedCallback();
	void RegisterDongleStateChangedHandle();
	bool SaveCalibrationFile(const FString& fileName /*= TEXT("BCP.bin")*/);
	bool LoadCalibrationFileTemp(const FString& fileName /*= TEXT("BCP.bin")*/);
	EGloveStatus ConvertHI5GloveStatus(HI5GloveStatus status);

	bool LoadOptiDeviceBindStateFromFile(const FString& fileName);

	void TestOptiLog();
	void TestLeftBvhLog();
	void TestRightBvhLog();
	void BindHandInfor();
public:
	static float leftTrackerInitOffset[3];
	static float rightTrackerInitOffset[3];
private:
	std::shared_ptr<FHI5_Calibration_Send_Data> m_Commands;
	std::thread  m_phread;
	std::shared_ptr<FHI5_MagneticStatus> m_magneticStatus;
	std::shared_ptr<FHI5_GloveStatus> m_HI5_GloveStatus;
	std::shared_ptr<FHi5_status> m_FHi5_status;
	std::shared_ptr<FHi5_BVHDataQuene> m_FHi5_BVHDataQuene;
	//std::shared_ptr<FHI5_Optic_DataArray>  
	ECalibrationState _CalibrationState;
	std::atomic<bool> m_isStop;
	bool IsStartDongle;
	bool IsCalibrationComplete;
	bool IsLoadData;
	bool isInertia;
	static FHi5_Manager_Thread* m_instance;
	static std::once_flag m_Flg;
	static FVector leftOffset;
	static FVector rightOffset;

	GloveBVHData*  m_gloveData;
	std::atomic<bool> m_Isconneted;
	std::shared_ptr<FHi5Source> m_HI5Source;
	std::shared_ptr<FSensorInforArray> m_FSensorInforArray;
	std::atomic<int>  m_Calibrationpercent;
	bool isEnableFingerAdbFixed;
	GloveBVHData* mleftData;
	GloveBVHData* mrightData;
	FSenserItemData SenserDatas[12];
	FVive_Tracker_Infor_Map mFViveTrackerInfor;
	bool isRequestBindhand;
	
public:
	static std::mutex   m_mutexClone;
	FHi5DataTest mHi5DataTest;
};

